{-$D}
{*************************************************************************}
{                                                                         }
{                        HlcCapture Component                             }
{                                                                         }
{  This component lets you capture the screen, or any window, object or   }
{  area (screenshot). You can provide a keycode (for example PrintScreen  }
{  or F11) or a delay (in ms) to start the capture. Works just like       }
{  PaintShopPro's capture feature.                                        }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{  Copyright (c) 1997, 1999 High-Logic [Erwin Denissen]                   }
{                                                                         }
{  You may use a Shareware Copy of this Software for an evaluation period }
{  of up to 30 days, in order to determine whether the Software meets     }
{  your needs before purchasing it. Once the evaluation period ends, you  }
{  agree to either purchase a copy of the software, or to stop using it   }
{  and remove it from your computer system.                               }
{                                                                         }
{  A registered version of this and other components and software can be  }
{  ordered at the following site: http://www.high-logic.com/index.html    }
{                                                                         }
{  Make sure you are using the latest version of this software.           }
{  Send bug reports to erwind@high-logic.com                              }
{                                                                         }
{  DISCLAIMER                                                             }
{  In no event will the author be held liable for any direct or indirect  }
{  damage resulting from the use of this software or any derivatives      }
{  thereof.                                                               }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{ Version 1.2 14-01-2000                                                  }
{ - Fixed Cursor position.                                                }
{ - New attemp to restore cursor after area capture.                      }
{   I think it is still not working fine.                                 }
{ Version 1.1 17-01-1999                                                  }
{ - Added CaptureArea method.                                             }
{ Version 1.0 21-12-1998                                                  }
{ - First release, works with Delphi 2, 3 and 4 and with C++Builder 3     }
{                                                                         }
{*************************************************************************}

unit HlcCapture;

interface

uses
  Windows, SysUtils, Classes, Graphics, hlcCaptureThread, HlcRegisterAbout;

const
  RegisterText1 = 'Unregistered ThlcCapture component';
  RegisterText2 = 'http://www.high-logic.com/';
  RegisterText3 = '(c) 1998, 1999 The Netherlands';

type

  TCaptureStyle = (
    csScreen,
    csWindow,
    csActiveWindow,
    csClientArea,
    csArea,
    csObject
    );

  TOnCaptureProcedure = procedure(Sender: TObject; bmpCapture :TBitmap) of object;
  TOnBeforeCaptureProcedure = procedure(Sender: TObject; hwndSource: integer; var Continue : boolean) of object;
  TOnCaptureErrorProcedure = procedure(Sender: TObject; ErrorMessage: string) of object;

  ThlcCapture = class(TComponent)
  private
    { Private declarations }
    FAbout : ThlcAbout;
    FCaptureStyle : TCaptureStyle;
    FIncludeCursor : boolean;
    FCaptureArea : TRect;
    FOnBeforeCaptureProcedure : TOnBeforeCaptureProcedure;
    FOnCaptureProcedure : TOnCaptureProcedure;
    FOnCaptureErrorProcedure : TOnCaptureErrorProcedure;
    MyCaptureThreadStarted : boolean;
    MyCaptureThread : ThlcCaptureThread;
    Continue : boolean;
    OldBitmapPos, StartPos, LastPos : TPoint;
    Start:boolean;
    OldBitmap : TBitmap;
    ThevKey: integer;
    CopiedCaptureCursor: HCursor;
    procedure DrawTheRect(StartPos, LastPos : TPoint);
    procedure StopAreaSelection;
    function DrawTextInXORMode(TheBitmap : TBitmap):TPoint;
    procedure DrawOldBitmap(OldBitmapPos : TPoint; OldBitmap: TBitmap);
    procedure GetTheCaptureArea(TheAreaRect : TRect);
    procedure SetCaptureStyle(Value : TCaptureStyle);
  public
    { Public declarations }
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure StartCapture(vKey: integer; Delay: DWORD);
    procedure CaptureArea(TheArea : TRect);
    procedure DoCapture;
  published
    { Published declarations }
    property About : ThlcAbout read FAbout write FAbout;
    property CaptureStyle : TCaptureStyle read FCaptureStyle write SetCaptureStyle default csSCREEN;
    property IncludeCursor : boolean read FIncludeCursor write FIncludeCursor;
    property OnCapture : TOnCaptureProcedure read FOnCaptureProcedure write FOnCaptureProcedure;
    property OnBeforeCapture : TOnBeforeCaptureProcedure read FOnBeforeCaptureProcedure write FOnBeforeCaptureProcedure;
    property OnCaptureError : TOnCaptureErrorProcedure read FOnCaptureErrorProcedure write FOnCaptureErrorProcedure;
  end;

implementation

{$R AREA.RES}

uses Forms, Controls;

const
  crCaptureCursor=1236;
{$IFDEF VER90}
  OCR_NORMAL              = 32512;
{$ENDIF}

constructor ThlcCapture.Create (AOwner: TComponent);
begin
  inherited Create( AOwner);
  FCaptureStyle:=csSCREEN;
  FCaptureArea:=Rect(0,0,0,0);
  MyCaptureThreadStarted:=False;
end;

destructor ThlcCapture.Destroy;
begin
  inherited Destroy;
end;

procedure ThlcCapture.StopAreaSelection;
begin
  DrawOldBitmap(OldBitmapPos, OldBitmap);
  DrawTheRect(StartPos, LastPos);
  Start:=False;
end;

procedure ThlcCapture.DrawOldBitmap(OldBitmapPos : TPoint; OldBitmap: TBitmap);
var
  MyCanvas : TCanvas;
begin
  MyCanvas :=TCanvas.Create;
  try
    MyCanvas.Handle:=GetDC(0);
    MyCanvas.Draw(OldBitmapPos.X, OldBitmapPos.Y, OldBitmap);
  finally
    MyCanvas.Free;
  end;
end;

function ThlcCapture.DrawTextInXORMode(TheBitmap : TBitmap):TPoint;
var
  MyCanvas : TCanvas;
  StartCanvasPoint : TPoint;
  TheSizeText: string;
begin
  StartCanvasPoint:=Point((StartPos.X+LastPos.X) div 2, (StartPos.Y+LastPos.Y) div 2);
  TheSizeText:=Format('[%d,%d]',[abs(StartPos.X-LastPos.X), abs(StartPos.Y-LastPos.Y)]);
  MyCanvas :=TCanvas.Create;
  try
    MyCanvas.Handle:=GetDC(0);
    SetTextCharacterExtra(MyCanvas.Handle, 1);
    TheBitmap.Width:=MyCanvas.TextWidth(TheSizeText)+2;
    TheBitmap.Height:=MyCanvas.TextHeight(TheSizeText)+2;
    StartCanvasPoint.x:=StartCanvasPoint.x-TheBitmap.Width div 2;
    StartCanvasPoint.y:=StartCanvasPoint.y-TheBitmap.Height div 2;
    TheBitmap.Canvas.CopyRect(Rect(0,0,TheBitmap.Width,TheBitmap.Height), MyCanvas, Rect(StartCanvasPoint.x, StartCanvasPoint.y, StartCanvasPoint.x + TheBitmap.Width, StartCanvasPoint.y + TheBitmap.Height));
    SetBkMode(MyCanvas.Handle, TRANSPARENT);
    MyCanvas.Font.Color:=clWhite;
    MyCanvas.TextOut(-1+StartCanvasPoint.x, -1+StartCanvasPoint.y, TheSizeText);
    MyCanvas.TextOut(-1+StartCanvasPoint.x,   StartCanvasPoint.y, TheSizeText);
    MyCanvas.TextOut(  StartCanvasPoint.x, -1+StartCanvasPoint.y, TheSizeText);
    MyCanvas.Font.Color:=clDkGray;
    MyCanvas.TextOut(1+StartCanvasPoint.x, 1+StartCanvasPoint.y, TheSizeText);
    MyCanvas.TextOut(1+StartCanvasPoint.x,   StartCanvasPoint.y, TheSizeText);
    MyCanvas.TextOut(  StartCanvasPoint.x, 1+StartCanvasPoint.y, TheSizeText);
    MyCanvas.Font.Color:=clBlack;
    MyCanvas.TextOut(StartCanvasPoint.x, StartCanvasPoint.y, TheSizeText);
  finally
    MyCanvas.Free;
  end;
  result:=StartCanvasPoint;
end;

procedure ThlcCapture.DrawTheRect(StartPos, LastPos : TPoint);
var
  MyCanvas : TCanvas;
begin
  MyCanvas :=TCanvas.Create;
  try
    MyCanvas.Handle:=GetDC(0);
    MyCanvas.Pen.Mode:=pmXor;
    MyCanvas.Pen.Color:=clBlack;
    MyCanvas.Pen.Style:=psDot;
    MyCanvas.MoveTo(StartPos.X, StartPos.Y);
    MyCanvas.LineTo(StartPos.X, LastPos.Y);
    MyCanvas.LineTo(LastPos.X, LastPos.Y);
    MyCanvas.LineTo(LastPos.X, StartPos.Y);
    MyCanvas.LineTo(StartPos.X, StartPos.Y);
  finally
    MyCanvas.Free;
  end;
end;

procedure ThlcCapture.SetCaptureStyle(Value : TCaptureStyle);
begin
  FCaptureStyle:=Value;
end;

procedure ThlcCapture.StartCapture(vKey: integer; Delay: DWORD);
begin
  if not MyCaptureThreadStarted then
  begin
    if (vKey=0) and (Delay<=0)
    then DoCapture
    else
    begin
      ThevKey:=vKey;
      MyCaptureThread:=ThlcCaptureThread.Create(True);
      try
        MyCaptureThread.FreeOnTerminate:=True;
        MyCaptureThread.vKey:=vKey;
        if Delay>0
        then MyCaptureThread.Delay:=Delay
        else MyCaptureThread.Delay:=0;
        MyCaptureThread.TheSender:=Self;
        MyCaptureThread.Resume;
        MyCaptureThreadStarted:=True;
      except
        MyCaptureThreadStarted:=False;
      end;
    end;
  end
end;

procedure ScreenShot(x : integer;
                     y : integer;
                     Width : integer;
                     Height : integer;
                     bm : TBitMap);
var
  dc: HDC;
  lpPal : PLOGPALETTE;
begin
 {test width and height}
  if ((Width = 0) OR
      (Height = 0)) then begin
    exit;
  end;
  bm.Width := Width;
  bm.Height := Height;
 {get the screen dc}
  dc := GetDc(0);
  if (dc = 0) then begin
    exit;
  end;
 {do we have a palette device?}
  if (GetDeviceCaps(dc, RASTERCAPS) AND
      RC_PALETTE = RC_PALETTE) then begin
   {allocate memory for a logical palette}
    GetMem(lpPal,
           sizeof(TLOGPALETTE) +
           (255 * sizeof(TPALETTEENTRY)));
   {zero it out to be neat}
    FillChar(lpPal^,
             sizeof(TLOGPALETTE) +
             (255 * sizeof(TPALETTEENTRY)),
             #0);
   {fill in the palette version}
    lpPal^.palVersion := $300;
   {grab the system palette entries}
    lpPal^.palNumEntries :=
      GetSystemPaletteEntries(dc,
                              0,
                              256,
                              lpPal^.palPalEntry);
    if (lpPal^.PalNumEntries <> 0) then begin
     {create the palette}
      bm.Palette := CreatePalette(lpPal^);
    end;
    FreeMem(lpPal, sizeof(TLOGPALETTE) +
            (255 * sizeof(TPALETTEENTRY)));
  end;
 {copy from the screen to the bitmap}
  BitBlt(bm.Canvas.Handle,
         0,
         0,
         Width,
         Height,
         Dc,
         x,
         y,
         SRCCOPY);
 {release the screen dc}
  ReleaseDc(0, dc);
end;

procedure ThlcCapture.GetTheCaptureArea(TheAreaRect : TRect);
var
  MyPoint : TPoint;
  rectSource : TRect;
  TheBitmap:TBitmap;
  Succeeded:boolean;
  ErrorStr : string;
  hulpint : integer;
  CurrentCursorInfo: TIconInfo;
begin
  Succeeded:=True;
  rectSource:=TheAreaRect;
  if rectSource.Left>rectSource.Right then
  begin
    hulpint:=rectSource.Left;
    rectSource.Left:=rectSource.Right;
    rectSource.Right:=hulpint;
  end;
  if rectSource.Top>rectSource.Bottom then
  begin
    hulpint:=rectSource.Top;
    rectSource.Top:=rectSource.Bottom;
    rectSource.Bottom:=hulpint;
  end;
  TheBitmap:=TBitmap.Create;
  try
    TheBitmap.Width:=rectSource.Right-rectSource.Left;
    TheBitmap.Height:=rectSource.Bottom-rectSource.Top;
    ScreenShot(rectSource.Left, rectSource.Top, TheBitmap.Width, TheBitmap.Height, TheBitmap);
    if IncludeCursor then
    begin
      if not Windows.GetCursorPos(MyPoint)
      then
      begin
        Succeeded:=False;
        ErrorStr:=SysErrorMessage(GetLastError);
      end
      else
      begin
        GetIconInfo(GetCursor, CurrentCursorInfo);
        MyPoint.x:=MyPoint.x-rectSource.Left-integer(CurrentCursorInfo.xHotspot);
        MyPoint.y:=MyPoint.y-rectSource.Top-integer(CurrentCursorInfo.yHotspot);
//        if not DrawIcon(TheBitmap.Canvas.Handle, MyPoint.X, MyPoint.Y, GetCursor) then
        if not DrawIconEx(TheBitmap.Canvas.Handle, MyPoint.X, MyPoint.Y, GetCursor, 0, 0, 0, 0, DI_NORMAL) then
        begin
          // Error Drawing MousePointer;
          Succeeded:=False;
          ErrorStr:=SysErrorMessage(GetLastError);
        end;
      end;
      if not Succeeded then
      begin
        if (Assigned(FOnCaptureErrorProcedure))
        then FOnCaptureErrorProcedure(Self, ErrorStr);
      end
    end;
    if Assigned(FOnCaptureProcedure)
    then FOnCaptureProcedure(Self, TheBitmap);
  finally
    TheBitmap.Free;
  end;
end;

procedure ThlcCapture.CaptureArea(TheArea : TRect);
begin
  GetTheCaptureArea(TheArea);
end;

procedure ThlcCapture.DoCapture;
var
  MyPoint : TPoint;
  MyHandle, hwndChild, hwndSource: HWND;
  rectSource : TRect;
  Succeeded:boolean;
  ErrorStr : string;
  TheMousePoint :TPoint;
  CaptureCursor: HCursor;
begin
  if FCaptureStyle=csAREA then
  begin
    GetCursorPos(StartPos);
    Start:=True;
    CaptureCursor := LoadCursor(HInstance, 'AREA');
    CopiedCaptureCursor := CopyIcon(Screen.Cursors[OCR_NORMAL]);
    SetSystemCursor(CaptureCursor, OCR_NORMAL);
    Windows.SetCapture(0);
    LastPos:=StartPos;
    OldBitmap:=TBitmap.Create;
    try
      OldBitmapPos:=DrawTextInXORMode(OldBitmap);
      DrawTheRect(StartPos, LastPos);
      repeat
      until GetAsyncKeyState(ThevKey) = 0;
      while (not (GetAsyncKeyState(ThevKey)<>0)) and (not Application.Terminated) do
      begin
        Windows.sleep(20);
        GetCursorPos(TheMousePoint);
        if not ((TheMousePoint.x=LastPos.x) and (TheMousePoint.y=LastPos.y)) then
        begin
          DrawOldBitmap(OldBitmapPos, OldBitmap);
          DrawTheRect(StartPos, LastPos);
          LastPos:=Point(TheMousePoint.X, TheMousePoint.Y);
          DrawTheRect(StartPos, LastPos);
          OldBitmapPos:=DrawTextInXORMode(OldBitmap);
        end;
      end;
    finally
      ReleaseCapture;
      StopAreaSelection;
      SetSystemCursor(CopiedCaptureCursor, OCR_NORMAL);
      OldBitmap.Free;
    end;
  end;
  MyCaptureThreadStarted:=False;
  hwndSource:=0;
  Succeeded:=True;
  ErrorStr:='';
  begin
    case FCaptureStyle of
      csSCREEN:
      begin
        hwndSource:=GetDesktopWindow;
        if hwndSource=0 then
        begin
          Succeeded:=False;
          ErrorStr:=SysErrorMessage(GetLastError);
        end
        else
        begin
          if not GetWindowRect(hwndSource, rectSource)
          then
          begin
            Succeeded:=False;
            ErrorStr:=SysErrorMessage(GetLastError);
          end;
        end
      end;
      csWINDOW, csCLIENTAREA:
      begin
        if not Windows.GetCursorPos(MyPoint)
        then
        begin
          Succeeded:=False;
          ErrorStr:=SysErrorMessage(GetLastError);
        end
        else
        begin
          MyHandle:=Windows.WindowFromPoint(MyPoint);
          if MyHandle=0 then
          begin
            Succeeded:=False;
            ErrorStr:=SysErrorMessage(GetLastError);
          end
          else
          begin
            // Ook controleren of breedte en hoogte nul zijn!!
            if GetParent(MyHandle)<>0
            then GetWindowRect(GetParent(MyHandle), rectSource);
            while (rectSource.Right-rectSource.Left<>0) and (rectSource.Bottom-rectSource.Top<>0) and (GetParent(MyHandle)<>0) and (IsWindowVisible(MyHandle)) and (not IsIconic(MyHandle)) do
            begin
              MyHandle:=GetParent(MyHandle);
              if GetParent(MyHandle)<>0
              then GetWindowRect(GetParent(MyHandle), rectSource);
            end;
            if FCaptureStyle=csWindow then
            begin
              if not GetWindowRect(MyHandle, rectSource)
              then
              begin
                Succeeded:=False;
                ErrorStr:=SysErrorMessage(GetLastError);
              end
              else
              begin
                hwndSource:=MyHandle;
              end;
            end
            else // FCaptureStyle=csClientArea
            begin
              if not GetClientRect(MyHandle, rectSource)
              then
              begin
                Succeeded:=False;
                ErrorStr:=SysErrorMessage(GetLastError);
              end
              else
              begin
                MyPoint.x:=0;
                MyPoint.y:=0;
                if not Windows.ClientToScreen(MyHandle, MyPoint)
                then
                begin
                  Succeeded:=False;
                  ErrorStr:=SysErrorMessage(GetLastError);
                end
                else
                begin
                  rectSource.Left:=rectSource.Left+MyPoint.x;
                  rectSource.Top:=rectSource.Top+MyPoint.y;
                  rectSource.Right:=rectSource.Right+MyPoint.x;
                  rectSource.Bottom:=rectSource.Bottom+MyPoint.y;
                  hwndSource:=MyHandle;
                end;
              end;
            end;
          end;
        end;
      end;
      csACTIVEWINDOW:
      begin
        MyHandle:=GetForegroundWindow;
        if MyHandle=0 then
        begin
          Succeeded:=False;
          ErrorStr:=SysErrorMessage(GetLastError);
        end
        else
        begin
          if not GetWindowRect(MyHandle, rectSource)
          then
          begin
            Succeeded:=False;
            ErrorStr:=SysErrorMessage(GetLastError);
          end
          else
          begin
            hwndSource:=MyHandle;
          end;
        end;
      end;
      csObject:
      begin
        if not Windows.GetCursorPos(MyPoint)
        then
        begin
          Succeeded:=False;
          ErrorStr:=SysErrorMessage(GetLastError);
        end
        else
        begin
          MyHandle:=Windows.WindowFromPoint(MyPoint);
          if MyHandle=0
          then
          begin
            Succeeded:=False;
            ErrorStr:=SysErrorMessage(GetLastError);
          end
          else
          begin
            if not Windows.ScreenToClient(MyHandle, MyPoint)
            then
            begin
              Succeeded:=False;
              ErrorStr:=SysErrorMessage(GetLastError);
            end
            else
            begin
              hwndChild:=ChildWindowFromPointEx(MyHandle, MyPoint, CWP_SKIPINVISIBLE or CWP_SKIPTRANSPARENT);
              if hwndChild<>0
              then hwndSource:=hwndChild
              else hwndSource:=MyHandle;
              if not GetClientRect(hwndSource, rectSource)
              then
              begin
                Succeeded:=False;
                ErrorStr:=SysErrorMessage(GetLastError);
              end
              else
              begin
                MyPoint.x:=0;
                MyPoint.y:=0;
                if not Windows.ClientToScreen(hwndSource, MyPoint)
                then
                begin
                  Succeeded:=False;
                  ErrorStr:=SysErrorMessage(GetLastError);
                end
                else
                begin
                  rectSource.Left:=rectSource.Left+MyPoint.x;
                  rectSource.Top:=rectSource.Top+MyPoint.y;
                  rectSource.Right:=rectSource.Right+MyPoint.x;
                  rectSource.Bottom:=rectSource.Bottom+MyPoint.y;
                end;
              end;
            end;
          end;
        end;
      end;
      csAREA:
      begin
        rectSource:=Rect(StartPos.X, StartPos.Y, LastPos.X, LastPos.Y);
      end;
    end;
    if not Succeeded then
    begin
      if (Assigned(FOnCaptureErrorProcedure))
      then FOnCaptureErrorProcedure(Self, ErrorStr);
    end
    else
    begin
      Continue:=True;
      if Assigned(FOnBeforeCaptureProcedure)
      then FOnBeforeCaptureProcedure(Self, hwndSource, Continue);
      if Continue
      then GetTheCaptureArea(rectSource);
    end;
  end;
end;

end.


