//{$D-}
{*************************************************************************}
{                                                                         }
{                  HlcExtendedSpinEdit Component                          }
{                                                                         }
{  The HlcExtendedSpinEdit is a SpinEdit control which supports extended  }
{  types. It handles integers, reals, doubles and extended types, like    }
{  4535,7454. Most of the code comes from the SpinEdit sample shipped     }
{  with Delphi.                                                           }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{  Copyright (c) 1997, 1999 High-Logic [Erwin Denissen]                   }
{                                                                         }
{  You may use a Shareware Copy of this Software for an evaluation period }
{  of up to 30 days, in order to determine whether the Software meets     }
{  your needs before purchasing it. Once the evaluation period ends, you  }
{  agree to either purchase a copy of the software, or to stop using it   }
{  and remove it from your computer system.                               }
{                                                                         }
{  A registered version of this and other components and software can be  }
{  ordered at the following site: http://www.high-logic.com/index.html    }
{                                                                         }
{  There are no known bugs at the moment, so please notify me when you    }
{  find one. Make sure you are using the latest version of this software. }
{  email: erwind@high-logic.com                                           }
{                                                                         }
{  DISCLAIMER                                                             }
{  In no event will the author be held liable for any direct or indirect  }
{  damage resulting from the use of this software or any derivatives      }
{  thereof.                                                               }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{ Version 1.1 27-04-1999                                                  }
{ - Added the ThlcExtendedCustomEdit as the base class also included an   }
{   ThlcExtendedEdit component                                            }
{                                                                         }
{ Version 1.0 17-04-1999                                                  }
{ - First release, works with Delphi 2, 3 and 4 and with C++Builder 3     }
{                                                                         }
{*************************************************************************}

unit HlcExtendedSpinEdit;

interface

uses Windows, Classes, StdCtrls, ExtCtrls, Controls, Messages, SysUtils,
  Forms, Graphics, Menus, Buttons, HlcRegisterAbout;

const
  InitRepeatPause = 400;  { pause before repeat timer (ms) }
  RepeatPause     = 100;  { pause before hint window displays (ms)}

type

  TNumGlyphs = Buttons.TNumGlyphs;

  ThlcTimerSpeedButton = class;

{ ThlcSpinButton }

  ThlcSpinButton = class (TWinControl)
  private
    FUpButton: ThlcTimerSpeedButton;
    FDownButton: ThlcTimerSpeedButton;
    FFocusedButton: ThlcTimerSpeedButton;
    FFocusControl: TWinControl;
    FOnUpClick: TNotifyEvent;
    FOnDownClick: TNotifyEvent;
    function CreateButton: ThlcTimerSpeedButton;
    function GetUpGlyph: TBitmap;
    function GetDownGlyph: TBitmap;
    procedure SetUpGlyph(Value: TBitmap);
    procedure SetDownGlyph(Value: TBitmap);
    function GetUpNumGlyphs: TNumGlyphs;
    function GetDownNumGlyphs: TNumGlyphs;
    procedure SetUpNumGlyphs(Value: TNumGlyphs);
    procedure SetDownNumGlyphs(Value: TNumGlyphs);
    procedure BtnClick(Sender: TObject);
    procedure BtnMouseDown (Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SetFocusBtn (Btn: ThlcTimerSpeedButton);
    procedure AdjustSize (var W, H: Integer); reintroduce;
    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
  protected
    procedure Loaded; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
    property Align;
    property Anchors;
    property Constraints;
    property Ctl3D;
    property DownGlyph: TBitmap read GetDownGlyph write SetDownGlyph;
    property DownNumGlyphs: TNumGlyphs read GetDownNumGlyphs write SetDownNumGlyphs default 1;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl: TWinControl read FFocusControl write FFocusControl;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property UpGlyph: TBitmap read GetUpGlyph write SetUpGlyph;
    property UpNumGlyphs: TNumGlyphs read GetUpNumGlyphs write SetUpNumGlyphs default 1;
    property Visible;
    property OnDownClick: TNotifyEvent read FOnDownClick write FOnDownClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnStartDock;
    property OnStartDrag;
    property OnUpClick: TNotifyEvent read FOnUpClick write FOnUpClick;
  end;

{ ThlcExtendedCustomEdit }

  ThlcExtendedCustomEdit = class(TCustomEdit)
  private
    FActualText: string;
    FColorEnabled : TColor;
    FColorDisabled: TColor;
    FClearTextWhileDisabled: boolean;
    procedure ChangeColor(Value: Boolean);
    procedure SetColorEnabled(const Value: TColor);
    procedure SetColorDisabled(const Value: TColor);
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure SetClearTextWhileDisabled(const Value: boolean);
  protected
    procedure SetEnabled(Value: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
  public
    property ClearTextWhileDisabled: boolean read FClearTextWhileDisabled write SetClearTextWhileDisabled;
    property ColorEnabled: TColor read FColorEnabled write SetColorEnabled;
    property ColorDisabled: TColor read FColorDisabled write SetColorDisabled;
  end;

{ ThlcExtendedEdit }

  ThlcExtendedEdit = class(ThlcExtendedCustomEdit)
  private
    SkipTextChanged: boolean;
    FAbout: ThlcAbout;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property About : ThlcAbout read FAbout write FAbout;
    property Anchors;
    property AutoSelect;
    property AutoSize;
    property BiDiMode;
    property BorderStyle;
    property CharCase;
    { We do not want to publish the Color property because we use two other
      properties instead. ColorEnabled and ColorDisabled.}
    property ColorEnabled;
    property ColorDisabled;
    property ClearTextWhileDisabled;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

{ ThlcExtendedSpinEdit }

  ThlcExtendedSpinEdit = class(ThlcExtendedCustomEdit)
  private
    SkipTextChanged: boolean;
    FMinValue: Extended;
    FMaxValue: Extended;
    FIncrement: dword;
    FPreviousValue : Extended;
    FDigits: integer;
    FButton: ThlcSpinButton;
    FEditorEnabled: Boolean;
    FAllowEmpty: Boolean;
    FAbout: ThlcAbout;
    function GetMinHeight: Integer;
    function GetValue: Extended;
    function CheckValue (NewValue: Extended): Extended;
    procedure SetValue (NewValue: Extended);
    procedure SetEditRect;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure CMEnter(var Message: TCMGotFocus); message CM_ENTER;
    procedure CMExit(var Message: TCMExit);   message CM_EXIT;
    procedure WMPaste(var Message: TWMPaste);   message WM_PASTE;
    procedure WMCut(var Message: TWMCut);   message WM_CUT;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure SetDigits(const Value: integer);
    procedure SetAllowEmpty(const Value: Boolean);
    procedure RegenerateText(NewValue: Extended);
  protected
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    function IsValidChar(Key: Char): Boolean; virtual;
    procedure UpClick (Sender: TObject); virtual;
    procedure DownClick (Sender: TObject); virtual;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Button: ThlcSpinButton read FButton;
  published
    property About : ThlcAbout read FAbout write FAbout;
    property AllowEmpty: Boolean read FAllowEmpty write SetAllowEmpty default False;
    property Anchors;
    property AutoSelect;
    property AutoSize;
    property ColorEnabled;
    property ColorDisabled;
    property ClearTextWhileDisabled;
    property Constraints;
    property Ctl3D;
    property Digits: integer read FDigits write SetDigits default 2;
    property DragCursor;
    property DragMode;
    property EditorEnabled: Boolean read FEditorEnabled write FEditorEnabled default True;
    property Enabled;
    property Font;
    property Increment: dword read FIncrement write FIncrement default 1;
    property MaxLength;
    property MaxValue: Extended read FMaxValue write FMaxValue;
    property MinValue: Extended read FMinValue write FMinValue;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Value: Extended read GetValue write SetValue;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

{ ThlcTimerSpeedButton }

  TTimeBtnState = set of (tbFocusRect, tbAllowTimer);

  ThlcTimerSpeedButton = class(TSpeedButton)
  private
    FRepeatTimer: TTimer;
    FTimeBtnState: TTimeBtnState;
    procedure TimerExpired(Sender: TObject);
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  public
    destructor Destroy; override;
    property TimeBtnState: TTimeBtnState read FTimeBtnState write FTimeBtnState;
  end;

implementation

{$R HLCSPIN}

{ ThlcSpinButton }

constructor ThlcSpinButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csAcceptsControls, csSetCaption] +
    [csFramed, csOpaque];

  FUpButton := CreateButton;
  FDownButton := CreateButton;
  UpGlyph := nil;
  DownGlyph := nil;
  Width := 20;
  Height := 25;
  FFocusedButton := FUpButton;
end;

function ThlcSpinButton.CreateButton: ThlcTimerSpeedButton;
begin
  Result := ThlcTimerSpeedButton.Create (Self);
  Result.OnClick := BtnClick;
  Result.OnMouseDown := BtnMouseDown;
  Result.Visible := True;
  Result.Enabled := True;
  Result.TimeBtnState := [tbAllowTimer];
  Result.Parent := Self;
end;

procedure ThlcSpinButton.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FFocusControl) then
    FFocusControl := nil;
end;

procedure ThlcSpinButton.AdjustSize (var W, H: Integer);
begin
  if (FUpButton = nil) or (csLoading in ComponentState) then Exit;
  if W < 15 then W := 15;
  FUpButton.SetBounds (0, 0, W, H div 2);
  FDownButton.SetBounds (0, FUpButton.Height - 1, W, H - FUpButton.Height + 1);
end;

procedure ThlcSpinButton.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  W, H: Integer;
begin
  W := AWidth;
  H := AHeight;
  AdjustSize (W, H);
  inherited SetBounds (ALeft, ATop, W, H);
end;

procedure ThlcSpinButton.WMSize(var Message: TWMSize);
var
  W, H: Integer;
begin
  inherited;
  { check for minimum size }
  W := Width;
  H := Height;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds(Left, Top, W, H);
  Message.Result := 0;
end;

procedure ThlcSpinButton.WMSetFocus(var Message: TWMSetFocus);
begin
  FFocusedButton.TimeBtnState := FFocusedButton.TimeBtnState + [tbFocusRect];
  FFocusedButton.Invalidate;
end;

procedure ThlcSpinButton.WMKillFocus(var Message: TWMKillFocus);
begin
  FFocusedButton.TimeBtnState := FFocusedButton.TimeBtnState - [tbFocusRect];
  FFocusedButton.Invalidate;
end;

procedure ThlcSpinButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_UP:
      begin
        SetFocusBtn (FUpButton);
        FUpButton.Click;
      end;
    VK_DOWN:
      begin
        SetFocusBtn (FDownButton);
        FDownButton.Click;
      end;
    VK_SPACE:
      FFocusedButton.Click;
  end;
end;

procedure ThlcSpinButton.BtnMouseDown (Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    SetFocusBtn (ThlcTimerSpeedButton (Sender));
    if (FFocusControl <> nil) and FFocusControl.TabStop and 
        FFocusControl.CanFocus and (GetFocus <> FFocusControl.Handle) then
      FFocusControl.SetFocus
    else if TabStop and (GetFocus <> Handle) and CanFocus then
      SetFocus;
  end;
end;

procedure ThlcSpinButton.BtnClick(Sender: TObject);
begin
  if Sender = FUpButton then
  begin
    if Assigned(FOnUpClick) then FOnUpClick(Self);
  end
  else
    if Assigned(FOnDownClick) then FOnDownClick(Self);
end;

procedure ThlcSpinButton.SetFocusBtn (Btn: ThlcTimerSpeedButton);
begin
  if TabStop and CanFocus and  (Btn <> FFocusedButton) then
  begin
    FFocusedButton.TimeBtnState := FFocusedButton.TimeBtnState - [tbFocusRect];
    FFocusedButton := Btn;
    if (GetFocus = Handle) then 
    begin
       FFocusedButton.TimeBtnState := FFocusedButton.TimeBtnState + [tbFocusRect];
       Invalidate;
    end;
  end;
end;

procedure ThlcSpinButton.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;

procedure ThlcSpinButton.Loaded;
var
  W, H: Integer;
begin
  inherited Loaded;
  W := Width;
  H := Height;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds (Left, Top, W, H);
end;

function ThlcSpinButton.GetUpGlyph: TBitmap;
begin
  Result := FUpButton.Glyph;
end;

procedure ThlcSpinButton.SetUpGlyph(Value: TBitmap);
begin
  if Value <> nil then
    FUpButton.Glyph := Value
  else
  begin
    FUpButton.Glyph.Handle := LoadBitmap(HInstance, 'HlcSpinUp');
    FUpButton.NumGlyphs := 1;
    FUpButton.Invalidate;
  end;
end;

function ThlcSpinButton.GetUpNumGlyphs: TNumGlyphs;
begin
  Result := FUpButton.NumGlyphs;
end;

procedure ThlcSpinButton.SetUpNumGlyphs(Value: TNumGlyphs);
begin
  FUpButton.NumGlyphs := Value;
end;

function ThlcSpinButton.GetDownGlyph: TBitmap;
begin
  Result := FDownButton.Glyph;
end;

procedure ThlcSpinButton.SetDownGlyph(Value: TBitmap);
begin
  if Value <> nil then
    FDownButton.Glyph := Value
  else
  begin
    FDownButton.Glyph.Handle := LoadBitmap(HInstance, 'HlcSpinDown');
    FUpButton.NumGlyphs := 1;
    FDownButton.Invalidate;
  end;
end;

function ThlcSpinButton.GetDownNumGlyphs: TNumGlyphs;
begin
  Result := FDownButton.NumGlyphs;
end;

procedure ThlcSpinButton.SetDownNumGlyphs(Value: TNumGlyphs);
begin
  FDownButton.NumGlyphs := Value;
end;

{ ThlcExtendedSpinEdit }

constructor ThlcExtendedSpinEdit.Create(AOwner: TComponent);
begin
  SkipTextChanged:=False;
  inherited Create(AOwner);
  FButton := ThlcSpinButton.Create (Self);
  FButton.Width := 15;
  FButton.Height := 17;
  FButton.Visible := True;
  FButton.Parent := Self;
  FButton.FocusControl := Self;
  FButton.OnUpClick := UpClick;
  FButton.OnDownClick := DownClick;
  Text := '0';
  FPreviousValue := 0;
  ControlStyle := ControlStyle - [csSetCaption];
  FIncrement := 1;
  FAllowEmpty := False;
  Digits := 2;
  FEditorEnabled := True;
end;

destructor ThlcExtendedSpinEdit.Destroy;
begin
  FButton := nil;
  inherited Destroy;
end;

procedure ThlcExtendedSpinEdit.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

procedure ThlcExtendedSpinEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Key = VK_UP then UpClick (Self)
  else if Key = VK_DOWN then DownClick (Self);
  inherited KeyDown(Key, Shift);
end;

procedure ThlcExtendedSpinEdit.KeyPress(var Key: Char);
begin
  if not IsValidChar(Key) then
  begin
    Key := #0;
    MessageBeep(0)
  end;
  if Key <> #0 then inherited KeyPress(Key);
end;

function ThlcExtendedSpinEdit.IsValidChar(Key: Char): Boolean;
begin
  Result := (Key in [DecimalSeparator, '+', '-', '0'..'9']) or
    ((Key < #32) and (Key <> Chr(VK_RETURN)));
  if not FEditorEnabled and Result and ((Key >= #32) or
      (Key = Char(VK_BACK)) or (Key = Char(VK_DELETE))) then
    Result := False;
end;

procedure ThlcExtendedSpinEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or ES_MULTILINE or WS_CLIPCHILDREN;
end;

procedure ThlcExtendedSpinEdit.CreateWnd;
begin
  inherited CreateWnd;
  SetEditRect;
end;

procedure ThlcExtendedSpinEdit.SetEditRect;
var
  Loc: TRect;
begin
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
  Loc.Bottom := ClientHeight + 1;  {+1 is workaround for windows paint bug}
  Loc.Right := ClientWidth - FButton.Width - 2;
  Loc.Top := 0;  
  Loc.Left := 0;  
  SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));  {debug}
end;

procedure ThlcExtendedSpinEdit.WMSize(var Message: TWMSize);
var
  MinHeight: Integer;
begin
  inherited;
  MinHeight := GetMinHeight;
    { text edit bug: if size to less than minheight, then edit ctrl does
      not display the text }
  if Height < MinHeight then   
    Height := MinHeight
  else if FButton <> nil then
  begin
    if NewStyleControls and Ctl3D then
      FButton.SetBounds(Width - FButton.Width - 5, 0, FButton.Width, Height - 5)
    else FButton.SetBounds (Width - FButton.Width, 1, FButton.Width, Height - 3);
    SetEditRect;
  end;
end;

function ThlcExtendedSpinEdit.GetMinHeight: Integer;
var
  DC: HDC;
  SaveFont: HFont;
  I: Integer;
  SysMetrics, Metrics: TTextMetric;
begin
  DC := GetDC(0);
  GetTextMetrics(DC, SysMetrics);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  I := SysMetrics.tmHeight;
  if I > Metrics.tmHeight then I := Metrics.tmHeight;
  Result := Metrics.tmHeight + I div 4 + GetSystemMetrics(SM_CYBORDER) * 4 + 2;
end;

procedure ThlcExtendedSpinEdit.UpClick (Sender: TObject);
var
  Sign: integer;
begin
  if ReadOnly then MessageBeep(0) else
  begin
    if Value>=0
    then Sign:=1
    else Sign:=-1;
    if Value>=0 then
    begin
      Value := Sign*trunc(abs(Value)) + FIncrement;
    end
    else
    begin
      if Value=Sign*trunc(abs(Value))
      then Value := Sign*trunc(abs(Value)) + FIncrement
      else Value := Sign*trunc(abs(Value)) + FIncrement -1;
    end;
    Text := FloatToStrF (CheckValue (Value), ffFixed, 18, Digits);
  end;
end;

procedure ThlcExtendedSpinEdit.DownClick (Sender: TObject);
var
  Sign: integer;
begin
  if ReadOnly then MessageBeep(0) else
  begin
    if Value>=0
    then Sign:=1
    else Sign:=-1;
    if Value>=0 then
    begin
      if Value=Sign*trunc(abs(Value))
      then Value := Sign*trunc(abs(Value)) - FIncrement
      else Value := Sign*trunc(abs(Value)) - FIncrement +1;
    end
    else
    begin
      Value := Sign*trunc(abs(Value)) - FIncrement;
    end;
    Text := FloatToStrF (CheckValue (Value), ffFixed, 18, Digits);
  end;
end;

procedure ThlcExtendedSpinEdit.WMPaste(var Message: TWMPaste);
begin
  if not FEditorEnabled or ReadOnly then Exit;
  inherited;
end;

procedure ThlcExtendedSpinEdit.WMCut(var Message: TWMPaste);
begin
  if not FEditorEnabled or ReadOnly then Exit;
  inherited;
end;               

procedure ThlcExtendedSpinEdit.CMExit(var Message: TCMExit);
begin
  inherited;
  if (Text='') and (not AllowEmpty)
  then Value := FPreviousValue;
  if CheckValue (Value) <> Value
  then SetValue (Value);
end;

function ThlcExtendedSpinEdit.GetValue: Extended;
begin
  if not Enabled then
  begin
    try
      Result := StrToFloat (FActualText);
    except
      Result := FPreviousValue;
      Value := FPreviousValue;
    end;
    exit;
  end;
  if Text='' then
  begin
    GetValue:=0;
    exit;
  end;
  try
    Result := StrToFloat (Text);
    FPreviousValue:=Result;
  except
    Result := FPreviousValue;
    Value := FPreviousValue;
  end;
  RegenerateText(Result);
end;

procedure ThlcExtendedSpinEdit.SetValue (NewValue: Extended);
begin
  FPreviousValue:=NewValue;
  Text := FloatToStrF (CheckValue (NewValue), ffFixed, 18, Digits);
end;

function ThlcExtendedSpinEdit.CheckValue (NewValue: Extended): Extended;
begin
  Result := NewValue;
  if (FMaxValue <> FMinValue) then
  begin
    if NewValue < FMinValue then
      Result := FMinValue
    else if NewValue > FMaxValue then
      Result := FMaxValue;
  end;
end;

procedure ThlcExtendedSpinEdit.CMEnter(var Message: TCMGotFocus);
begin
  if AutoSelect and not (csLButtonDown in ControlState) then
    SelectAll;
  inherited;
end;

procedure ThlcExtendedSpinEdit.SetDigits(const Value: integer);
begin
  FDigits:=Value;
  if Value<0
  then FDigits:=0;
  if Value>18
  then FDigits:=18;
  RegenerateText(StrToFloat(Text));
end;

procedure ThlcExtendedSpinEdit.SetAllowEmpty(const Value: Boolean);
var
  tmpValue: Extended;
begin
  FAllowEmpty := Value;
  if not AllowEmpty then
  begin
    try
      tmpValue := StrToFloat (Text);
    except
      tmpValue := FPreviousValue;
    end;
    RegenerateText(tmpValue);
  end;
end;             

procedure ThlcExtendedSpinEdit.RegenerateText(NewValue: Extended);
begin
  if not (AllowEmpty and (Text=''))
  then Text := FloatToStrF (CheckValue (NewValue), ffFixed, 18, Digits);
end;

procedure ThlcExtendedSpinEdit.CMTextChanged(var Message: TMessage);
var
  NewText, TextSubmitted: string;
begin
  inherited;
  if SkipTextChanged then exit;
  SkipTextChanged:=True;
  TextSubmitted:=Text;
  NewText:=Text;
  if (not Enabled) and (ClearTextWhileDisabled) then
  begin // Make sure you can't see the text while disabled and ClearTextWhileDisabled is set!
    if NewText<>'' then
    begin
      FActualText:=NewText;
      Text:='';                                
    end;
  end
  else
  begin
    if not ((Text='') and (AllowEmpty)) then
    try
      StrToFloat (TextSubmitted);
    except
      TextSubmitted := FloatToStrF (CheckValue (FPreviousValue), ffFixed, 18, Digits);
    end;
    if Text<>TextSubmitted
    then NewText:=TextSubmitted;
    if Text<>NewText
    then Text:=NewText;
  end;
  SkipTextChanged:=False;
end;

{ThlcTimerSpeedButton}

destructor ThlcTimerSpeedButton.Destroy;
begin
  if FRepeatTimer <> nil
  then FRepeatTimer.Free;
  inherited Destroy;
end;

procedure ThlcTimerSpeedButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseDown (Button, Shift, X, Y);
  if tbAllowTimer in FTimeBtnState then
  begin
    if FRepeatTimer = nil
    then FRepeatTimer := TTimer.Create(Self);
    FRepeatTimer.OnTimer := TimerExpired;
    FRepeatTimer.Interval := InitRepeatPause;
    FRepeatTimer.Enabled  := True;
  end;
end;

procedure ThlcTimerSpeedButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp (Button, Shift, X, Y);
  if FRepeatTimer <> nil
  then FRepeatTimer.Enabled  := False;
end;

procedure ThlcTimerSpeedButton.TimerExpired(Sender: TObject);
begin
  FRepeatTimer.Interval := RepeatPause;
  if (FState = bsDown) and MouseCapture then
  begin
    try
      Click;
    except
      FRepeatTimer.Enabled := False;
      raise;
    end;
  end;
end;

procedure ThlcTimerSpeedButton.Paint;
var
  R: TRect;
begin
  inherited Paint;
  if tbFocusRect in FTimeBtnState then
  begin
    R := Bounds(0, 0, Width, Height);
    InflateRect(R, -3, -3);
    if FState = bsDown then
      OffsetRect(R, 1, 1);
    DrawFocusRect(Canvas.Handle, R);
  end;
end;

{ ThlcExtendedCustomEdit }

procedure ThlcExtendedCustomEdit.ChangeColor(Value: Boolean);
begin
  if Value then
  begin
    Color:=ColorEnabled;
    if ClearTextWhileDisabled then
    begin
      if FActualText<>''
      then Text:=FActualText;
    end;
    FActualText:='';
  end
  else
  begin
    Color:=ColorDisabled;
    if Text<>''
    then FActualText:=Text;
    if ClearTextWhileDisabled then
    begin
      Text:='';
    end;
  end;
end;

procedure ThlcExtendedCustomEdit.CMColorChanged(var Message: TMessage);
begin
  inherited;
  if Enabled
  then ColorEnabled:=Color
  else ColorDisabled:=Color;
end;
             
constructor ThlcExtendedCustomEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FColorEnabled:=clWindow;
  FColorDisabled:=clBtnFace;
  FActualText:=Text;
  FClearTextWhileDisabled:=False;
end;
          
procedure ThlcExtendedCustomEdit.SetClearTextWhileDisabled(
  const Value: boolean);
begin
  if FClearTextWhileDisabled <> Value then
  begin
    FClearTextWhileDisabled := Value;
    if not Enabled then
    begin
      if FClearTextWhileDisabled then
      begin
        Text:='';
      end
      else
      begin
        Text:=FActualText;
      end;
    end;
  end;
end;

procedure ThlcExtendedCustomEdit.SetColorDisabled(const Value: TColor);
begin
  FColorDisabled := Value;
  if not Enabled
  then ChangeColor(False);
end;

procedure ThlcExtendedCustomEdit.SetColorEnabled(const Value: TColor);
begin
  FColorEnabled := Value;
  if Enabled
  then ChangeColor(True);
end;

procedure ThlcExtendedCustomEdit.SetEnabled(Value: Boolean);
begin
  inherited;
  ChangeColor(Enabled);
end;

{ ThlcExtendedEdit }

procedure ThlcExtendedEdit.CMTextChanged(var Message: TMessage);
var
  NewText, TextSubmitted: string;
begin
  inherited;
  if SkipTextChanged then exit;
  SkipTextChanged:=True;
  try
    TextSubmitted:=Text;
    NewText:=Text;
    if (not Enabled) and (ClearTextWhileDisabled) then
    begin // Make sure you can't see the text while disabled and ClearTextWhileDisabled is set!
      if NewText<>'' then
      begin
        FActualText:=NewText;
        Text:='';
      end;
    end
    else
    begin
      if Text<>TextSubmitted
      then NewText:=TextSubmitted;
      if Text<>NewText
      then Text:=NewText;
    end;
  finally
    SkipTextChanged:=False;
  end;
end;

constructor ThlcExtendedEdit.Create(AOwner: TComponent);
begin
  SkipTextChanged:=False;
  inherited Create(AOwner);
end;

end.
