{-$D}
{*************************************************************************}
{                                                                         }
{                      HlcFileSearch Component                            }
{                                                                         }
{  This component lets you search folders and files.                      }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{  Copyright (c) 1997, 2000 High-Logic [Erwin Denissen]                   }
{                                                                         }
{  You may use a Shareware Copy of this Software for an evaluation period }
{  of up to 30 days, in order to determine whether the Software meets     }
{  your needs before purchasing it. Once the evaluation period ends, you  }
{  agree to either purchase a copy of the software, or to stop using it   }
{  and remove it from your computer system.                               }
{                                                                         }
{  A registered version of this and other components and software can be  }
{  ordered at the following site: http://www.high-logic.com/index.html    }
{                                                                         }
{  Make sure you are using the latest version of this software.           }
{  Send bug reports to erwind@high-logic.com                              }
{                                                                         }
{  DISCLAIMER                                                             }
{  In no event will the author be held liable for any direct or indirect  }
{  damage resulting from the use of this software or any derivatives      }
{  thereof.                                                               }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{ Version 1.0 04-02-2000                                                  }
{ - First release                                                         }
{ - Works with Delphi 2 and higher and C++Builder 3 and higher            }
{                                                                         }
{*************************************************************************}

unit HlcFileSearch;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  HlcFileSearchThread, HlcRegisterAbout;

type

  EhlcFileSearch = class(Exception);

  TOnPathChange = procedure(Sender: TObject; PathName: string; SearchRec: TSearchRec; var Continue : boolean) of object;
  TOnFileFound = procedure(Sender: TObject; FileName: string; SearchRec: TSearchRec; var Continue : boolean) of object;

  ThlcFileSearch = class(TComponent)
  private
    { Private declarations }
    MaskList,
    PathList:TSTringList;
    TheHlcFileSearchThread: THlcFileSearchThread;
    ThreadIsRunning: boolean;
    FOnFileFound: TOnFileFound;
    FOnPathChange: TOnPathChange;
    FOnStop: TNotifyEvent;
    FRecursive: boolean;
    FAbout: THLCAbout;
    procedure ThreadOnTerminate(Sender: TObject);
    function LogFiles(const Path: string; const SRec: TSearchRec): Boolean;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Start;
    procedure Cancel;
    procedure ClearTasks;
    procedure AddSearchTask(const SearchPath, SearchMask: string);
  published
    { Published declarations }
    property About : THLCAbout read FAbout write FAbout;
    property OnPathChange: TOnPathChange read FOnPathChange write FOnPathChange;
    property OnFileFound: TOnFileFound read FOnFileFound write FOnFileFound;
    property OnStop: TNotifyEvent read FOnStop write FOnStop;
    property Recursive: boolean read FRecursive write FRecursive;
  end;

implementation


{ ThlcFileSearch }

procedure ThlcFileSearch.AddSearchTask(const SearchPath,
  SearchMask: string);
begin
  MaskList.Add(SearchMask);
  PathList.Add(SearchPath);
end;

procedure ThlcFileSearch.Cancel;
begin
  if ThreadIsRunning then
  begin
    ThreadIsRunning:=False;
    TheHlcFileSearchThread.Terminate;
  end;
end;

procedure ThlcFileSearch.ClearTasks;
begin
  MaskList.Clear;
  PathList.Clear;
end;

constructor ThlcFileSearch.Create(AOwner: TComponent);
begin
  inherited;
  ThreadIsRunning:=False;
  MaskList := TSTringList.Create;
  PathList := TSTringList.Create;
  Recursive := True;
end;

destructor ThlcFileSearch.Destroy;
begin
  if ThreadIsRunning then
  begin
    ThreadIsRunning:=False;
    TheHlcFileSearchThread.Terminate;
  end;
  MaskList.Free;
  MaskList:=nil;
  PathList.Free;
  PathList:=nil;
  inherited;
end;

function ThlcFileSearch.LogFiles(const Path: string;
  const SRec: TSearchRec): Boolean;
var
  TheName: string;
  Continue: boolean;
  Separator : string;

  function DirectoryExists(const Name: string): Boolean;
  var
    Code: Integer;
  begin
    Code := GetFileAttributes(PChar(Name));
    Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code <> 0);
  end;

begin
  if not ThreadIsRunning then
  begin
    Result:=False;
    Exit;
  end;
  Continue:=True;
  if Copy(Path, Length(Path),1)='\'
  then Separator := ''
  else Separator := '\';
  TheName:=Path+Separator+SRec.Name;
  if FileExists(TheName) then
  begin
    if Assigned(OnFileFound)
    then OnFileFound(Self, TheName, SRec, Continue);
  end else
  begin
    if DirectoryExists(TheName) then
    begin
      if Assigned(OnPathChange)
      then OnPathChange(Self, TheName, SRec, Continue);
    end;
  end;
  Result := Continue;
end;

procedure ThlcFileSearch.Start;
var
  iCount: integer;
begin
  if ThreadIsRunning then
  begin
    raise EhlcFileSearch.CreateFmt('%s already started.', [ClassName]);
  end else
  begin
    ThreadIsRunning:=True;
    TheHlcFileSearchThread := THlcFileSearchThread.Create(LogFiles, Recursive);
    TheHlcFileSearchThread.OnTerminate := ThreadOnTerminate;
    // Vullen van zoek opdrachten
    if PathList.Count>0 then
    begin
      for iCount:=0 to PathList.Count-1 do
      begin
        TheHlcFileSearchThread.AddSearchTask(PathList[iCount], MaskList[iCount]);
      end;
    end;
    // Start the thread
    TheHlcFileSearchThread.Resume;
  end;
end;

procedure ThlcFileSearch.ThreadOnTerminate(Sender: TObject);
begin
  ThreadIsRunning:=False;
  if Assigned(OnStop)
  then OnStop(Self);
end;

end.
