{$D-}
{*************************************************************************}
{                                                                         }
{                    HlcRegionInline Component                            }
{                                                                         }
{  The HlcRegionInline not only stores a region but also maintains it     }
{  inside your application without resources and coding! This way you can }
{  create and add a SplashScreen to your project. You can use the         }
{  included property editor at designtime.                                }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{  Copyright (c) 1997, 1999 High-Logic [Erwin Denissen]                   }
{                                                                         }
{  You may use a Shareware Copy of this Software for an evaluation period }
{  of up to 30 days, in order to determine whether the Software meets     }
{  your needs before purchasing it. Once the evaluation period ends, you  }
{  agree to either purchase a copy of the software, or to stop using it   }
{  and remove it from your computer system.                               }
{                                                                         }
{  A registered version of this and other components and software can be  }
{  ordered at the following site: http://www.high-logic.com/index.html    }
{                                                                         }
{  There are no known bugs at the moment, so please notify me when you    }
{  find one. Make sure you are using the latest version of this software. }
{  email: erwind@high-logic.com                                           }
{                                                                         }
{  DISCLAIMER                                                             }
{  In no event will the author be held liable for any direct or indirect  }
{  damage resulting from the use of this software or any derivatives      }
{  thereof.                                                               }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{ Version 1.0 17-01-1999                                                  }
{ - First release, works with Delphi 2, 3 and 4 and with C++Builder 3     }
{                                                                         }
{*************************************************************************}

unit HlcRegionInline;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, DsgnIntf, HlcRegisterAbout, HlcResourceInline;

const
  TheConstName : string[63] = 'hlcRegionInline v1.0 - Copyright (c) 1997, 1999 High-Logic   ';

type

  TTimerAction = (taDestroy, taRecreateWnd);

  TTimerActions = set of TTimerAction;

  ThlcRegionInline = class;

  ThlcRegionInline = class(ThlcResourceInline)
  protected
    procedure ResourceChanged(Sender: TObject); override;
  private
    { Private declarations }
    MyTimer:TTimer;
    MyTimerActions: TTimerActions;
    FActive : Boolean;
    FRegionControl : TWinControl;
    FLeftIndent : integer;
    FTopIndent : integer;
    FAutoCreateRegionBorder,
    FIncludeBorderIndent : boolean;
    FHandle: HWND;
    Form: TForm;
    FClientInstance: TFarProc;
    FDefClientProc: TFarProc;
    procedure SetActive(Value: Boolean);
    procedure SetRegionControl(Value: TWinControl);
    procedure SetLeftIndent(Value: integer);
    procedure SetTopIndent(Value: integer);
    procedure SetIncludeBorderIndent(Value: Boolean);
    procedure SetAutoCreateRegionBorder(Value: Boolean);
    procedure ClientWndProc(var Message: TMessage);
    procedure Attach;
    procedure Detach;
    function SetTheRegion:boolean;
    function RestoreRegion:boolean;
    procedure HandleChanged(Sender: TObject);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  protected
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  published
    property Active: Boolean read FActive write SetActive;
    property LeftIndent: integer read FLeftIndent write SetLeftIndent;
    property TopIndent: integer read FTopIndent write SetTopIndent;
    property IncludeBorderIndent: Boolean read FIncludeBorderIndent write SetIncludeBorderIndent;
    property AutoCreateRegionBorder: Boolean read FAutoCreateRegionBorder write SetAutoCreateRegionBorder;
    property RegionControl: TWinControl read FRegionControl write SetRegionControl;
  end;

  EResourceEmpty = class(Exception);

implementation

procedure ThlcRegionInline.HandleChanged(Sender: TObject);
begin
  MyTimer.Enabled:=False;
  if taRecreateWnd in MyTimerActions then
  begin
    MyTimerActions:=MyTimerActions-[taRecreateWnd];
    if Active then
    begin
      if Resource.MemoryStream.Size>0
      then SetTheRegion
      else Active:=False;
    end;
  end;
  if taDestroy in MyTimerActions then
  begin
    MyTimerActions:=MyTimerActions-[taDestroy];
    Free;
  end;
end;

procedure ThlcRegionInline.ResourceChanged(Sender: TObject);
begin
  inherited;
  if Active then
  begin
    if Resource.MemoryStream.Size>0
    then SetTheRegion
    else Active:=False;
  end;
end;

procedure ThlcRegionInline.Attach;
begin
  if not (Owner is TForm)
  then raise Exception.Create('Can''t attach ThlcRegionInline component to something else than a TForm.');
  if not Assigned(FClientInstance) then // only attach once!
  begin
    if (Owner as TForm).FormStyle = fsMDIForm
    then FHandle := (Owner as TForm).ClientHandle
    else FHandle := (Owner as TForm).Handle;
    FClientInstance := MakeObjectInstance(ClientWndProc);
    FDefClientProc := Pointer(GetWindowLong(FHandle, GWL_WNDPROC));
    SetWindowLong(FHandle, GWL_WNDPROC, LongInt(FClientInstance));
    Form := Owner as TForm;
  end;
end;

procedure ThlcRegionInline.Detach;
begin
  if Assigned(FClientInstance) then // if attached
  begin
    SetWindowLong(FHandle, GWL_WNDPROC, LongInt(FDefClientProc));
    FreeObjectInstance(FClientInstance);
  end;
  FClientInstance := nil;
end;

procedure ThlcRegionInline.ClientWndProc(var Message: TMessage);

  procedure Default;
  begin
    with Message
    do Result := CallWindowProc(FDefClientProc, FHandle, Msg, wParam, lParam);
  end;

begin
  with Message do
  begin
    case Msg of
      WM_SIZE :
        begin
          Default;
          SetTheRegion;
        end;
      WM_DESTROY :
        begin
          Detach;
          Default;
        if (csDestroying in ComponentState) or (csDesigning in ComponentState) then
        begin
        end
        else
        begin
          if not Application.Terminated then
          begin
            MyTimerActions:=MyTimerActions+[taRecreateWnd];
            MyTimer.Enabled:=True;
          end;
        end;
      end;
      else Default;
    end;
  end;
end;

destructor ThlcRegionInline.Destroy;
begin
  Detach;
  if Active
  then Active:=False;
  if Assigned(MyTimer)
  then MyTimer.Free;
  inherited;
end;

constructor ThlcRegionInline.Create(AOwner: TComponent);
begin
  if not (AOwner is TForm) then exit;
  inherited Create(AOwner);
  FActive:=False;
  FRegionControl:=nil;
  FLeftIndent:=0;
  FTopIndent:=0;
  FIncludeBorderIndent:=True;
  FAutoCreateRegionBorder:=True;
  MyTimer:=TTimer.Create(nil);
  MyTimer.OnTimer:=HandleChanged;
  MyTimer.Enabled:=False;
  MyTimer.Interval:=1;
end;

function ThlcRegionInline.SetTheRegion:boolean;
var
  Rgn, Rgn1, Rgn2: HRgn;
  TheRect : TRect;
  ThePoint : TPoint;
begin
  result:=false;
  if csLoading in ComponentState
  then Exit;
  if Resource.MemoryStream.Memory<>nil then
  begin
    Rgn:=ExtCreateRegion(nil, Resource.MemoryStream.Size, TRgnData(Resource.MemoryStream.Memory^));
    OffsetRgn(Rgn, LeftIndent, TopIndent);
    if RegionControl=nil then
    begin
      GetWindowRect(TForm(Owner).Handle, TheRect);
      ThePoint:=TForm(Owner).ScreenToClient(Point(TheRect.Left, TheRect.Top));
      if IncludeBorderIndent then
      begin
        OffsetRgn(Rgn, -ThePoint.x, -ThePoint.y);
      end;
      if AutoCreateRegionBorder then
      begin
        Rgn1:=CreateRectRgn(0, 0, TForm(Owner).Width, TForm(Owner).Height);
        Rgn2:=CreateRectRgn(-ThePoint.x, -ThePoint.y, -ThePoint.x+TForm(Owner).ClientWidth, -ThePoint.y+TForm(Owner).ClientHeight);
        CombineRgn(Rgn1, Rgn1, Rgn2, RGN_DIFF);
        CombineRgn(Rgn,Rgn,Rgn1,RGN_OR);
      end;
      result:=boolean(boolean(SetWindowRgn(TForm(Owner).Handle, Rgn, True)));
      Attach;
    end
    else
    begin
     Rgn1:=CreateRectRgn(0, 0, TWinControl(RegionControl).Width, TWinControl(RegionControl).Height);
     CombineRgn(Rgn,Rgn,Rgn1,RGN_AND);
     DeleteObject(Rgn1);
     result:=boolean(SetWindowRgn(TWinControl(RegionControl).Handle, Rgn, True));
    end;
  end;
end;

function ThlcRegionInline.RestoreRegion:boolean;
begin
  if RegionControl=nil then
  begin
    if Assigned(TForm(Owner))
    then result:=boolean(SetWindowRgn(TForm(Owner).Handle, 0, True))
    else result:=False;
  end
  else
  begin
    result:=boolean(SetWindowRgn(TWinControl(RegionControl).Handle, 0, True));
  end;
end;

procedure ThlcRegionInline.SetActive(Value: Boolean);
begin
  if csLoading in ComponentState then
  begin
    FActive:=Value;
    Exit;
  end;
  if Value and (Resource.MemoryStream.Size=0) then
  begin
    raise EResourceEmpty.Create('No Resource Loaded.');
    exit;
  end;
  begin
    FActive:=Value;
    if Value then
    begin
      if Resource.MemoryStream.Size>0
      then SetTheRegion
      else
      FActive:=False;
    end
    else RestoreRegion;
  end;
end;

procedure ThlcRegionInline.SetRegionControl(Value: TWinControl);
begin
  if FRegionControl<>Value then
  begin
    RestoreRegion;
    FRegionControl:=Value;
    if Active
    then SetTheRegion;
  end;
end;

procedure ThlcRegionInline.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FRegionControl) then
  begin
    FRegionControl := nil;
    FActive:=False;
  end;
end;

procedure ThlcRegionInline.Loaded;
begin
  inherited Loaded;
  if Active
  then SetTheRegion;
end;

procedure ThlcRegionInline.SetLeftIndent(Value: integer);
begin
  if FLeftIndent<>Value then
  begin
    FLeftIndent:=Value;
    if Active
    then SetTheRegion;
  end;
end;

procedure ThlcRegionInline.SetTopIndent(Value: integer);
begin
  if FTopIndent<>Value then
  begin
    FTopIndent:=Value;
    if Active
    then SetTheRegion;
  end;
end;

procedure ThlcRegionInline.SetIncludeBorderIndent(Value: Boolean);
begin
  if FIncludeBorderIndent<>Value then
  begin
    FIncludeBorderIndent:=Value;
    if Active and (FRegionControl=nil) // Only applies with Form control.
    then SetTheRegion;
  end;
end;

procedure ThlcRegionInline.SetAutoCreateRegionBorder(Value: Boolean);
begin
  if FAutoCreateRegionBorder<>Value then
  begin
    FAutoCreateRegionBorder:=Value;
    if Active and (FRegionControl=nil) // Only applies with Form control.
    then SetTheRegion;
  end;
end;

end.
