{-$D}
{*************************************************************************}
{                                                                         }
{                    HlcResourceInline Component                          }
{                                                                         }
{  The HlcResourceInline stores files inside your EXE without resources!  }
{  This way you can make a one file install program or embedd fonts, text }
{  or wav-files into your program. You can use the included property      }
{  editor at designtime.                                                  }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{  Copyright (c) 1997, 1999 High-Logic [Erwin Denissen]                   }
{                                                                         }
{  You may use a Shareware Copy of this Software for an evaluation period }
{  of up to 30 days, in order to determine whether the Software meets     }
{  your needs before purchasing it. Once the evaluation period ends, you  }
{  agree to either purchase a copy of the software, or to stop using it   }
{  and remove it from your computer system.                               }
{                                                                         }
{  A registered version of this and other components and software can be  }
{  ordered at the following site: http://www.high-logic.com/index.html    }
{                                                                         }
{  There are no known bugs at the moment, so please notify me when you    }
{  find one. Make sure you are using the latest version of this software. }
{  email: erwind@high-logic.com                                           }
{                                                                         }
{  DISCLAIMER                                                             }
{  In no event will the author be held liable for any direct or indirect  }
{  damage resulting from the use of this software or any derivatives      }
{  thereof.                                                               }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{ Version 1.0 21-12-1998                                                  }
{ - First release, works with Delphi 2, 3 and 4 and with C++Builder 3     }
{                                                                         }
{*************************************************************************}

unit HlcResourceInline;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, DsgnIntf, HlcRegisterAbout;

const
  TheConstName : string[63] = 'hlcResourceInline v1.0 - Copyright (c) 1997, 1999 High-Logic   ';
  SubString = 'High-Logic';  // Must be a substring of TheConstName!
type
  TResourceEditorDlg = class(TForm)
    btnLoad: TButton;
    btnSave: TButton;
    btnClear: TButton;
    btnOk: TButton;
    btnCancel: TButton;
    Bevel1: TBevel;
    MemoHex: TMemo;
    lblSize: TLabel;
    procedure btnLoadClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    NewMemoryStream : TMemoryStream;
    OldMemoryStream : TMemoryStream;
    FShowPreview : boolean;
    UseNew : boolean;
    procedure SetMemoryStream(Value : TMemoryStream);
    function GetMemoryStream: TMemoryStream;
    procedure UpdateSettings;
    procedure ShowHex(TheStream : TCustomMemoryStream);
  public
    { Public declarations }
    property TheResource: TMemoryStream read GetMemoryStream write SetMemoryStream;
  end;

  ThlcResourceProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

  { Component editor - brings up resource editor when double clicking on
    resource property }
  ThlcResourceInlineEditor = class(TDefaultEditor)
  protected
    procedure EditProperty(PropertyEditor: TPropertyEditor;
      var Continue, FreeEditor: Boolean); override;
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;


  ThlcResource = class
  private
    FOnChange: TNotifyEvent;
  public
    MemoryStream:TMemoryStream;
    constructor Create;
    destructor Destroy; override;
  published
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  ThlcResourceInline = class(TComponent)
  protected
    procedure ResourceChanged(Sender: TObject); virtual;
  private
    { Private declarations }
    FAbout : ThlcAbout;
    FhlcResource: ThlcResource;
    procedure SethlcResource(Value: ThlcResource);
    procedure ReadData(Stream: TStream);
    procedure WriteData(Stream: TStream);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DefineProperties(Filer: TFiler); override;
  published
    property About : ThlcAbout read FAbout write FAbout;
    property Resource: ThlcResource read FhlcResource write SethlcResource;
  end;

implementation

{$R *.DFM}

{ ThlcResource }

destructor ThlcResource.Destroy;
begin
  if Assigned(MemoryStream)
  then MemoryStream.Free;
  inherited;
end;

constructor ThlcResource.Create;
begin
  inherited Create;
  MemoryStream:=TMemoryStream.Create;
end;

{ TResourceEditorDlg }

procedure TResourceEditorDlg.btnSaveClick(Sender: TObject);
var
  TheSaveDialog : TSaveDialog;
begin
  TheSaveDialog :=TSaveDialog.Create(Application);
  try
    TheSaveDialog.Filter:='All Files (*.*)|*.*|Region Files (*.rgn)|(*.rgn)';
    if TheSaveDialog.Execute
    then TheResource.SaveToFile(TheSaveDialog.FileName);
  finally
    TheSaveDialog.Free;
  end;
end;

procedure TResourceEditorDlg.FormCreate(Sender: TObject);
begin
  NewMemoryStream:=TMemoryStream.Create;
  UseNew:=False;
  FShowPreview:=True;
end;

procedure TResourceEditorDlg.FormDestroy(Sender: TObject);
begin
  NewMemoryStream.Free;
  NewMemoryStream:=nil;
end;

procedure TResourceEditorDlg.btnOkClick(Sender: TObject);
begin
  if UseNew then
  begin
    if NewMemoryStream.Size=0
    then OldMemoryStream.SetSize(0)
    else
    begin
      OldMemoryStream.SetSize(NewMemoryStream.Size);
      OldMemoryStream.Position:=0;
      OldMemoryStream.CopyFrom(NewMemoryStream, 0);
    end;
  end
  else
  ModalResult:=mrCancel;
end;

procedure TResourceEditorDlg.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#27
  then Close;
end;

procedure TResourceEditorDlg.ShowHex(TheStream : TCustomMemoryStream);
const
  breed=16;
  aantal=1;
  MaxPreviewSize=1024*4;
var
  MyCurrentBufSize, i, teller, veld : integer;
  TheText, TheChars : string;
  MyBuffer : pchar;

function AlignWithSpaces(TheText : string; breed:integer):string;
begin
  result:=TheText;
  while length(result)<breed do result:=result+' ';
end;

begin
  MemoHex.Lines.BeginUpdate;
  MemoHex.Lines.Clear;
  MyBuffer:=pchar(TheStream.Memory);
  MyCurrentBufSize:=TheStream.Size;
  if MyCurrentBufSize>MaxPreviewSize
  then MyCurrentBufSize:=MaxPreviewSize;
  if MyCurrentBufSize>0 then
  for i:=0 to (MyCurrentBufSize div (breed*aantal)) do
  if i*breed*aantal<MyCurrentBufSize then
  begin
    TheText:=Format('%4.4x', [i*breed*aantal]);
    for teller:=0 to aantal-1 do
    begin
      if i*breed*aantal+teller*breed<MyCurrentBufSize then
      begin
        TheText:=TheText+' ';
        TheChars:='';
        for veld:=0 to breed-1 do
        begin
          if i*breed*aantal+teller*breed+veld<MyCurrentBufSize then
          begin
            TheText:=TheText+Format('%2.2x',[byte(MyBuffer[i*breed*aantal+teller*breed+veld])]);
            if veld=7
            then TheText:=TheText+' ';
            if char(MyBuffer[i*breed*aantal+teller*breed+veld])>=' '
            then TheChars:=TheChars+char(MyBuffer[i*breed*aantal+teller*breed+veld])
            else TheChars:=TheChars+' ';
          end
          else
          begin
            TheText:=TheText+' ';
            TheChars:=TheChars+' ';
          end;
        end;
      end
      else
      begin
        TheText:=TheText+'  ';
        TheChars:='';
      end;
      TheText:=AlignWithSpaces(TheText, 2*breed+6);
      TheText:=TheText+' '+TheChars;
    end;
    MemoHex.Lines.Add(TheText);
  end;
  if TheStream.Size>0 then
  begin
    if TheStream.Size>MaxPreviewSize then
    begin
      MemoHex.Lines.Add('');
      MemoHex.Lines.Add('Only the first '+IntToStr(MaxPreviewSize)+' bytes are visible in this preview.');
      MemoHex.Lines.Add('However the full resource ('+IntToStr(TheStream.Size)+' bytes) has been loaded.');
    end;
    MemoHex.Lines.Add('');
    MemoHex.Lines.Add('(c) 1998, 1999 High-Logic');
    MemoHex.Lines.Add('Please register.');
  end
  else
  begin
    MemoHex.Lines.Add('(Empty)');
  end;
  MemoHex.SelStart:=0;
  MemoHex.Lines.EndUpdate;
end;

procedure TResourceEditorDlg.UpdateSettings;
begin
  if not Assigned(TheResource) then
  begin
    exit;
  end;
  if TheResource.Size=0
  then lblSize.Caption:='No resource loaded.'
  else lblSize.Caption:='Resource size '+IntToStr(TheResource.Size)+' bytes.';
  btnSave.Enabled:=TheResource.Size>0;
  btnClear.Enabled:=TheResource.Size>0;
  ShowHex(TheResource);
end;

function TResourceEditorDlg.GetMemoryStream: TMemoryStream;
begin
  if UseNew
  then result:=NewMemoryStream
  else result:=OldMemoryStream;
end;

procedure TResourceEditorDlg.SetMemoryStream(Value : TMemoryStream);
begin
  OldMemoryStream:=Value;
  UpdateSettings;
end;

procedure TResourceEditorDlg.btnLoadClick(Sender: TObject);
var
  TheOpenDialog : TOpenDialog;
begin
  TheOpenDialog :=TOpenDialog.Create(Application);
  try
    TheOpenDialog.Filter:='All Files (*.*)|*.*|Region Files (*.rgn)|(*.rgn)';
    if TheOpenDialog.Execute then
    begin
      NewMemoryStream.LoadFromFile(TheOpenDialog.FileName);
      UseNew:=True;
      UpdateSettings;
    end;
  finally
    TheOpenDialog.Free;
  end;
end;

procedure TResourceEditorDlg.btnClearClick(Sender: TObject);
begin
  if UseNew
  then NewMemoryStream.SetSize(0)
  else UseNew:=True;
  UpdateSettings;
end;

{ ThlcResourceProperty }

procedure ThlcResourceProperty.Edit;
var
  tmphlcResource: ThlcResource;
  tmpResource: TMemoryStream;
  ResourceEditor: TResourceEditorDlg;
begin
  tmphlcResource := ThlcResource(GetOrdValue);
  tmpResource:=tmphlcResource.MemoryStream;
  ResourceEditor := TResourceEditorDlg.Create(Application);
  try
    ResourceEditor.TheResource := tmpResource;
    if ResourceEditor.ShowModal=mrOk
    then tmphlcResource.OnChange(Self);
  finally
    ResourceEditor.Free;
    Modified;
  end;
end;

function ThlcResourceProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog] - [paSubProperties];
end;

function ThlcResourceProperty.GetValue: string;
var
  TheResource: ThlcResource;
begin                    
  TheResource := ThlcResource(GetOrdValue);
  if TheResource.MemoryStream.Size>0
    then Result := '(TResource)'
    else Result := '(Empty)';
end;

{ ThlcResourceInline }               

destructor ThlcResourceInline.Destroy;
begin
  if Assigned(Resource)
  then Resource.Free;
  inherited;
end;

constructor ThlcResourceInline.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FhlcResource:=ThlcResource.Create;
  FhlcResource.OnChange := ResourceChanged;
end;

procedure ThlcResourceInline.ResourceChanged(Sender: TObject);
begin
end;

procedure ThlcResourceInline.SethlcResource(Value: ThlcResource);
begin
  FhlcResource.MemoryStream.CopyFrom(Value.MemoryStream, 0);
end;

procedure ThlcResourceInline.ReadData(Stream: TStream);
const
  ErrorString ='Corrupt resource data.';
var
  ErrStr, CName: string[63];
  StreamIsCorrect: boolean;
begin
  StreamIsCorrect:=True;
  if Stream.Size=0
  then StreamIsCorrect:=False
  else
  begin
    Stream.Read(CName[0], 1);
    if Stream.Size<Integer(CName[0])
    then StreamIsCorrect:=False
    else
    begin
      Stream.Read(CName[1], Integer(CName[0]));
      if Pos(SubString, CName)=0
      then StreamIsCorrect:=False
      else
      begin
        if Stream.Size<>Stream.Position then
        begin
          Resource.MemoryStream.CopyFrom(Stream, Stream.Size-Stream.Position);
          Resource.MemoryStream.Position:=0;
        end;
      end;
    end;
  end;
  if not StreamIsCorrect then
  begin
    ErrStr:=ErrorString;
    Resource.MemoryStream.SetSize(Length(ErrStr));
    Resource.MemoryStream.Position:=0;
    Resource.MemoryStream.Write(ErrStr[1], Length(ErrStr));
    Resource.MemoryStream.Position:=0;
  end;
end;

procedure ThlcResourceInline.WriteData(Stream: TStream);
var
  CName: string[63];
begin
  with Stream do
  begin
    CName := TheConstName;
    Write(CName, Length(CName) + 1);
    Resource.MemoryStream.SaveToStream(Stream);
  end;
end;

procedure ThlcResourceInline.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineBinaryProperty('Data', ReadData, WriteData, True{ResData<>nil});
end;

{ ThlcResourceInlineEditor }

procedure ThlcResourceInlineEditor.EditProperty(PropertyEditor: TPropertyEditor;
  var Continue, FreeEditor: Boolean);
var
  PropName: string;
begin
  PropName := PropertyEditor.GetName;
  if (CompareText(PropName, 'RESOURCE') = 0) then
  begin
    PropertyEditor.Edit;
    Continue := False;
  end;
end;

function ThlcResourceInlineEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function ThlcResourceInlineEditor.GetVerb(Index: Integer): string;
begin
  if Index = 0 then
    Result := 'Edit Resource...'
  else Result := '';
end;

procedure ThlcResourceInlineEditor.ExecuteVerb(Index: Integer);
begin
  if Index = 0 then
  begin
    Edit;
  end;
end;

end.
