unit HlcRuler;

interface

uses
  classes, extctrls, Windows, graphics, Controls, SysUtils, Math,
  HlcRegisterAbout;

type

{$IFDEF VER90} // Delphi 2
  {$DEFINE DELPHI2OR3}
{$ENDIF}
{$IFDEF VER100} // Delphi 3
  {$DEFINE DELPHI2OR3}
{$ENDIF}

{$IFDEF DELPHI2OR3} // Delphi 2 or 3
  INT64 = integer;
{$ENDIF}

  TRulerKind =
    (rkLeft, rkRight, rkTop, rkBottom);
  TPlacement =
    (tpNone, tpAbove, tpBelow, tpBoth, tpMiddle);
  TNumberOfUnitFractionMarkers =
    (numOne, numTwo, numFour, numEight, numTen, numSixteen, numTwenty);

  // THRuler derived from TCustomControl because it is an original windowed
  // control that does not correspond to standard Windows components
  ThlcRuler = class (TCustomControl)
  private
    FFlat: boolean;
    FCaption: string;
    FColor: TColor;
    FFont: TFont;
    FKind: TRulerKind;
    FMinUnitStepSize: Word;
    FShortLineSize: Word;
    FLargeLineSize: Word;
    FNumDecimals: Word;
    FDocumentVisualEnd: Integer;
    FDocumentVisualStart: Integer;
    FRulerSize: Word;
    FRulerOffset: Word;
    FDocumentScreenEnd: Integer;
    FDocumentScreenStart: Integer;
    FDocumentMarginAfter: Integer;
    FDocumentMarginBefore: Integer;
    FMargeColor: TColor;
    FDocumentWorldEnd: double;
    FDocumentWorldStart: double;
    FDividerPlacement: TPlacement;
    FFontOffsetVertical: integer;
    FFontOffsetHorizontal: integer;
    FMiddleLineSize: Word;
    FLineColor: TColor;
    FNumberOfUnitFractionMarkers: TNumberOfUnitFractionMarkers;
    FWindowsColor: boolean;
    FShowMargins: boolean;
    FAbout: THLCAbout;
    function GetTotalBetween: integer;
    procedure RulerTextOut(TheCanvas: TCanvas; XPos, YPos: integer;
      TheText: string);
    procedure RulerPolyLine(TheCanvas: TCanvas; StartPoint, EndPoint: TPoint);
    procedure SetWindowsColor(const Value: boolean);
    procedure SetShowMargins(const Value: boolean);
    property TotalBetween: integer read GetTotalBetween;
    procedure DrawIt;
    procedure SetFont(const Value: TFont);
    procedure SetCaption(const Value: string);
    procedure SetColor(const Value: TColor);
    procedure SetFlat(const Value: boolean);
    procedure SetKind(const Value: TRulerKind);
    procedure SetLargeLineSize(const Value: Word);
    procedure SetMinUnitStepSize(const Value: Word);
    procedure SetNumDecimals(const Value: Word);
    procedure SetShortLineSize(const Value: Word);
    procedure SetDocumentVisualEnd(const Value: Integer);
    procedure SetDocumentVisualStart(const Value: Integer);
    procedure SetRulerOffset(const Value: Word);
    procedure SetRulerSize(const Value: Word);
    procedure SetDocumentScreenEnd(const Value: Integer);
    procedure SetDocumentScreenStart(const Value: Integer);
    procedure SetDocumentMarginAfter(const Value: Integer);
    procedure SetDocumentMarginBefore(const Value: Integer);
    procedure SetMargeColor(const Value: TColor);
    procedure SetDocumentWorldEnd(const Value: double);
    procedure SetDocumentWorldStart(const Value: double);
    function WorldToScreen(Value: double): Int64;
    function ScreenToWorld(Value: integer): double;
    procedure SetDividerPlacement(const Value: TPlacement);
    procedure SetFontOffsetVertical(const Value: integer);
    procedure SetFontOffsetHorizontal(const Value: integer);
    procedure SetMiddleLineSize(const Value: Word);
    procedure SetLineColor(const Value: TColor);
    procedure SetNumberOfUnitFractionMarkers(
      const Value: TNumberOfUnitFractionMarkers);
//    procedure Loaded; override;
    function RulerCreateRectRgn(TheLeft, TheTop, TheRight,
      TheBottom: integer): HRGN;
  protected
//    procedure CreateParams(var Params: TCreateParams); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property About : THLCAbout read FAbout write FAbout;
    // Caption is used only with horizontal rulers
    property Caption : string read FCaption write SetCaption;
    property Color: TColor read FColor write SetColor;
    property Font: TFont read FFont write SetFont;
    // LineSizes used with NumberOfUnitFractionMarkers
    property LargeLineSize: Word read FLargeLineSize write SetLargeLineSize;
    property MiddleLineSize: Word read FMiddleLineSize write SetMiddleLineSize;
    property ShortLineSize: Word read FShortLineSize write SetShortLineSize;
    // Kind specifies if the ruler is a horizontal or a vertical one.
    property Kind: TRulerKind read FKind write SetKind;
    property Flat: boolean read FFlat write SetFlat;
    property NumDecimals: Word read FNumDecimals write SetNumDecimals;
    property MinUnitStepSize: Word read FMinUnitStepSize write SetMinUnitStepSize;
    // RulerOffset: Distance between top of control and the actual ruler
    property RulerOffset: Word read FRulerOffset write SetRulerOffset;
    property RulerSize: Word read FRulerSize write SetRulerSize;
    // DocumentVisual... Positions between the ruler is diplayed
    property DocumentVisualStart: Integer read FDocumentVisualStart write SetDocumentVisualStart;
    property DocumentVisualEnd: Integer read FDocumentVisualEnd write SetDocumentVisualEnd;
    // DocumentScreen... internal size of the ruler in screen coordinates
    property DocumentScreenStart: Integer read FDocumentScreenStart write SetDocumentScreenStart;
    property DocumentScreenEnd: Integer read FDocumentScreenEnd write SetDocumentScreenEnd;
    // DocumentWorld... internal size of the ruler in world coordinates
    property DocumentWorldStart: double read FDocumentWorldStart write SetDocumentWorldStart;
    property DocumentWorldEnd: double read FDocumentWorldEnd write SetDocumentWorldEnd;
    // DocumentMargin... Positions were the margins start
    property DocumentMarginBefore: Integer read FDocumentMarginBefore write SetDocumentMarginBefore;
    property DocumentMarginAfter: Integer read FDocumentMarginAfter write SetDocumentMarginAfter;
    // MargeColor Color of the margins when flat is true
    property MargeColor: TColor read FMargeColor write SetMargeColor;
    // DividerPlacement specifies were the lines and labels are drawn
    property DividerPlacement: TPlacement read FDividerPlacement write SetDividerPlacement;
    // With FontOffset... you can adjust the position of the text in the ruler
    property FontOffsetVertical: integer read FFontOffsetVertical write SetFontOffsetVertical;
    property FontOffsetHorizontal: integer read FFontOffsetHorizontal write SetFontOffsetHorizontal;
    // LineColor Color of the lines
    property LineColor: TColor read FLineColor write SetLineColor;
    // NumberOfUnitFractionMarkers specifies the number of lines between each label including the line at the label
    property NumberOfUnitFractionMarkers: TNumberOfUnitFractionMarkers read FNumberOfUnitFractionMarkers write SetNumberOfUnitFractionMarkers;
    // WindowsColor specifies wheater to use the clWindows color or the controls color
    property WindowsColor: boolean read FWindowsColor write SetWindowsColor;
    // ShowMargins specifies wheater to show the margins or not
    property ShowMargins: boolean read FShowMargins write SetShowMargins;
    // Properties inherited from derived component:
    property Enabled;
    property Align;
  end;
                               
implementation

{ ThlcRuler }

constructor ThlcRuler.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
//  ControlStyle:=ControlStyle + [csOpaque];
  Width := 360;
  Height := 25;
  Caption := 'units';
  Color := clBtnFace;
  LargeLineSize := 5;
  MiddleLineSize := 6;
  ShortLineSize := 3;
  FKind := rkTop;
  Flat := False;
  MinUnitStepSize := 1;
  RulerOffset := 2;
  RulerSize := 20;
  DocumentVisualStart := 10;
  DocumentVisualEnd := 340;
  DocumentScreenStart := 30;
  DocumentScreenEnd := 330;
  DocumentWorldEnd := 1000;
  DocumentMarginBefore := 50;
  DocumentMarginAfter := 300;
  MargeColor := clBtnShadow;
  DividerPlacement := tpMiddle;
  NumberOfUnitFractionMarkers:=numFour;
  FontOffsetVertical := 0;
  FontOffsetHorizontal := 0;
  LineColor := clBlack;
  // A font is a component that must be created
  FFont:=TFont.Create;
  FFont.Name := 'Times New Roman';
  FFont.Size := 7;
  WindowsColor := True;
  ShowMargins := True;
end;

destructor ThlcRuler.Destroy;
begin
  // Free the font object
  FFont.Free;
  // After we finished our stuff call the inherited Destroy;
  inherited Destroy;
end;

function BestFit(TheMaxDigitsLengthWorld: Int64): Int64;
// Calculates the best value for the MaxDigitsLengthWorld.
var
  Multiplier, MaxDigitsLengthWorld: Int64;
begin
  MaxDigitsLengthWorld:=TheMaxDigitsLengthWorld;
  if MaxDigitsLengthWorld>10 then
  begin
    Multiplier:=1;
    while MaxDigitsLengthWorld>100 do
    begin
      Multiplier:=Multiplier*10;
      MaxDigitsLengthWorld:=MaxDigitsLengthWorld div 10;
    end;
    if MaxDigitsLengthWorld>50 then
    MaxDigitsLengthWorld:=100
    else
    if MaxDigitsLengthWorld>25 then
    MaxDigitsLengthWorld:=50
    else
    if MaxDigitsLengthWorld>20 then
    MaxDigitsLengthWorld:=25
    else
    if MaxDigitsLengthWorld>15 then
    MaxDigitsLengthWorld:=20
    else
    if MaxDigitsLengthWorld>10 then
    MaxDigitsLengthWorld:=15
    else
    MaxDigitsLengthWorld:=10;
    MaxDigitsLengthWorld:=MaxDigitsLengthWorld*Multiplier;
  end
  else
  if MaxDigitsLengthWorld>5 then
  MaxDigitsLengthWorld:=10
  else
  if MaxDigitsLengthWorld>2 then
  MaxDigitsLengthWorld:=5
  else
  if MaxDigitsLengthWorld>1 then
  MaxDigitsLengthWorld:=2;
  result:=MaxDigitsLengthWorld;
end;

procedure ThlcRuler.RulerTextOut(TheCanvas: TCanvas; XPos, YPos: integer; TheText: string);
// Function that encapsulated the TextOut function.
// It draws text rotated when kind is vertical
var
  MyFont: HFont;
  PreviousFont: TFont;
  ExtraOffsetX, ExtraOffsetY: integer;
  TheRotation,
  TheCharSet,
  ThePitchAndFamily,
  TheWeight,
  IsItalic, IsUnderlined, IsStrikeOut: Cardinal;
begin
  case Kind of
    rkTop, rkBottom :
    begin
      TheCanvas.TextOut(Xpos, YPos, TheText);
    end;
    else
    begin
      // rotate the text
      if Kind=rkLeft then
      begin
        TheRotation:=900;
      end
      else
      begin
        TheRotation:=2700;
      end;
      if fsBold in TheCanvas.Font.Style
      then TheWeight:=FW_BOLD
      else TheWeight:=FW_NORMAL;
      if fsItalic in TheCanvas.Font.Style
      then IsItalic:=1
      else IsItalic:=0;
      if fsUnderline in TheCanvas.Font.Style
      then IsUnderlined:=1
      else IsUnderlined:=0;
      if fsStrikeOut in TheCanvas.Font.Style
      then IsStrikeOut:=1
      else IsStrikeOut:=0;
      ThePitchAndFamily:=FF_DONTCARE;
      if TheCanvas.Font.Pitch=fpDefault
      then ThePitchAndFamily:=ThePitchAndFamily or DEFAULT_PITCH;
      if TheCanvas.Font.Pitch=fpFixed
      then ThePitchAndFamily:=ThePitchAndFamily or FIXED_PITCH;
      if TheCanvas.Font.Pitch=fpVariable
      then ThePitchAndFamily:=ThePitchAndFamily or VARIABLE_PITCH;
      TheCharSet:=TheCanvas.Font.Charset;
      MyFont:=CreateFont(TheCanvas.Font.Height, 0, TheRotation, TheRotation, TheWeight, IsItalic, IsUnderlined, IsStrikeOut, TheCharSet, OUT_TT_ONLY_PRECIS, CLIP_DEFAULT_PRECIS, PROOF_QUALITY, ThePitchAndFamily, PChar(TheCanvas.Font.Name));
      try
        PreviousFont:=TheCanvas.Font;
        TheCanvas.Font.Handle:=MyFont;
        TheCanvas.Font.Color:=PreviousFont.Color;

        if Kind=rkLeft then
        begin
          ExtraOffsetX:=0;
          ExtraOffsetY:=TheCanvas.TextWidth(TheText);
        end
        else
        begin
          ExtraOffsetX:=TheCanvas.TextHeight(TheText);
          ExtraOffsetY:=0;
        end;

        TheCanvas.TextOut(Ypos+ExtraOffsetX, XPos+ExtraOffsetY, TheText);
        TheCanvas.Font:=PreviousFont;
      finally
        DeleteObject(MyFont);
      end;
    end;
  end;
end;

{$IFNDEF VER90} // not Delphi 2
  {$IFNDEF VER100} // not Delphi 3
  {$ELSE}
function Min(A, B: Integer): Integer;
begin
  if A < B
  then Result := A
  else Result := B;
end;

function Max(A, B: Integer): Integer;
begin
  if A > B
  then Result := A
  else Result := B;
end;
  {$ENDIF}
{$ENDIF}

procedure ThlcRuler.RulerPolyLine(TheCanvas: TCanvas; StartPoint, EndPoint: TPoint);
// Function that encapsulated the PolyLine function.
// It swaps x and y when kind is vertical
var
  tmpStart, tmpEnd: TPoint;
begin
  if (StartPoint.x=EndPoint.x) and (StartPoint.y=EndPoint.y)
  then exit;
  case Kind of
    rkTop, rkBottom :
    begin
      tmpStart.x := Min(startPoint.x, EndPoint.x);
      tmpStart.y := Min(startPoint.y, EndPoint.y);
      tmpEnd.x := Max(startPoint.x, EndPoint.x);
      tmpEnd.y := Max(startPoint.y, EndPoint.y);
//      TheCanvas.PolyLine([StartPoint, EndPoint]);
//      TheCanvas.MoveTo(StartPoint.x, StartPoint.y);
//      TheCanvas.LineTo(EndPoint.x, EndPoint.y);
//      TheCanvas.MoveTo(EndPoint.x, EndPoint.y);
//      TheCanvas.LineTo(StartPoint.x, StartPoint.y);
    end;
    else
    begin
//      TheCanvas.PolyLine([Point(StartPoint.y, StartPoint.x), Point(EndPoint.y, EndPoint.x)]);
//      TheCanvas.PolyLine([Point(EndPoint.y, EndPoint.x), Point(StartPoint.y, StartPoint.x)]);
      tmpStart.x := Min(startPoint.y, EndPoint.y);
      tmpStart.y := Min(startPoint.x, EndPoint.x);
      tmpEnd.y := Max(startPoint.x, EndPoint.x);
      tmpEnd.x := Max(startPoint.y, EndPoint.y);

      // Fix a F!@#^$@king Windows Bug
      if tmpStart.y = tmpEnd.y
      then tmpEnd.x := tmpEnd.x - 1;

      if tmpStart.x = tmpEnd.x
      then tmpEnd.y := tmpEnd.y + 1;


{
      if StartPoint.x=EndPoint.x then
      begin
        TheCanvas.MoveTo(StartPoint.y, StartPoint.x);
        TheCanvas.LineTo(EndPoint.y-1, EndPoint.x);
      end
      else
      begin
        if StartPoint.y=EndPoint.y then
        begin
          TheCanvas.MoveTo(StartPoint.y, StartPoint.x);
          TheCanvas.LineTo(EndPoint.y, EndPoint.x);
        end
        else
        begin
          TheCanvas.MoveTo(StartPoint.y, StartPoint.x);
          TheCanvas.LineTo(EndPoint.y, EndPoint.x);
        end;

      end;
//      TheCanvas.MoveTo(EndPoint.y, EndPoint.x);
//      TheCanvas.LineTo(StartPoint.y, StartPoint.x);
}
    end;
  end;
  TheCanvas.MoveTo(tmpStart.x, tmpStart.y);
  TheCanvas.LineTo(tmpEnd.x, tmpEnd.y);
end;

function ThlcRuler.RulerCreateRectRgn(TheLeft, TheTop, TheRight, TheBottom: integer):HRGN;
begin
  case Kind of
    rkTop, rkBottom :
    begin
      result:=CreateRectRgn(TheLeft, TheTop, TheRight, TheBottom);
    end;
    else
    begin
      result:=CreateRectRgn(TheTop, TheLeft, TheBottom, TheRight);
    end;
  end;
end;

procedure ThlcRuler.DrawIt;
// The actual function that draws the ruler
var
  WorldValueStart, WorldValueEnd: double;
  MultiplyFactor, iCount, MaxDigitsLengthWorld : Int64;
  Between, TheMaxLength,
  MaxDigitsLength,
  iTeller, CurrentLineSize, AboveRulerOffset, BelowRulerOffset, MiddleRulerOffset: integer;
  DigitOffset, TextOffset, OverallTextHeight, OverallDigitHeight, LinePlace, ScreenValueStepStart, ScreenValueStepEnd, BitBltEnd, BitBltStart, MarginAfterSize: integer;
//  TheNewRulerBitmap: TBitmap;
  TheDocumentRect, ClientArea: TRect;
  MaxDigitSample,
  HulpString : string;
  NumberOfOccupiedParts: integer;
  OccupiedParts: array[1..250] of
  record
    StartPos, EndPos: integer;
  end;
  Occupied: boolean;
  UsedDocumentMarginBefore, UsedDocumentMarginAfter,
  iParts: integer;
  TheRegionHandle: HRGN;
begin
  if ShowMargins then
  begin
    UsedDocumentMarginBefore:=DocumentMarginBefore;
    UsedDocumentMarginAfter:=DocumentMarginAfter;
  end
  else
  begin
    UsedDocumentMarginBefore:=DocumentScreenStart;
    UsedDocumentMarginAfter:=DocumentScreenEnd;
  end;
  // Make sure UsedDocumentMarginAfter is not greater than the visual document part
  if UsedDocumentMarginAfter>DocumentVisualEnd
  then UsedDocumentMarginAfter:=DocumentVisualEnd;
  // Make sure UsedDocumentMarginBefore is not less than the visual document part
  if UsedDocumentMarginBefore<DocumentVisualStart
  then UsedDocumentMarginBefore:=DocumentVisualStart;
  // NumberOfOccupiedParts is used to count the number of used fields in the OccupiedParts array
  NumberOfOccupiedParts:=0;
  ClientArea:=GetClientRect;
  Canvas.Brush.Color := Self.Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.FillRect(ClientArea);
  Canvas.Brush.Style := bsClear;
{  TheNewRulerBitmap:=TBitmap.Create;
  try}
//    TheNewRulerBitmap.Width:=Width;
//    TheNewRulerBitmap.Height:=Height;

    if DocumentVisualStart<DocumentScreenStart
    then BitBltStart:=DocumentScreenStart
    else BitBltStart:=DocumentVisualStart;
    if DocumentVisualEnd>DocumentScreenEnd
    then BitBltEnd:=DocumentScreenEnd
    else BitBltEnd:=DocumentVisualEnd;


    if Flat
    then TheRegionHandle:=RulerCreateRectRgn(BitBltStart+2, RulerOffset+2, BitBltEnd-2, RulerOffset+RulerSize-2)
    else TheRegionHandle:=RulerCreateRectRgn(BitBltStart  , RulerOffset  , BitBltEnd  , RulerOffset+RulerSize  );
    try
      SelectClipRgn(Canvas.Handle, TheRegionHandle);
    finally
      DeleteObject(TheRegionHandle);
    end;

    TheDocumentRect:=ClientArea;
//    with TheNewRulerBitmap do
    begin
(*      // Code om ruler een andere kleur te geven
      Canvas.Brush.Color := clBtnHighLight;
      Canvas.Brush.Style := bsSolid;
      Canvas.FillRect(Rect(0, 0, Width, Height));*)
      // Code om binnenkant schaalverdeling Windows kleur te geven
      if Enabled or (csDesigning in ComponentState) then
      begin
        if WindowsColor
        then Canvas.Brush.Color := clWindow
        else Canvas.Brush.Color := Color;
      end
      else
      begin
        Canvas.Brush.Color := clBtnFace;
      end;

      // Draw scale bar
      case Kind of
        rkTop, rkBottom :
        begin
          if Flat
          then Canvas.FillRect(Rect(UsedDocumentMarginBefore, RulerOffset  , UsedDocumentMarginAfter, RulerOffset+RulerSize))
          else Canvas.FillRect(Rect(UsedDocumentMarginBefore + 1, RulerOffset+2, UsedDocumentMarginAfter - 1, RulerOffset+RulerSize-2));
        end;
        else
        begin
          if Flat
          then Canvas.FillRect(Rect(RulerOffset  , UsedDocumentMarginBefore, RulerOffset+RulerSize  , UsedDocumentMarginAfter))
          else Canvas.FillRect(Rect(RulerOffset+2, UsedDocumentMarginBefore + 1, RulerOffset+RulerSize-2, UsedDocumentMarginAfter - 1));
        end;
      end;

      Canvas.Brush.Style := bsClear;
      // Tekenen van de eenheden
      MultiplyFactor:=1;
      if NumDecimals>0 then
      begin
        for iTeller:=1 to NumDecimals do
        begin
          MultiplyFactor:=MultiplyFactor*10;
        end;
      end;
      WorldValueStart:=MinUnitStepSize*(-1+(round(ScreenToWorld(BitBltStart)*MultiplyFactor) div MinUnitStepSize) {div MultiplyFactor} );
      WorldValueEnd:=MinUnitStepSize*(1+(round(ScreenToWorld(BitBltEnd)*MultiplyFactor) div MinUnitStepSize) {div MultiplyFactor} );
      ScreenValueStepStart:=WorldToScreen(WorldValueStart/ MultiplyFactor);
      ScreenValueStepEnd:=WorldToScreen(WorldValueEnd/ MultiplyFactor);
      begin
        Canvas.Font.Assign(Font);
        // Calculating the maximum distance between the offsets of two consecutive numbers
        if Length(IntToStr(round(DocumentWorldStart)))>Length(IntToStr(round(DocumentWorldEnd)))
        then TheMaxLength:=Length(IntToStr(round(DocumentWorldStart)))
        else TheMaxLength:=Length(IntToStr(round(DocumentWorldEnd)));
        MaxDigitSample:='-,';
        for iTeller:=1 to TheMaxLength + NumDecimals do MaxDigitSample:=MaxDigitSample+'0';
        MaxDigitsLength:=Canvas.TextWidth(MaxDigitSample)+16;
        // Make sure minimum distance is 30 or higher
        if MaxDigitsLength<30
        then MaxDigitsLength:=30;
        OverallTextHeight:=Canvas.TextHeight(Caption);
        OverallDigitHeight:=Canvas.TextHeight('0123456789');
        AboveRulerOffset:=RulerOffSet+2;
        BelowRulerOffset:=RulerOffSet+RulerSize-2;
        MiddleRulerOffset:=RulerOffSet+(RulerSize div 2);
        case DividerPlacement of
          tpAbove:
            begin
              TextOffset:=RulerOffset+RulerSize-2-OverallTextHeight;
              DigitOffset := RulerOffset+RulerSize-2-OverallDigitHeight;
            end;
          tpBelow:
            begin
              TextOffset:=RulerOffset+2;
              DigitOffset:=RulerOffset+2;
            end;
          tpBoth, tpMiddle:
            begin
              TextOffset:=RulerOffset+((RulerSize-OverallTextHeight) div 2);
              DigitOffset:=RulerOffset+((RulerSize-OverallDigitHeight) div 2);
            end;
          else
          begin
            TextOffset:=0;
            DigitOffset:=0;
          end;
        end;
        TextOffset:=TextOffset+FontOffsetVertical;
        DigitOffset:=DigitOffset+FontOffsetVertical;
        // De Caption positie bepalen
    case Kind of
      rkTop, rkBottom :
      begin

        if DividerPlacement<>tpNone then
        begin
          Canvas.Brush.Style := bsClear;
          MarginAfterSize:=BitBltEnd-UsedDocumentMarginAfter;
          if Canvas.TextWidth(Caption)+6>MarginAfterSize then
          begin
            if Canvas.TextWidth(Caption)+8<=UsedDocumentMarginAfter-UsedDocumentMarginBefore then
            begin
              inc(NumberOfOccupiedParts);
              OccupiedParts[NumberOfOccupiedParts].StartPos:=UsedDocumentMarginAfter-Canvas.TextWidth(Caption)-4;
              OccupiedParts[NumberOfOccupiedParts].EndPos:=UsedDocumentMarginAfter-4;
            end;
          end
          else
          begin
            inc(NumberOfOccupiedParts);
            OccupiedParts[NumberOfOccupiedParts].StartPos:=BitBltEnd-Canvas.TextWidth(Caption)-4;
            OccupiedParts[NumberOfOccupiedParts].EndPos:=BitBltEnd-4;
          end;
        end;
      end;

    end;

    if Flat
    then TheRegionHandle:=RulerCreateRectRgn(UsedDocumentMarginBefore, RulerOffset+2, UsedDocumentMarginAfter, RulerOffset+RulerSize-2)
    else TheRegionHandle:=RulerCreateRectRgn(UsedDocumentMarginBefore+1, RulerOffset  , UsedDocumentMarginAfter-1, RulerOffset+RulerSize  );
    try
      SelectClipRgn(Canvas.Handle, TheRegionHandle);
    finally
      DeleteObject(TheRegionHandle);
    end;

        MaxDigitsLengthWorld:=round(MultiplyFactor*(ScreenToWorld(MaxDigitsLength)-ScreenToWorld(0)));
        if MaxDigitsLengthWorld<MinUnitStepSize
        then MaxDigitsLengthWorld:=MinUnitStepSize;
        MaxDigitsLengthWorld:=BestFit(MaxDigitsLengthWorld);
        // Draw labels
        if DividerPlacement<>tpNone then
        begin
          if MaxDigitsLengthWorld>0 then
          begin
            iCount:=round(WorldValueStart);
            iCount:=(iCount div MaxDigitsLengthWorld) * MaxDigitsLengthWorld;
            while iCount<round(WorldValueEnd)do
            begin
              HulpString:=Format('%.*f', [NumDecimals, iCount/ MultiplyFactor]);
              inc(NumberOfOccupiedParts);
              OccupiedParts[NumberOfOccupiedParts].StartPos:=WorldToScreen(iCount/ MultiplyFactor)- (Canvas.TextWidth(HulpString) div 2) + FontOffsetHorizontal;
              OccupiedParts[NumberOfOccupiedParts].EndPos:=WorldToScreen(iCount/ MultiplyFactor)- (Canvas.TextWidth(HulpString) div 2) + FontOffsetHorizontal + Canvas.TextWidth(HulpString);
              // Check if Part is already occupied
              Occupied:=False;
              if NumberOfOccupiedParts>1 then
              for iParts:=1 to NumberOfOccupiedParts-1 do
              begin
                if (OccupiedParts[NumberOfOccupiedParts].StartPos<=OccupiedParts[iParts].EndPos) and
                   (OccupiedParts[NumberOfOccupiedParts].EndPos>=OccupiedParts[iParts].StartPos)
                then Occupied:=True;
              end;
              if Occupied then
              begin
                dec(NumberOfOccupiedParts);
              end
              else
              begin
                RulerTextOut(Canvas, WorldToScreen(iCount/ MultiplyFactor)- (Canvas.TextWidth(HulpString) div 2) + FontOffsetHorizontal, DigitOffset, HulpString);
              end;
              inc(iCount, MaxDigitsLengthWorld);
            end;
          end;
        end;


    if Flat
    then TheRegionHandle:=RulerCreateRectRgn(BitBltStart+2, RulerOffset+2, BitBltEnd-2, RulerOffset+RulerSize-2)
    else TheRegionHandle:=RulerCreateRectRgn(BitBltStart  , RulerOffset  , BitBltEnd  , RulerOffset+RulerSize  );
    try
      SelectClipRgn(Canvas.Handle, TheRegionHandle);
    finally
      DeleteObject(TheRegionHandle);
    end;

        // Code om marges een andere kleur te geven
        if flat
        then Canvas.Brush.Color := MargeColor
        else Canvas.Brush.Color := clBtnFace;

      case Kind of
        rkTop, rkBottom :
        begin
          if Flat then
          begin
            Canvas.FillRect(Rect(DocumentScreenStart, RulerOffset, UsedDocumentMarginBefore, RulerOffset+RulerSize));
            Canvas.FillRect(Rect(UsedDocumentMarginAfter, RulerOffset, DocumentScreenEnd, RulerOffset+RulerSize));
          end
          else
          begin
            Canvas.FillRect(Rect(DocumentScreenStart+2, RulerOffset+2, UsedDocumentMarginBefore-2, RulerOffset+RulerSize-2));
            Canvas.FillRect(Rect(UsedDocumentMarginAfter+2, RulerOffset+2, DocumentScreenEnd-2, RulerOffset+RulerSize-2));
          end;
        end;
        else
        begin
          if Flat then
          begin
            Canvas.FillRect(Rect(RulerOffset, DocumentScreenStart, RulerOffset+RulerSize, UsedDocumentMarginBefore));
            Canvas.FillRect(Rect(RulerOffset, UsedDocumentMarginAfter, RulerOffset+RulerSize, DocumentScreenEnd));
          end
          else
          begin
            Canvas.FillRect(Rect(RulerOffset+2, DocumentScreenStart+2, RulerOffset+RulerSize-2, UsedDocumentMarginBefore-2));
            Canvas.FillRect(Rect(RulerOffset+2, UsedDocumentMarginAfter+2, RulerOffset+RulerSize-2, DocumentScreenEnd-2));
          end;
        end;
      end;

        // Code om de schaalverdelingslijntjes te tekenen.
        if MaxDigitsLengthWorld>0 then
        begin
          Canvas.Pen.Color:=LineColor;
          iCount:=round(WorldValueStart);
          iCount:=(iCount div MaxDigitsLengthWorld) * MaxDigitsLengthWorld;
          while iCount<round(WorldValueEnd)do
          begin
            for Between:=0 to TotalBetween-1 do
            begin
              LinePlace:=WorldToScreen(iCount/ MultiplyFactor+Between*(MaxDigitsLengthWorld/ MultiplyFactor/TotalBetween));
              // De lijnen niet tekenen bij margin overgangen
              if ((LinePlace>UsedDocumentMarginBefore-2) and (LinePlace<UsedDocumentMarginBefore+2)) or
                 ((LinePlace>UsedDocumentMarginAfter-2) and (LinePlace<UsedDocumentMarginAfter+2))
              then
              begin
              end
              else
              begin
                if Between=0
                then CurrentLineSize:=LargeLineSize
                else
                  if  2*Between=TotalBetween
                  then CurrentLineSize:=MiddleLineSize
                  else CurrentLineSize:=ShortLineSize;
                Occupied:=False;
                for iParts:=1 to NumberOfOccupiedParts do
                begin
                  if (LinePlace+2>=OccupiedParts[iParts].StartPos) and
                     (LinePlace-2<=OccupiedParts[iParts].EndPos)
                  then Occupied:=True;
                end;
                if (DividerPlacement=tpAbove) or (DividerPlacement=tpBoth) then
                begin
                  RulerPolyLine(Canvas, Point(LinePlace, AboveRulerOffset), Point(LinePlace, AboveRulerOffset+CurrentLineSize));

//                  RulerMoveTo(Canvas, LinePlace, AboveRulerOffset);
//                  RulerLineTo(Canvas, LinePlace, AboveRulerOffset+CurrentLineSize);
                end;
                if (DividerPlacement=tpBelow) or (DividerPlacement=tpBoth) then
                begin
                  RulerPolyLine(Canvas, Point(LinePlace, BelowRulerOffset), Point(LinePlace, BelowRulerOffset-CurrentLineSize));
//                  RulerMoveTo(Canvas, LinePlace, BelowRulerOffset);
//                  RulerLineTo(Canvas, LinePlace, BelowRulerOffset-CurrentLineSize);
                end;
                if (not Occupied) and (DividerPlacement=tpMiddle) then
                begin
                  RulerPolyLine(Canvas, Point(LinePlace, MiddleRulerOffset-CurrentLineSize div 2), Point(LinePlace, MiddleRulerOffset+CurrentLineSize div 2));
//                  RulerMoveTo(Canvas, LinePlace, MiddleRulerOffset-CurrentLineSize div 2);
//                  RulerLineTo(Canvas, LinePlace, MiddleRulerOffset+CurrentLineSize div 2);
                end;
              end;
            end;
            inc(iCount, MaxDigitsLengthWorld);
          end;
        end;
        // De Caption tekenen

    case Kind of
      rkTop, rkBottom :
      begin

        if DividerPlacement<>tpNone then
        begin
          Canvas.Brush.Style := bsClear;
          MarginAfterSize:=BitBltEnd-UsedDocumentMarginAfter;
          if Canvas.TextWidth(Caption)+6>MarginAfterSize then
          begin
            if Canvas.TextWidth(Caption)+8<=UsedDocumentMarginAfter-UsedDocumentMarginBefore
            then RulerTextOut(Canvas, UsedDocumentMarginAfter-Canvas.TextWidth(Caption)-4, TextOffset, Caption);
          end
          else
          begin
            RulerTextOut(Canvas, BitBltEnd-Canvas.TextWidth(Caption)-4, TextOffset, Caption);
          end;
        end;
      end;

      end;
    end;


      if Flat then
      begin
        Canvas.Pen.Color:=Color;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore, RulerOffset), Point(UsedDocumentMarginBefore, RulerOffset+RulerSize));
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore-1, RulerOffset), Point(UsedDocumentMarginBefore-1, RulerOffset+RulerSize));
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter, RulerOffset), Point(UsedDocumentMarginAfter, RulerOffset+RulerSize));
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter+1, RulerOffset), Point(UsedDocumentMarginAfter+1, RulerOffset+RulerSize));
      end
      else
      begin
        Canvas.Pen.Color:=clBtnShadow;
//        Canvas.Pen.Color:=clRed;
        RulerPolyLine(Canvas, Point(DocumentScreenStart, RulerOffset), Point(DocumentScreenEnd-1, RulerOffset));
        RulerPolyLine(Canvas, Point(DocumentScreenStart, RulerOffset), Point(DocumentScreenStart, RulerOffset+RulerSize));
        RulerPolyLine(Canvas, Point(DocumentScreenStart, RulerOffset+RulerSize-2), Point(UsedDocumentMarginBefore-3, RulerOffset+RulerSize-2));

        Canvas.Pen.Color:=clBtnHighlight;
//      Canvas.Pen.Color:=clRed;
        RulerPolyLine(Canvas, Point(DocumentScreenStart+1, RulerOffset+1), Point(UsedDocumentMarginBefore-3, RulerOffset+1));
        RulerPolyLine(Canvas, Point(DocumentScreenStart+1, RulerOffset+1), Point(DocumentScreenStart+1, RulerOffset+RulerSize));
        RulerPolyLine(Canvas, Point(DocumentScreenStart+1, RulerOffset+RulerSize-1), Point(DocumentScreenEnd-1, RulerOffset+RulerSize-1));

        Canvas.Pen.Color:=clBtnShadow;
//      Canvas.Pen.Color:=clRed;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore-2, RulerOffset+1), Point(UsedDocumentMarginBefore-2, RulerOffset+RulerSize-1));
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore-1, RulerOffset+1), Point(UsedDocumentMarginBefore-1, RulerOffset+RulerSize-1));

        Canvas.Pen.Color:=cl3DDkShadow;
//      Canvas.Pen.Color:=clRed;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore, RulerOffset+1), Point(UsedDocumentMarginAfter-1, RulerOffset+1));
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore, RulerOffset+1), Point(UsedDocumentMarginBefore, RulerOffset+RulerSize-1));

        Canvas.Pen.Color := clBtnShadow;
//      Canvas.Pen.Color:=clRed;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter - 1, RulerOffset+1), Point(UsedDocumentMarginAfter-1, RulerOffset+RulerSize-1));

        Canvas.Pen.Color := clBtnFace;
//      Canvas.Pen.Color:=clRed;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore+1, RulerOffset+RulerSize-2), Point(UsedDocumentMarginAfter-2, RulerOffset+RulerSize-2));

        Canvas.Pen.Color:=clBtnShadow;
//      Canvas.Pen.Color:=clRed;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter+2, RulerOffset+rulerSize-2), Point(DocumentScreenEnd-2, RulerOffset+RulerSize - 2));
        RulerPolyLine(Canvas, Point(DocumentScreenEnd-2, RulerOffset+RulerSize - 2), Point(DocumentScreenEnd-2, RulerOffset+1));
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter, RulerOffset + 1), Point(UsedDocumentMarginAfter, RulerOffset + RulerSize - 1));

        Canvas.Pen.Color:=clBtnHighlight;
//      Canvas.Pen.Color:=clRed;
        RulerPolyLine(Canvas, Point(DocumentScreenEnd-3, RulerOffset+1), Point(UsedDocumentMarginAfter + 1, RulerOffset + 1));
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter + 1, RulerOffset + 2), Point(UsedDocumentMarginAfter + 1, RulerOffset+RulerSize - 1));
        RulerPolyLine(Canvas, Point(DocumentscreenEnd-1, RulerOffset), Point(DocumentscreenEnd-1, RulerOffset+RulerSize - 1));





        // Tekenen van Ruler frame inclusief de marges (geoptimaliseerd)
        // Left margin

{        Canvas.Pen.Color:=clBtnShadow;
        RulerPolyLine(Canvas, Point(DocumentScreenStart, RulerOffset), Point(DocumentScreenStart, RulerOffset+RulerSize));
        RulerPolyLine(Canvas, Point(DocumentScreenStart, RulerOffset+RulerSize-2), Point(UsedDocumentMarginBefore-3, RulerOffset+RulerSize-2));
        RulerPolyLine(Canvas, Point(DocumentScreenStart, RulerOffset), Point(UsedDocumentMarginBefore-3, RulerOffset));

        Canvas.Pen.Color:=clBtnHighlight;
        RulerPolyLine(Canvas, Point(DocumentScreenStart+1, RulerOffset+1), Point(UsedDocumentMarginBefore-3, RulerOffset+1));
        RulerPolyLine(Canvas, Point(DocumentScreenStart+1, RulerOffset+1), Point(DocumentScreenStart+1, RulerOffset+RulerSize));
        RulerPolyLine(Canvas, Point(DocumentScreenStart+1, RulerOffset+RulerSize-1), Point(UsedDocumentMarginBefore-3, RulerOffset+RulerSize-1));



        // Right margin
        Canvas.Pen.Color:=clBtnShadow;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter+1, RulerOffset), Point(DocumentScreenEnd-1, RulerOffset));
        Canvas.Pen.Color:=clBtnHighlight;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter+1, RulerOffset+RulerSize-1), Point(DocumentScreenEnd-1, RulerOffset+RulerSize-1));
        Canvas.Pen.Color:=clBtnShadow;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter+2, RulerOffset+rulerSize-2), Point(DocumentScreenEnd-2, RulerOffset+RulerSize - 2));
        RulerPolyLine(Canvas, Point(DocumentScreenEnd-2, RulerOffset+RulerSize - 2), Point(DocumentScreenEnd-2, RulerOffset+1));
        Canvas.Pen.Color:=clBtnHighlight;
        RulerPolyLine(Canvas, Point(DocumentScreenEnd-3, RulerOffset+1), Point(UsedDocumentMarginAfter + 1, RulerOffset + 1));
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter + 1, RulerOffset + 2), Point(UsedDocumentMarginAfter + 1, RulerOffset+RulerSize - 1));
        RulerPolyLine(Canvas, Point(DocumentscreenEnd-1, RulerOffset), Point(DocumentscreenEnd-1, RulerOffset+RulerSize - 1));

}

        // Actual ruler
        Canvas.Pen.Color:=clBtnShadow;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore-2, RulerOffset), Point(UsedDocumentMarginAfter, RulerOffset));
        Canvas.Pen.Color:=clBtnHighlight;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore-2, RulerOffset+RulerSize-1), Point(UsedDocumentMarginAfter, RulerOffset+RulerSize-1));
        Canvas.Pen.Color:=clBtnShadow;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore-2, RulerOffset+1), Point(UsedDocumentMarginBefore-2, RulerOffset+RulerSize-1));
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore-1, RulerOffset+1), Point(UsedDocumentMarginBefore-1, RulerOffset+RulerSize-1));
        Canvas.Pen.Color:=cl3DDkShadow;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore, RulerOffset+1), Point(UsedDocumentMarginAfter-1, RulerOffset+1));
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore, RulerOffset+1), Point(UsedDocumentMarginBefore, RulerOffset+RulerSize-1));
        Canvas.Pen.Color := clBtnShadow;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter - 1, RulerOffset+1), Point(UsedDocumentMarginAfter-1, RulerOffset+RulerSize-1));
        Canvas.Pen.Color := clBtnFace;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginBefore+1, RulerOffset+RulerSize-2), Point(UsedDocumentMarginAfter-2, RulerOffset+RulerSize-2));
        Canvas.Pen.Color:=clBtnShadow;
        RulerPolyLine(Canvas, Point(UsedDocumentMarginAfter, RulerOffset + 1), Point(UsedDocumentMarginAfter, RulerOffset + RulerSize - 1));


      end;
    end;


      SelectClipRgn(Canvas.Handle, 0);

{    if Flat
    then RulerBitBlt(Canvas.Handle, BitBltStart+2, RulerOffset+2, BitBltEnd-BitBltStart-4, RulerSize-4, TheNewRulerBitmap.Canvas.Handle, BitBltStart+2, RulerOffset+2, SRCCOPY)
    else RulerBitBlt(Canvas.Handle, BitBltStart, RulerOffset, BitBltEnd-BitBltStart, RulerSize, TheNewRulerBitmap.Canvas.Handle, BitBltStart, RulerOffset, SRCCOPY);}
{  finally
    TheNewRulerBitmap.Free;
  end;}
end;

procedure ThlcRuler.Paint;
begin
{  if not AllowPaint
  then exit;}
  inherited Paint;
  DrawIt;
end;

procedure ThlcRuler.SetFont(const Value: TFont);
begin
  FFont.Assign(Value);
  Invalidate;
end;

procedure ThlcRuler.SetCaption(const Value: string);
begin
  FCaption := Value;
  Invalidate;
end;

procedure ThlcRuler.SetColor(const Value: TColor);
begin
  FColor := Value;
  Invalidate;
end;

procedure ThlcRuler.SetFlat(const Value: boolean);
begin
  FFlat := Value;
  Invalidate;
end;

procedure ThlcRuler.SetKind(const Value: TRulerKind);
begin
  if FKind <> Value then
  begin
    case Kind of
      rkTop, rkBottom :
      begin
        case Value of
          rkTop, rkBottom :
          begin
            FKind := Value;
          end;
          rkLeft, rkRight:
          begin
            FKind := Value;
            if not (csLoading in ComponentState) then SetBounds(Left, Top, Height, Width);
          end;
        end;
      end;
      rkLeft, rkRight:
      begin
        case Value of
          rkTop, rkBottom :
          begin
            FKind := Value;
            if not (csLoading in ComponentState) then SetBounds(Left, Top, Height, Width);
          end;
          rkLeft, rkRight:
          begin
            FKind := Value;
            Invalidate;
          end;
        end;
      end;
    end;
  end;
end;

procedure ThlcRuler.SetLargeLineSize(const Value: Word);
begin
  FLargeLineSize := Value;
  Invalidate;
end;

procedure ThlcRuler.SetMinUnitStepSize(const Value: Word);
begin
  if Value > 0 then
  begin
    FMinUnitStepSize := Value;
    Invalidate;
  end;
end;

procedure ThlcRuler.SetNumDecimals(const Value: Word);
begin
  FNumDecimals := Value;
  Invalidate;
end;

procedure ThlcRuler.SetShortLineSize(const Value: Word);
begin
  FShortLineSize := Value;
  Invalidate;
end;

procedure ThlcRuler.SetDocumentVisualEnd(const Value: Integer);
begin
  FDocumentVisualEnd := Value;
  Invalidate;
end;

procedure ThlcRuler.SetDocumentVisualStart(const Value: Integer);
begin
  FDocumentVisualStart := Value;
  Invalidate;
end;

procedure ThlcRuler.SetRulerOffset(const Value: Word);
begin
  FRulerOffset := Value;
  Invalidate;
end;

procedure ThlcRuler.SetRulerSize(const Value: Word);
begin
  FRulerSize := Value;
  Invalidate;
end;

procedure ThlcRuler.SetDocumentScreenEnd(const Value: Integer);
begin
  FDocumentScreenEnd := Value;
  Invalidate;
end;

procedure ThlcRuler.SetDocumentScreenStart(const Value: Integer);
begin
  FDocumentScreenStart := Value;
  Invalidate;
end;

procedure ThlcRuler.SetDocumentMarginAfter(const Value: Integer);
begin
  FDocumentMarginAfter := Value;
  Invalidate;
end;

procedure ThlcRuler.SetDocumentMarginBefore(const Value: Integer);
begin
  FDocumentMarginBefore := Value;
  Invalidate;
end;

procedure ThlcRuler.SetMargeColor(const Value: TColor);
begin
  FMargeColor := Value;
  Invalidate;
end;

procedure ThlcRuler.SetDocumentWorldEnd(const Value: double);
begin
  FDocumentWorldEnd := Value;
//  Paint;
  Invalidate;
end;

procedure ThlcRuler.SetDocumentWorldStart(const Value: double);
begin
  FDocumentWorldStart := Value;
//  Paint;
  Invalidate;
end;

function ThlcRuler.ScreenToWorld(Value: integer): double;
var
  Factor: double;
begin
  result:=0;
  if DocumentScreenEnd-DocumentScreenStart<>0 then
  begin
    Factor:=(DocumentWorldEnd-DocumentWorldStart)/(DocumentScreenEnd-DocumentScreenStart);
    result:=(Value-DocumentScreenStart)*Factor+DocumentWorldStart;
  end;
end;

function ThlcRuler.WorldToScreen(Value: double): Int64;
var
  Factor: double;
begin
  result:=0;
  if DocumentWorldEnd-DocumentWorldStart<>0 then
  begin
    Factor:=(DocumentScreenEnd-DocumentScreenStart)/(DocumentWorldEnd-DocumentWorldStart);
    result:=trunc((Value-DocumentWorldStart)*Factor)+DocumentScreenStart;
  end;
end;

procedure ThlcRuler.SetDividerPlacement(const Value: TPlacement);
begin
  FDividerPlacement := Value;
  Invalidate;
end;

procedure ThlcRuler.SetFontOffsetVertical(const Value: integer);
begin
  FFontOffsetVertical := Value;
  Invalidate;
end;

procedure ThlcRuler.SetFontOffsetHorizontal(const Value: integer);
begin
  FFontOffsetHorizontal := Value;
  Invalidate;
end;

procedure ThlcRuler.SetMiddleLineSize(const Value: Word);
begin
  FMiddleLineSize := Value;
  Invalidate;
end;

procedure ThlcRuler.SetLineColor(const Value: TColor);
begin
  FLineColor := Value;
  Invalidate;
end;

procedure ThlcRuler.SetNumberOfUnitFractionMarkers(
  const Value: TNumberOfUnitFractionMarkers);
begin
  FNumberOfUnitFractionMarkers := Value;
  Invalidate;
end;

function ThlcRuler.GetTotalBetween: integer;
begin
  case NumberOfUnitFractionMarkers of
    numOne : result:=1;
    numTwo : result:=2;
    numFour : result:=4;
    numEight : result:=8;
    numTen : result:=10;
    numSixteen : result:=16;
    numTwenty : result:=20;
    else result:=1;
  end;
end;

procedure ThlcRuler.SetWindowsColor(const Value: boolean);
begin
  FWindowsColor := Value;
  Invalidate;
end;

procedure ThlcRuler.SetShowMargins(const Value: boolean);
begin
  FShowMargins := Value;
  Invalidate;
end;

{procedure ThlcRuler.Loaded;
begin
//  AllowPaint:=True;
end;}

(*procedure ThlcRuler.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
//    Param:=pa

{    Style := WS_POPUP or WS_BORDER;
    WindowClass.Style := WindowClass.Style or CS_SAVEBITS;
    if NewStyleControls then ExStyle := WS_EX_TOOLWINDOW;
    AddBiDiModeExStyle(ExStyle);}
  end;
end;*)

end.



