{-$D}
{*************************************************************************}
{                                                                         }
{                    HlcUndoRedoRepeat Component                          }
{                                                                         }
{  By using the hlcUndoRedoRepeat Component you are able to control the   }
{  undo, redo and repeat functionality in an easy way. This               }
{  proffesionality is provided in one component. It's easy to use.        }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{  Copyright (c) 1997, 1999 High-Logic [Erwin Denissen]                   }
{                                                                         }
{  You may use a Shareware Copy of this Software for an evaluation period }
{  of up to 30 days, in order to determine whether the Software meets     }
{  your needs before purchasing it. Once the evaluation period ends, you  }
{  agree to either purchase a copy of the software, or to stop using it   }
{  and remove it from your computer system.                               }
{                                                                         }
{  A registered version of this and other components and software can be  }
{  ordered at the following site: http://www.high-logic.com/index.html    }
{                                                                         }
{  There are no known bugs at the moment, so please notify me when you    }
{  find one. Make sure you are using the latest version of this software. }
{  email: erwind@high-logic.com                                           }
{                                                                         }
{  DISCLAIMER                                                             }
{  In no event will the author be held liable for any direct or indirect  }
{  damage resulting from the use of this software or any derivatives      }
{  thereof.                                                               }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{  Available methods                                                      }
{  -----------------                                                      }
{                                                                         }
{*************************************************************************}
{                                                                         }
{ Version 1.3 8-8-1999                                                    }
{  Added CRC32 checksum                                                   }
{                                                                         }
{ Version 1.2 19-10-1998                                                  }
{  First official release                                                 }
{                                                                         }
{ Version 1.103 14-10-1998                                                }
{ Added 4 Methods                                                         }
{  procedure LoadFromStream(Stream: TStream);                             }
{  procedure SaveToStream(Stream: TStream);                               }
{  procedure LoadFromFile(const FileName: string);                        }
{  procedure SaveToFile(const FileName: string);                          }
{                                                                         }
{ Version 1.102 10-10-1998                                                }
{   Added Group support and the Active property                           }
{                                                                         }
{ Version 1.1 14-09-1998                                                  }
{   Fixed two problems, now works with Delphi 4                           }
{                                                                         }
{ Version 1.0 30-07-1997                                                  }
{   First release, works with Delphi 2 and 3                              }
{                                                                         }
{*************************************************************************}
{                                                                         }
{ More things to do:                                                      }
{                                                                         }
{ * More debug messages and other debug options                           }
{ * Better Documentation in the code but also in some readme file         }
{ * Maybe change the MenuItem Support, because Delphi 4.0 has a new       }
{   action component.                                                     }
{ * Recalculate the MemoryAllocationSize at some points                   }
{                                                                         }
{*************************************************************************}

unit HlcUndoRedoRepeat;

interface

uses
  Windows, SysUtils, Dialogs, Classes, Menus, HlcRegisterAbout;

type

  TFunctionVariable = function(TheData1, TheData2, TheData3: Pointer): Pointer of object;

  TPointerRecord =
  record
    procedureVariable : TFunctionVariable;
    procedureData1 : Pointer;
    procedureData2 : Pointer;
    procedureData3 : Pointer;
    lengte1, lengte2, lengte3 : longint;
    Number : integer;
    CRC32 : DWORD;
  end;
  TPointerArrayRecord = array [1..1] of TPointerRecord;
  PPointerArray = ^TPointerArrayRecord;

  TName = string[25];
  TNameArray = array [1..1] of TName;
  PNameArray = ^TNameArray;

  ThlcUndoRedoRepeat = class(TComponent)
  private
    TheItems : TStringList;
    FAbout : THLCAbout;
    FActive : Boolean;
    FMaxSize : longint;
    FMaxCount : longint;
    DebugMode : Boolean;
    MIUndo : TMenuItem;
    MIRedo : TMenuItem;
    MIRepeat : TMenuItem;
    MIAllowGroupUndo : Boolean;
    PossibleGroupUndo : Boolean;
    NumberOfRepeatPointers, NumberOfNames, TheTempNumber, CurrentPointer, LastPointer, NumberOfPointers : integer;
    TheRepeatPointers, TheTempPointers, ThePointers : PPointerArray;
    TheNames : PNameArray;
    InRepeatMode, Started, InUndoMode, InRedoMode, Updating : boolean;
    procedure SetMaxCount(Value : longint);
    procedure SetMaxSize(Value : longint);
    procedure SetActiveProperty(Value : Boolean);
    procedure FreeRepeatPointers;
    procedure SetMenuItems;
    procedure CheckLimits;
    procedure ShowMessage(TheMessage: string);
    procedure RemoveFirstPointers;
  public
    MemoryAllocationSize : longint;
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Add(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: Pointer; TheSize1, TheSize2, TheSize3 : integer );
    procedure AddRepeat(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: Pointer; TheSize1, TheSize2, TheSize3 : integer );
    procedure AddToPrevious(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: Pointer; TheSize1, TheSize2, TheSize3 : integer );
    function Undo:boolean;
    function Redo:boolean;
    function DoRepeat:boolean;
    function CanUndo:boolean;
    function CanRedo:boolean;
    function CanRepeat:boolean;
    procedure UndoBeginUpdate(TheName : TName);
    procedure UndoEndUpdate;
    function IsUpdating:boolean;
    function GetUndoName: TName;
    function GetRedoName: TName;
    function GetUndoItems: TStringList;
    function GetRedoItems: TStringList;
    function GetRepeatName: TName;
    procedure Reset;
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToStream(Stream: TStream);
    procedure LoadFromFile(const FileName: string);
    procedure SaveToFile(const FileName: string);
  published
    property About : THLCAbout read FAbout write FAbout;
    property Active:Boolean read FActive write SetActiveProperty;
    property Debugging:Boolean read DebugMode write DebugMode;
    property MenuItemUndo:TMenuItem read MIUndo write MIUndo;
    property MaxSize:longint read FMaxSize write SetMaxSize;
    property MaxCount:longint read FMaxCount write SetMaxCount;
    property MenuItemRedo:TMenuItem read MIRedo write MIRedo;
    property MenuItemRepeat:TMenuItem read MIRepeat write MIRepeat;
    property AllowGroupUndo:Boolean read MIAllowGroupUndo write MIAllowGroupUndo;
  end;

implementation

type
  ThlcUndoRedoRepeatfileHeader = record
    Version : integer;
    VersionInfo : array[1..10] of char;
    NumberOfPoints,
    CurrentPointer,
    LastPointer,
    NumberOfNames : integer;
    Reserved : integer;
  end;
  ThlcUndoRedoRepeatNameRecord = record
    Name : TName;
  end;
  ThlcUndoRedoRepeatPointerRecord = record
    Size1, Size2, Size3 : integer;
    procedureVariable : TFunctionVariable;
    Number : integer;
  end;

{$R-}


const
  table:  array[0..255] of DWORD =
 ($00000000, $77073096, $EE0E612C, $990951BA,
  $076DC419, $706AF48F, $E963A535, $9E6495A3,
  $0EDB8832, $79DCB8A4, $E0D5E91E, $97D2D988,
  $09B64C2B, $7EB17CBD, $E7B82D07, $90BF1D91,
  $1DB71064, $6AB020F2, $F3B97148, $84BE41DE,
  $1ADAD47D, $6DDDE4EB, $F4D4B551, $83D385C7,
  $136C9856, $646BA8C0, $FD62F97A, $8A65C9EC,
  $14015C4F, $63066CD9, $FA0F3D63, $8D080DF5,
  $3B6E20C8, $4C69105E, $D56041E4, $A2677172,
  $3C03E4D1, $4B04D447, $D20D85FD, $A50AB56B,
  $35B5A8FA, $42B2986C, $DBBBC9D6, $ACBCF940,
  $32D86CE3, $45DF5C75, $DCD60DCF, $ABD13D59,
  $26D930AC, $51DE003A, $C8D75180, $BFD06116,
  $21B4F4B5, $56B3C423, $CFBA9599, $B8BDA50F,
  $2802B89E, $5F058808, $C60CD9B2, $B10BE924,
  $2F6F7C87, $58684C11, $C1611DAB, $B6662D3D,

  $76DC4190, $01DB7106, $98D220BC, $EFD5102A,
  $71B18589, $06B6B51F, $9FBFE4A5, $E8B8D433,
  $7807C9A2, $0F00F934, $9609A88E, $E10E9818,
  $7F6A0DBB, $086D3D2D, $91646C97, $E6635C01,
  $6B6B51F4, $1C6C6162, $856530D8, $F262004E,
  $6C0695ED, $1B01A57B, $8208F4C1, $F50FC457,
  $65B0D9C6, $12B7E950, $8BBEB8EA, $FCB9887C,
  $62DD1DDF, $15DA2D49, $8CD37CF3, $FBD44C65,
  $4DB26158, $3AB551CE, $A3BC0074, $D4BB30E2,
  $4ADFA541, $3DD895D7, $A4D1C46D, $D3D6F4FB,
  $4369E96A, $346ED9FC, $AD678846, $DA60B8D0,
  $44042D73, $33031DE5, $AA0A4C5F, $DD0D7CC9,
  $5005713C, $270241AA, $BE0B1010, $C90C2086,
  $5768B525, $206F85B3, $B966D409, $CE61E49F,
  $5EDEF90E, $29D9C998, $B0D09822, $C7D7A8B4,
  $59B33D17, $2EB40D81, $B7BD5C3B, $C0BA6CAD,

  $EDB88320, $9ABFB3B6, $03B6E20C, $74B1D29A,
  $EAD54739, $9DD277AF, $04DB2615, $73DC1683,
  $E3630B12, $94643B84, $0D6D6A3E, $7A6A5AA8,
  $E40ECF0B, $9309FF9D, $0A00AE27, $7D079EB1,
  $F00F9344, $8708A3D2, $1E01F268, $6906C2FE,
  $F762575D, $806567CB, $196C3671, $6E6B06E7,
  $FED41B76, $89D32BE0, $10DA7A5A, $67DD4ACC,
  $F9B9DF6F, $8EBEEFF9, $17B7BE43, $60B08ED5,
  $D6D6A3E8, $A1D1937E, $38D8C2C4, $4FDFF252,
  $D1BB67F1, $A6BC5767, $3FB506DD, $48B2364B,
  $D80D2BDA, $AF0A1B4C, $36034AF6, $41047A60,
  $DF60EFC3, $A867DF55, $316E8EEF, $4669BE79,
  $CB61B38C, $BC66831A, $256FD2A0, $5268E236,
  $CC0C7795, $BB0B4703, $220216B9, $5505262F,
  $C5BA3BBE, $B2BD0B28, $2BB45A92, $5CB36A04,
  $C2D7FFA7, $B5D0CF31, $2CD99E8B, $5BDEAE1D,

  $9B64C2B0, $EC63F226, $756AA39C, $026D930A,
  $9C0906A9, $EB0E363F, $72076785, $05005713,
  $95BF4A82, $E2B87A14, $7BB12BAE, $0CB61B38,
  $92D28E9B, $E5D5BE0D, $7CDCEFB7, $0BDBDF21,
  $86D3D2D4, $F1D4E242, $68DDB3F8, $1FDA836E,
  $81BE16CD, $F6B9265B, $6FB077E1, $18B74777,
  $88085AE6, $FF0F6A70, $66063BCA, $11010B5C,
  $8F659EFF, $F862AE69, $616BFFD3, $166CCF45,
  $A00AE278, $D70DD2EE, $4E048354, $3903B3C2,
  $A7672661, $D06016F7, $4969474D, $3E6E77DB,
  $AED16A4A, $D9D65ADC, $40DF0B66, $37D83BF0,
  $A9BCAE53, $DEBB9EC5, $47B2CF7F, $30B5FFE9,
  $BDBDF21C, $CABAC28A, $53B39330, $24B4A3A6,
  $BAD03605, $CDD70693, $54DE5729, $23D967BF,
  $B3667A2E, $C4614AB8, $5D681B02, $2A6F2B94,
  $B40BBE37, $C30C8EA1, $5A05DF1B, $2D02EF8D);

procedure CalcCRC32 (p:  Pointer; nByte: Cardinal; var CRCvalue:  DWORD);
var
  i:  Cardinal;
  q:  pByteArray;  // ^buffer;
begin
  if nByte>0 then
  begin
    q := p;
    for   i := 0 to nByte-1
    do CRCvalue := (CRCvalue shr 8) xor Table[q[i] xor (CRCvalue and $000000FF)];
  end;
end; {CalcCRC32}

constructor ThlcUndoRedoRepeat.Create (AOwner: TComponent);
begin
  inherited Create(AOwner);
  Active:=True;
  Updating:=false;
  InUndoMode:=false;
  InRedoMode:=false;
  NumberOfPointers:=0;
  CurrentPointer:=0;
  LastPointer:=0;
  ThePointers:=nil;
  NumberOfNames:=0;
  TheNames:=nil;
  TheTempPointers:=nil;
  NumberOfRepeatPointers:=0;
  TheRepeatPointers:=nil;
  InRepeatMode:=false;
  SetMenuItems;
  AllowGroupUndo:=True;
  PossibleGroupUndo:=False;
  DebugMode:=True;
  MemoryAllocationSize:=0;
  FMaxSize:=0;
  FMaxCount:=0;
  TheItems:=TStringList.Create;
end;

destructor ThlcUndoRedoRepeat.Destroy;
var
  t : integer;
begin
  try
    FreeMem(TheNames);
    if NumberOfPointers>0
    then for t:=1 to NumberOfPointers do
    begin
      FreeMem(ThePointers[t].procedureData1);
      FreeMem(ThePointers[t].procedureData2);
      FreeMem(ThePointers[t].procedureData3);
    end;
    FreeMem(ThePointers);
    NumberOfRepeatPointers:=0;
    CurrentPointer:=0;
    LastPointer:=0;
    TheItems.Free;
  except
    ShowMessage('Unable to free all memory in Undo object!');
  end;
  inherited Destroy;
end;

procedure ThlcUndoRedoRepeat.SetMaxSize(Value : longint);
begin
  if FMaxSize<>Value then
  begin
    if Value>=0
    then FMaxSize:=Value
    else FMaxSize:=0;
    CheckLimits;
  end;
end;

procedure ThlcUndoRedoRepeat.SetMaxCount(Value : longint);
begin
  if FMaxCount<>Value then
  begin
    if Value>=0
    then FMaxCount:=Value
    else FMaxCount:=0;
    CheckLimits;
  end;
end;

procedure ThlcUndoRedoRepeat.SetActiveProperty(Value : Boolean);
begin
  if Value<>FActive then
  begin
    FActive:=Value;
    SetMenuItems;
  end;
end;

procedure ThlcUndoRedoRepeat.ShowMessage(TheMessage: string);
begin
  if DebugMode then
  begin
    MessageDlg(TheMessage, mtError, [mbOK], 0);
  end;
end;

procedure ThlcUndoRedoRepeat.Reset;
var
  t : integer;
begin
  if not (Updating or InUndoMode or InRedoMode) then
  begin
    FreeMem(TheNames);
    TheNames:=nil;
    if NumberOfPointers>0
    then for t:=1 to NumberOfPointers do
    begin
      FreeMem(ThePointers[t].procedureData1);
      FreeMem(ThePointers[t].procedureData2);
      FreeMem(ThePointers[t].procedureData3);
    end;
    FreeMem(ThePointers);
    ThePointers:=nil;
    Updating:=false;
    InUndoMode:=false;
    InRedoMode:=false;
    NumberOfPointers:=0;
    CurrentPointer:=0;
    LastPointer:=0;
    NumberOfNames:=0;
    NumberOfRepeatPointers:=0;
    MemoryAllocationSize:=0;
    SetMenuItems;
  end;
end;

function ThlcUndoRedoRepeat.GetRepeatName: TName;
begin
  if CanRepeat
  then result:=TheNames[NumberOfNames]
  else result:='';
end;

function ThlcUndoRedoRepeat.GetUndoName: TName;
begin
  if CanUndo
  then result:=TheNames[CurrentPointer]
  else result:='';
end;

function ThlcUndoRedoRepeat.GetRedoName: TName;
begin
  if CanRedo
  then result:=TheNames[CurrentPointer+1]
  else result:='';
end;

function ThlcUndoRedoRepeat.GetUndoItems: TStringList;
var
  i : integer;
begin
  TheItems.Clear;
  if CanUndo then
  begin
    for i:=CurrentPointer downto 1 do
    TheItems.Add(TheNames[i]);
  end;
  result:=TheItems;
end;

function ThlcUndoRedoRepeat.GetRedoItems: TStringList;
var
  i : integer;
begin
  TheItems.Clear;
  if CanRedo then
  begin
    for i:=CurrentPointer+1 to LastPointer do
    TheItems.Add(TheNames[i]);
  end;
  result:=TheItems;
end;

procedure ThlcUndoRedoRepeat.AddToPrevious(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: Pointer; TheSize1, TheSize2, TheSize3 : integer );
begin
  if Active then
  begin
    if Updating and (not Started) then
    begin
      inc(NumberOfPointers);
      ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
      with ThePointers[NumberOfPointers] do
      begin
        procedureVariable:=TheFunction;
        procedureData1:=AllocMem(TheSize1);
        lengte1:=TheSize1;
        move(TheData1^, procedureData1^, TheSize1);
        procedureData2:=AllocMem(TheSize2);
        lengte2:=TheSize2;
        move(TheData2^, procedureData2^, TheSize2);
        procedureData3:=AllocMem(TheSize3);
        lengte3:=TheSize3;
        move(TheData3^, procedureData3^, TheSize3);
        Number:=CurrentPointer-1;
        CRC32:=$ffffffff;
        CalcCRC32(procedureData1, TheSize1, CRC32);
        CalcCRC32(procedureData2, TheSize2, CRC32);
        CalcCRC32(procedureData3, TheSize3, CRC32);
        MemoryAllocationSize:=MemoryAllocationSize+lengte1+lengte2+lengte3+3*SizeOf(Longint)+SizeOf(Pointer);
      end;
    end;
  end;
end;

procedure ThlcUndoRedoRepeat.Add(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: Pointer; TheSize1, TheSize2, TheSize3 : integer );
begin
  if Active then
  begin
    if InRedoMode or InUndoMode or InRepeatMode then
    begin
      inc(TheTempNumber);
      ReAllocMem(TheTempPointers, TheTempNumber*SizeOf(TPointerRecord));
      with TheTempPointers[TheTempNumber] do
      begin
        procedureVariable:=TheFunction;
        procedureData1:=AllocMem(TheSize1);
        lengte1:=TheSize1;
        move(TheData1^, procedureData1^, TheSize1);
        procedureData2:=AllocMem(TheSize2);
        lengte2:=TheSize2;
        move(TheData2^, procedureData2^, TheSize2);
        procedureData3:=AllocMem(TheSize3);
        lengte3:=TheSize3;
        move(TheData3^, procedureData3^, TheSize3);
        if InUndoMode
        then Number:=CurrentPointer
        else Number:=CurrentPointer+1;
        CRC32:=$ffffffff;
        CalcCRC32(procedureData1, TheSize1, CRC32);
        CalcCRC32(procedureData2, TheSize2, CRC32);
        CalcCRC32(procedureData3, TheSize3, CRC32);
        MemoryAllocationSize:=MemoryAllocationSize+lengte1+lengte2+lengte3+3*SizeOf(Longint)+SizeOf(Pointer);
      end;
    end
    else
    if Updating then
    begin
      Started:=true;
      inc(NumberOfPointers);
      ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
      with ThePointers[NumberOfPointers] do
      begin
        procedureVariable:=TheFunction;
        procedureData1:=AllocMem(TheSize1);
        lengte1:=TheSize1;
        move(TheData1^, procedureData1^, TheSize1);
        procedureData2:=AllocMem(TheSize2);
        lengte2:=TheSize2;
        move(TheData2^, procedureData2^, TheSize2);
        procedureData3:=AllocMem(TheSize3);
        lengte3:=TheSize3;
        move(TheData3^, procedureData3^, TheSize3);
        Number:=CurrentPointer;
        CRC32:=$ffffffff;
        CalcCRC32(procedureData1, TheSize1, CRC32);
        CalcCRC32(procedureData2, TheSize2, CRC32);
        CalcCRC32(procedureData3, TheSize3, CRC32);
        MemoryAllocationSize:=MemoryAllocationSize+lengte1+lengte2+lengte3+3*SizeOf(Longint)+SizeOf(Pointer);
      end;
    end;
    SetMenuItems;
  end;
end;

procedure ThlcUndoRedoRepeat.AddRepeat(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: Pointer; TheSize1, TheSize2, TheSize3 : integer );
begin
  if Active then
  begin
    if Updating then
    begin
      inc(NumberOfRepeatPointers);
      ReAllocMem(TheRepeatPointers, NumberOfRepeatPointers*SizeOf(TPointerRecord));
      with TheRepeatPointers[NumberOfRepeatPointers] do
      begin
        procedureVariable:=TheFunction;
        procedureData1:=AllocMem(TheSize1);
        lengte1:=TheSize1;
        move(TheData1^, procedureData1^, TheSize1);
        procedureData2:=AllocMem(TheSize2);
        lengte2:=TheSize2;
        move(TheData2^, procedureData2^, TheSize2);
        procedureData3:=AllocMem(TheSize3);
        lengte3:=TheSize3;
        move(TheData3^, procedureData3^, TheSize3);
        Number:=CurrentPointer;
        CRC32:=$ffffffff;
        CalcCRC32(procedureData1, TheSize1, CRC32);
        CalcCRC32(procedureData2, TheSize2, CRC32);
        CalcCRC32(procedureData3, TheSize3, CRC32);
        MemoryAllocationSize:=MemoryAllocationSize+lengte1+lengte2+lengte3+3*SizeOf(Longint)+SizeOf(Pointer);
      end;
    end;
    SetMenuItems;
  end;
end;

procedure ThlcUndoRedoRepeat.FreeRepeatPointers;
var
  t : integer;
begin
  if NumberOfRepeatPointers>0 then
  begin
    for t:=1 to NumberOfRepeatPointers do with TheRepeatPointers[t] do
    begin
      FreeMem(procedureData1);
      FreeMem(procedureData2);
      FreeMem(procedureData3);
      MemoryAllocationSize:=MemoryAllocationSize-lengte1-lengte2-lengte3-3*SizeOf(Longint)-SizeOf(Pointer);
    end;
    FreeMem(TheRepeatPointers);
    TheRepeatPointers:=nil;
    NumberOfRepeatPointers:=0;
  end;
end;

function ThlcUndoRedoRepeat.Undo:boolean;
var
  LastOfThisNumber, TheTotalOfThisNumber, u, i, t : integer;
  CurrentCRC32: DWORD;
begin
  result:=false;
  if Active then
  begin
    if Updating
    then ShowMessage('Can''t Undo, in updating mode!')
    else if CanUndo then
    begin
      InUndoMode:=true;
      FreeRepeatPointers;
      TheTempNumber:=0;
      TheTotalOfThisNumber:=0;
      t:=NumberOfPointers;
      while ThePointers[t].Number>CurrentPointer do dec(t);
      result:=true;
      repeat
        inc(TheTotalOfThisNumber);
        with ThePointers[t] do
        begin
          // Check if CRC32 is valid!
          CurrentCRC32:=$ffffffff;
          CalcCRC32(procedureData1, lengte1, CurrentCRC32);
          CalcCRC32(procedureData2, lengte2, CurrentCRC32);
          CalcCRC32(procedureData3, lengte3, CurrentCRC32);
          if CurrentCRC32=CRC32 then
          begin
            procedureVariable(procedureData1, procedureData2, procedureData3);
            try
              FreeMem(procedureData1);
            finally
              procedureData1:=nil;
            end;
            try
              FreeMem(procedureData2);
            finally
              procedureData2:=nil;
            end;
            try
              FreeMem(procedureData3);
            finally
              procedureData3:=nil;
            end;
            MemoryAllocationSize:=MemoryAllocationSize-lengte1-lengte2-lengte3-3*SizeOf(Longint)-SizeOf(Pointer);
          end
          else
          begin
            ShowMessage('CRC32 check: corrupted data encountered!');
          end;
        end;
        dec(t);
      until (t=0) or (ThePointers[t].Number<CurrentPointer);
  // Voeg TheTempPointer in ThePointer
      if TheTempNumber>0 then
      begin
        if TheTempNumber<>TheTotalOfThisNumber then
        begin
          LastOfThisNumber:=t+TheTotalOfThisNumber;
          NumberOfPointers:=NumberOfPointers+TheTempNumber-TheTotalOfThisNumber;
          if TheTempNumber>TheTotalOfThisNumber then
          begin // uitbreiden
            ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
            for u:=NumberOfPointers-TheTempNumber+TheTotalOfThisNumber downto LastOfThisNumber
            do ThePointers[u+TheTempNumber-TheTotalOfThisNumber]:=ThePointers[u];
          end
          else
          begin // inkrimpen
            if LastOfThisNumber<NumberOfPointers-TheTempNumber+TheTotalOfThisNumber then
            for u:= LastOfThisNumber+1 to NumberOfPointers-TheTempNumber+TheTotalOfThisNumber
            do ThePointers[u+TheTempNumber-TheTotalOfThisNumber]:=ThePointers[u];
            ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
          end
        end;
        i:=TheTempNumber;
        repeat
          inc(t);
          ThePointers[t]:=TheTempPointers[i];
          dec(i);
        until i=0;
        FreeMem(TheTempPointers);
        TheTempPointers:=nil;
      end;
      dec(CurrentPointer);
      InUndoMode:=false;
    end;
    CheckLimits;
    SetMenuItems;
    PossibleGroupUndo:=False;
  end;
end;

function ThlcUndoRedoRepeat.DoRepeat:boolean;
var
  i, t : integer;
  CurrentCRC32: DWORD;
begin
  result:=false;
  if Active then
  begin
    if Updating
    then ShowMessage('Can''t Repeat, in updating mode!')
    else
    begin
      if NumberOfRepeatPointers>0 then
      begin
        TheTempNumber:=0;
        InRepeatMode:=true;
        result:=true;
        for t:=1 to NumberOfRepeatPointers do with TheRepeatPointers[t] do
        begin
          // Check if CRC32 is valid!
          CurrentCRC32:=$ffffffff;
          CalcCRC32(procedureData1, lengte1, CurrentCRC32);
          CalcCRC32(procedureData2, lengte2, CurrentCRC32);
          CalcCRC32(procedureData3, lengte3, CurrentCRC32);
          if CurrentCRC32=CRC32 then
          begin
            procedureVariable(procedureData1, procedureData2, procedureData3);
          end
          else
          begin
            ShowMessage('CRC32 check: corrupted data encountered!');
          end;
        end;
        InRepeatMode:=false;
      end;
  // Voeg TheTempPointer in ThePointer
      if TheTempNumber>0 then
      begin
        CurrentPointer:=ThePointers[NumberOfPointers].Number+1;
        NumberOfPointers:=NumberOfPointers+TheTempNumber{-TheTotalOfThisNumber};
        begin // uitbreiden
          ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
        end
      end;
      t:=NumberOfPointers-TheTempNumber;
      i:=TheTempNumber;
      repeat
        inc(t);
        ThePointers[t]:=TheTempPointers[i];
        dec(i);
      until i=0;
      FreeMem(TheTempPointers);
      TheTempPointers:=nil;
      TheTempNumber:=0;
      if NumberOfNames<CurrentPointer
      then ReAllocMem(TheNames, CurrentPointer*SizeOf(TName));
      NumberOfNames:=CurrentPointer;
      TheNames[CurrentPointer]:=TheNames[CurrentPointer-1];
      LastPointer:=CurrentPointer;
    end;
    CheckLimits;
    SetMenuItems;
  end;
end;

function ThlcUndoRedoRepeat.Redo:boolean;
var
  LastOfThisNumber, TheTotalOfThisNumber, u, i, t : integer;
  CurrentCRC32: DWORD;
begin
  result:=false;
  if Active then
  begin
    if Updating
    then ShowMessage('Can''t Redo, in updating mode!')
    else if CanRedo then
    begin
      InRedoMode:=true;
      FreeRepeatPointers;
      TheTempNumber:=0;
      TheTotalOfThisNumber:=0;
      t:=1;
      while ThePointers[t].Number<=CurrentPointer do inc(t);
      result:=true;
      repeat
        inc(TheTotalOfThisNumber);
        with ThePointers[t] do
        begin
          // Check if CRC32 is valid!
          CurrentCRC32:=$ffffffff;
          CalcCRC32(procedureData1, lengte1, CurrentCRC32);
          CalcCRC32(procedureData2, lengte2, CurrentCRC32);
          CalcCRC32(procedureData3, lengte3, CurrentCRC32);
          if CurrentCRC32=CRC32 then
          begin
            procedureVariable(procedureData1, procedureData2, procedureData3);
            try
              FreeMem(procedureData1);
            finally
              procedureData1:=nil;
            end;
            try
              FreeMem(procedureData2);
            finally
              procedureData2:=nil;
            end;
            try
              FreeMem(procedureData3);
            finally
              procedureData3:=nil;
            end;
            MemoryAllocationSize:=MemoryAllocationSize-lengte1-lengte2-lengte3-3*SizeOf(Longint)-SizeOf(Pointer);
          end
          else
          begin
            ShowMessage('CRC32 check: corrupted data encountered!');
          end;
        end;
        inc(t);
      until (t>NumberOfPointers) or (ThePointers[t].Number>CurrentPointer+1);
  // Voeg TheTempPointer in ThePointer
      t:=t-TheTotalOfThisNumber-1;
      if TheTempNumber>0 then
      begin
        if TheTempNumber<>TheTotalOfThisNumber then
        begin
          LastOfThisNumber:=t+TheTotalOfThisNumber;
          NumberOfPointers:=NumberOfPointers+TheTempNumber-TheTotalOfThisNumber;
          if TheTempNumber>TheTotalOfThisNumber then
          begin // uitbreiden
            ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
            if LastOfThisNumber<NumberOfPointers-TheTempNumber+TheTotalOfThisNumber then
            for u:=NumberOfPointers-TheTempNumber+TheTotalOfThisNumber downto LastOfThisNumber+1
            do ThePointers[u+TheTempNumber-TheTotalOfThisNumber]:=ThePointers[u];
          end
          else
          begin // inkrimpen
            if LastOfThisNumber<NumberOfPointers-TheTempNumber+TheTotalOfThisNumber then
            for u:= LastOfThisNumber+1 to NumberOfPointers-TheTempNumber+TheTotalOfThisNumber
            do ThePointers[u+TheTempNumber-TheTotalOfThisNumber]:=ThePointers[u];
            ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
          end
        end;
        i:=0;
        repeat
          inc(i);
          inc(t);
          ThePointers[t]:=TheTempPointers[i];
        until i=TheTempNumber;
        FreeMem(TheTempPointers);
        TheTempPointers:=nil;
      end;
      inc(CurrentPointer);
      InRedoMode:=false;
    end;
    CheckLimits;
    SetMenuItems;
    PossibleGroupUndo:=False;
  end;
end;

function ThlcUndoRedoRepeat.CanRepeat:boolean;
begin
  result:=(NumberOfRepeatPointers>0) and (not Updating) and (Active);
end;

function ThlcUndoRedoRepeat.CanUndo:boolean;
begin
  result:=(CurrentPointer<>0) and (not Updating) and (Active);
end;

function ThlcUndoRedoRepeat.CanRedo:boolean;
begin
  result:=(LastPointer<>CurrentPointer) and (not Updating) and (Active);
end;

procedure ThlcUndoRedoRepeat.UndoBeginUpdate(TheName : TName);
var
  i, t : integer;
begin
  if Active then
  begin
    if Updating
    then ShowMessage('Already updating the undo option!')
    else
    if InRedoMode or InUndoMode or InRepeatMode
    then ShowMessage('Already in some other undo mode!')
    else
    begin
      SetMenuItems;
      FreeRepeatPointers;
      if (CurrentPointer<>LastPointer) and (NumberOfPointers>0) then
      begin
        t:=1;
        while ThePointers[t].Number<=CurrentPointer do inc(t);
        for i:=t to NumberOfPointers do with ThePointers[i] do
        begin
          FreeMem(procedureData1);
          FreeMem(procedureData2);
          FreeMem(procedureData3);
          MemoryAllocationSize:=MemoryAllocationSize-lengte1-lengte2-lengte3-3*SizeOf(Longint)-SizeOf(Pointer);
        end;
        NumberOfPointers:=t-1;
        ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
      end;
      if (CurrentPointer>0) and (AllowGroupUndo) then
      begin
        if (TheNames[CurrentPointer]=TheName) and (PossibleGroupUndo)
        then
        begin
        end
        else inc(CurrentPointer);
      end
      else inc(CurrentPointer);
      if NumberOfNames<CurrentPointer
      then ReAllocMem(TheNames, CurrentPointer*SizeOf(TName));
      NumberOfNames:=CurrentPointer;
      TheNames[CurrentPointer]:=TheName;
      LastPointer:=CurrentPointer;
      Updating:=true;
      Started:=false;
    end;
  end;
end;

procedure ThlcUndoRedoRepeat.UndoEndUpdate;
begin
  if Active then
  begin
    if NumberOfPointers=0 then
    begin
      CurrentPointer:=0;
      LastPointer:=0;
      FreeRepeatPointers;
    end
    else
      if ThePointers[NumberOfPointers].Number>CurrentPointer
      then
      begin
        LastPointer:=CurrentPointer;
        FreeRepeatPointers;
      end
      else
      begin
        CurrentPointer:=ThePointers[NumberOfPointers].Number;
        LastPointer:=ThePointers[NumberOfPointers].Number;
      end;
    Updating:=false;
    // Check if MemoryAllocationSize does not exceed the maximum size.
    // Check if LastPointer does not exceed the maximum count.
    CheckLimits;
    SetMenuItems;
    PossibleGroupUndo:=True;
  end;
end;

procedure ThlcUndoRedoRepeat.RemoveFirstPointers;
var
  t, TheSameNumber, Count : integer;
begin
  if CurrentPointer<=0 then
  begin
    // Something really went wrong!!!
    NumberOfPointers:=0;
    CurrentPointer:=0;
    LastPointer:=0;
    MemoryAllocationSize:=0;
  end;
  if CurrentPointer>1 then
  begin
    t:=1;
    TheSameNumber:=ThePointers[t].Number;
    while ThePointers[t].Number=TheSameNumber do inc(t);
    dec(t);
    if t<NumberOfPointers then
    begin
      for Count:=1 to t do
      with ThePointers[Count] do
      begin
        FreeMem(procedureData1);
        FreeMem(procedureData2);
        FreeMem(procedureData3);
        MemoryAllocationSize:=MemoryAllocationSize-lengte1-lengte2-lengte3-3*SizeOf(Longint)-SizeOf(Pointer);
      end;
      for Count:=t+1 to NumberOfPointers do
      begin
        ThePointers[Count-t]:=ThePointers[Count];
        dec(ThePointers[Count-t].Number);
        //:=ThePointers[Count-t].Number-1;
      end;
      Dec(NumberOfPointers,t);
      Dec(CurrentPointer);
      Dec(LastPointer);
      ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
      for Count:=1 to NumberOfNames-1 do
      begin
        TheNames[Count]:=TheNames[Count+1];
      end;
      dec(NumberOfNames);
    end;
  end
  else
  begin
    FreeMem(TheNames);
    TheNames:=nil;
    NumberOfNames:=0;
    if NumberOfPointers>0
    then for t:=1 to NumberOfPointers do
    begin
      FreeMem(ThePointers[t].procedureData1);
      FreeMem(ThePointers[t].procedureData2);
      FreeMem(ThePointers[t].procedureData3);
    end;
    FreeMem(ThePointers);
    NumberOfPointers:=0;
    CurrentPointer:=0;
    LastPointer:=0;
    MemoryAllocationSize:=0;
    ThePointers:=nil;
  end;
end;

procedure ThlcUndoRedoRepeat.CheckLimits;
begin
  while ((MemoryAllocationSize>MaxSize) and (MaxSize<>0)) do
  begin
    RemoveFirstPointers;
  end;
  while ((LastPointer>MaxCount) and (MaxCount<>0)) do
  begin
    RemoveFirstPointers;
  end;
end;

function ThlcUndoRedoRepeat.IsUpdating:boolean;
begin
  result:=Updating;
end;

procedure ThlcUndoRedoRepeat.SetMenuItems;
begin
  if Assigned(MenuItemUndo) then
  begin
    MenuItemUndo.Enabled:=CanUndo;
    MenuItemUndo.Caption:='&Undo';
    if CanUndo
    then MenuItemUndo.Caption:=MenuItemUndo.Caption+' '+GetUndoName;
  end;
  if Assigned(MenuItemRedo) then
  begin
    MenuItemRedo.Enabled:=CanRedo;
    MenuItemRedo.Caption:='&Redo';
    if CanRedo
    then MenuItemRedo.Caption:=MenuItemRedo.Caption+' '+GetRedoName;
  end;
  if Assigned(MenuItemRepeat) then
  begin
    MenuItemRepeat.Enabled:=CanRepeat;
    MenuItemRepeat.Caption:='R&epeat';
    if CanRepeat
    then MenuItemRepeat.Caption:=MenuItemRepeat.Caption+' '+GetRepeatName;
  end;
end;

procedure ThlcUndoRedoRepeat.LoadFromStream(Stream: TStream);
var
  t : integer;
  UARH: ThlcUndoRedoRepeatfileHeader;
  UARPH : ThlcUndoRedoRepeatPointerRecord;
  UARNH : ThlcUndoRedoRepeatNameRecord;
begin
  if not (Updating or InUndoMode or InRedoMode) then
  begin
    Reset;
    Stream.Read(UARH, SizeOf(UARH));
    NumberOfPointers:=UARH.NumberOfPoints;
    if (NumberOfPointers>0) and (UARH.Version<256*256) then
    begin
      CurrentPointer:=UARH.CurrentPointer;
      LastPointer:=UARH.LastPointer;
      NumberOfNames:=UARH.NumberOfNames;
      TheNames:=AllocMem(NumberOfNames);
      for t:=1 to NumberOfNames do
      begin
        Stream.Read(UARNH, SizeOf(UARNH));
        TheNames[t]:=UARNH.Name;
      end;
      MemoryAllocationSize:=0;
      ThePointers:=AllocMem(NumberOfPointers*SizeOf(TPointerRecord));
      for t:=1 to NumberOfPointers do
      begin
        Stream.Read(UARPH, SizeOf(UARPH));
        ThePointers[t].procedureVariable:=UARPH.procedureVariable;
        ThePointers[t].lengte1:=UARPH.Size1;
        ThePointers[t].lengte2:=UARPH.Size2;
        ThePointers[t].lengte3:=UARPH.Size3;
        ThePointers[t].Number:=UARPH.Number;
        ThePointers[t].procedureData1:=AllocMem(ThePointers[t].lengte1);
        ThePointers[t].procedureData2:=AllocMem(ThePointers[t].lengte2);
        ThePointers[t].procedureData3:=AllocMem(ThePointers[t].lengte3);
        Stream.Read(ThePointers[t].procedureData1^, ThePointers[t].lengte1);
        Stream.Read(ThePointers[t].procedureData2^, ThePointers[t].lengte2);
        Stream.Read(ThePointers[t].procedureData3^, ThePointers[t].lengte3);
        ThePointers[t].CRC32:=$ffffffff;
        CalcCRC32(ThePointers[t].procedureData1, ThePointers[t].lengte1, ThePointers[t].CRC32);
        CalcCRC32(ThePointers[t].procedureData2, ThePointers[t].lengte2, ThePointers[t].CRC32);
        CalcCRC32(ThePointers[t].procedureData3, ThePointers[t].lengte3, ThePointers[t].CRC32);
        MemoryAllocationSize:=MemoryAllocationSize+ThePointers[t].lengte1+ThePointers[t].lengte2+ThePointers[t].lengte3+3*SizeOf(Longint)+SizeOf(Pointer);
      end;
    end;
    SetMenuItems;
  end;
end;

procedure ThlcUndoRedoRepeat.SaveToStream(Stream: TStream);
var
  t : integer;
  UARH: ThlcUndoRedoRepeatfileHeader;
  UARPH : ThlcUndoRedoRepeatPointerRecord;
  UARNH : ThlcUndoRedoRepeatNameRecord;
begin
  if not (Updating or InUndoMode or InRedoMode) then
  begin
    UARH.Version:=256+8;
    UARH.VersionInfo[ 1]:='H';
    UARH.VersionInfo[ 2]:='i';
    UARH.VersionInfo[ 3]:='g';
    UARH.VersionInfo[ 4]:='h';
    UARH.VersionInfo[ 5]:='-';
    UARH.VersionInfo[ 6]:='L';
    UARH.VersionInfo[ 7]:='o';
    UARH.VersionInfo[ 8]:='g';
    UARH.VersionInfo[ 9]:='i';
    UARH.VersionInfo[10]:='c';
    UARH.NumberOfPoints:=NumberOfPointers;
    UARH.CurrentPointer:=CurrentPointer;
    UARH.LastPointer:=LastPointer;
    UARH.NumberOfNames:=NumberOfNames;
    UARH.Reserved:=0;
    Stream.Write(UARH, SizeOf(UARH));
    if (NumberOfPointers>0) then
    begin
      for t:=1 to NumberOfNames do
      begin
        UARNH.Name:=TheNames[t];
        Stream.Write(UARNH, SizeOf(UARNH));
      end;
      for t:=1 to NumberOfPointers do
      begin
        UARPH.procedureVariable:=ThePointers[t].procedureVariable;
        UARPH.Size1:=ThePointers[t].lengte1;
        UARPH.Size2:=ThePointers[t].lengte2;
        UARPH.Size3:=ThePointers[t].lengte3;
        UARPH.Number:=ThePointers[t].Number;
        Stream.Write(UARPH, SizeOf(UARPH));
        Stream.Write(ThePointers[t].procedureData1^, ThePointers[t].lengte1);
        Stream.Write(ThePointers[t].procedureData2^, ThePointers[t].lengte2);
        Stream.Write(ThePointers[t].procedureData3^, ThePointers[t].lengte3);
      end;
    end;
    SetMenuItems;
  end;
end;

procedure ThlcUndoRedoRepeat.LoadFromFile(const Filename: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename, fmOpenRead);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure ThlcUndoRedoRepeat.SaveToFile(const Filename: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

end.

