{*************************************************************************}
{                                                                         }
{  Capture example to illustrate the Capture Component                    }
{                                                                         }
{  Copyright (c) 1997, 1999 High-Logic [Erwin Denissen]                   }
{                                                                         }
{  There are no known bugs at the moment, so please notify us when you    }
{  find one. Make sure you are using the latest version of this software. }
{  email: erwind@high-logic.com                                           }
{                                                                         }
{*************************************************************************}

unit CaptureFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, Menus, ComCtrls, HlcCapture, JPeg;

type
  TCaptureForm = class(TForm)
    Panel1: TPanel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Capture2: TMenuItem;
    Area1: TMenuItem;
    Object1: TMenuItem;
    Client1: TMenuItem;
    Window1: TMenuItem;
    Screen1: TMenuItem;
    N1: TMenuItem;
    ActiveWindow1: TMenuItem;
    N2: TMenuItem;
    Settings1: TMenuItem;
    ScrollBox1: TScrollBox;
    Image1: TImage;
    StatusBar1: TStatusBar;
    SaveAs1: TMenuItem;
    N3: TMenuItem;
    Save1: TMenuItem;
    SaveDialog1: TSaveDialog;
    hlcCapture: ThlcCapture;
    f1: TMenuItem;
    Edit1: TMenuItem;
    Copy1: TMenuItem;
    procedure hlcCaptureCapture(Sender: TObject; bmpCapture: TBitmap);
    procedure Area1Click(Sender: TObject);
    procedure Object1Click(Sender: TObject);
    procedure Client1Click(Sender: TObject);
    procedure Window1Click(Sender: TObject);
    procedure Screen1Click(Sender: TObject);
    procedure ActiveWindow1Click(Sender: TObject);
    procedure Settings1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure hlcCaptureBeforeCapture(Sender: TObject; hwndSource: Integer;
      var Continue: Boolean);
    procedure hlcCaptureCaptureError(Sender: TObject; ErrorMessage: String);
    procedure f1Click(Sender: TObject);
    procedure Copy1Click(Sender: TObject);
  private
    { Private declarations }
    TheDelayTime: integer;
    TheVirtualKey: integer;
    KeyText : string;
    UseVirtual, UseDelay: boolean;
    procedure StartCapture(TheStyle : TCaptureStyle);
    procedure SaveAsJPG(TheFileName: string);
  public
    { Public declarations }
  end;

var
  CaptureForm: TCaptureForm;

implementation

uses AreaFrm, CaptureSettingsFrm, ClipBrd;

{$R *.DFM}

procedure TCaptureForm.StartCapture(TheStyle : TCaptureStyle);
begin
  hlcCapture.CaptureStyle:=TheStyle;
  if UseVirtual and UseDelay then
  begin
    StatusBar1.Panels[0].Text:='Capturing... Press '+KeyText+' to start,  or wait '+IntToStr(TheDelayTime)+' ms.';
    hlcCapture.StartCapture(TheVirtualKey, TheDelayTime);
  end
  else
  begin
    if UseVirtual then
    begin
      StatusBar1.Panels[0].Text:='Capturing... Press '+KeyText+' to start.';
      hlcCapture.StartCapture(TheVirtualKey, 0);
    end
    else
    begin
      if UseDelay then
      begin
        StatusBar1.Panels[0].Text:='Capturing... wait '+IntToStr(TheDelayTime)+' ms.';
        hlcCapture.StartCapture(0, TheDelayTime);
      end
      else
      begin
        hlcCapture.StartCapture(0, 0);
      end;
    end;
  end;
end;

procedure TCaptureForm.hlcCaptureCapture(Sender: TObject; bmpCapture: TBitmap);
begin
  Image1.Width:=bmpCapture.Width;
  Image1.Height:=bmpCapture.Height;
  Image1.Picture.Assign(bmpCapture);
  Application.BringToFront;
  SaveAs1.Enabled:=True;
  StatusBar1.Panels[0].Text:=StatusBar1.Panels[0].Text+' Captured.';
end;

procedure TCaptureForm.hlcCaptureBeforeCapture(Sender: TObject; hwndSource: Integer; var Continue: Boolean);
var
  buf1, buf2: array [0..255] of Char;
begin
  GetClassname(hwndSource, buf1, 255 );
  GetWindowText(hwndSource, buf2, 255 );
  StatusBar1.Panels[0].Text:='['+StrPas(buf1)+'] ['+StrPas(buf2)+']';
  SetForegroundWindow(hwndSource);
  Application.ProcessMessages;
  Sleep(50);
end;

procedure TCaptureForm.f1Click(Sender: TObject);
begin
  StartCapture(csArea);
end;

procedure TCaptureForm.Area1Click(Sender: TObject);
begin
  with AreaForm do
  if ShowModal=mrOk then
  begin
    hlcCapture.CaptureArea(Rect(SpinEditLeft.Value, SpinEditTop.Value, SpinEditLeft.Value+SpinEditWidth.Value, SpinEditTop.Value+SpinEditHeight.Value));
  end;
end;

procedure TCaptureForm.Object1Click(Sender: TObject);
begin
  StartCapture(csObject);
end;

procedure TCaptureForm.Client1Click(Sender: TObject);
begin
  StartCapture(csClientArea);
end;

procedure TCaptureForm.Window1Click(Sender: TObject);
begin
  StartCapture(csWindow);
end;

procedure TCaptureForm.Screen1Click(Sender: TObject);
begin
  StartCapture(csScreen);
end;

procedure TCaptureForm.ActiveWindow1Click(Sender: TObject);
begin
  StartCapture(csActiveWindow);
end;

function GetVkey(Value:integer):integer;
begin
  if Value=0
  then result:=VK_SNAPSHOT
  else result:=Value+$70-1;
end;

procedure TCaptureForm.Settings1Click(Sender: TObject);
begin
  with CaptureSettingsForm do
  if ShowModal=mrOk then
  begin
    KeyText:=ComboBox1.Items[ComboBox1.ItemIndex];
    hlcCapture.IncludeCursor:=cbIncludeMouse.Checked;
    TheDelayTime:=SpinEditTime.Value;;
    TheVirtualKey:=GetVkey(ComboBox1.ItemIndex);
    case RadioGroup1.ItemIndex of
      0 :
      begin
        UseDelay:=False;
        UseVirtual:=False;
      end;
      1 :
      begin
        UseDelay:=True;
        UseVirtual:=False;
      end;
      2 :
      begin
        UseDelay:=False;
        UseVirtual:=True;
      end;
      3 :
      begin
        UseDelay:=True;
        UseVirtual:=True;
      end;
    end;
  end;
end;

procedure TCaptureForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TCaptureForm.FormCreate(Sender: TObject);
begin
  hlcCapture.IncludeCursor:=True;
  UseVirtual:=True;
  UseDelay:=False;
  TheDelayTime:=2000;
  TheVirtualKey:=VK_SNAPSHOT;
  KeyText:='Print Screen';
end;

procedure TCaptureForm.Save1Click(Sender: TObject);
begin
  Image1.Picture.Graphic.SaveToFile(SaveDialog1.FileName);
end;

procedure TCaptureForm.SaveAsJPG(TheFileName: string);
var
  TheJPEGImage: TJPEGImage;
begin
  TheJPEGImage:=TJPEGImage.Create;
  try
    TheJPEGImage.Assign(Image1.Picture.Bitmap);
    TheJPEGImage.SaveToFile(TheFileName);
  finally
    TheJPEGImage.Free;
  end;
end;

procedure TCaptureForm.SaveAs1Click(Sender: TObject);
begin
  if SaveDialog1.Execute then
  begin
    if UpperCase(ExtractFileExt(SaveDialog1.FileName))='JPG'
    then SaveAsJPG(SaveDialog1.FileName)
    else Image1.Picture.Graphic.SaveToFile(SaveDialog1.FileName);
    Save1.Enabled:=True;
  end;
end;

procedure TCaptureForm.hlcCaptureCaptureError(Sender: TObject;
  ErrorMessage: String);
begin
  StatusBar1.Panels[0].Text:='Error occured: '+ErrorMessage;
end;

procedure TCaptureForm.Copy1Click(Sender: TObject);
var
  MyFormat: Word;
  AData: Cardinal;
  APalette: HPALETTE;
begin
  Image1.Picture.Bitmap.SaveToClipBoardFormat(MyFormat, AData, APalette);
  ClipBoard.SetAsHandle(MyFormat,AData);
end;

end.
