{*************************************************************************}
{                                                                         }
{  Dialog example to illustrate the UndoRedoRepeat Component              }
{                                                                         }
{  Example for Delphi 3 or higher                                         }
{                                                                         }
{  Copyright (c) 1997, 1999 High-Logic [Erwin Denissen]                   }
{                                                                         }
{  There are no known bugs at the moment, so please notify us when you    }
{  find one. Make sure you are using the latest version of this software. }
{  email: erwind@high-logic.com                                           }
{                                                                         }
{*************************************************************************}

unit DialogFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ToolWin, Menus, ExtCtrls, DeclUnit, HlcUndoRedoRepeat,
  ImgList;

type
  TDialogForm = class(TForm)
    ToolBar: TToolBar;
    OpenButton: TToolButton;
    SaveButton: TToolButton;
    PrintButton: TToolButton;
    ToolButton5: TToolButton;
    ToolButtonUndo: TToolButton;
    ToolButtonRedo: TToolButton;
    ToolButtonRepeat: TToolButton;
    ToolButton1: TToolButton;
    CutButton: TToolButton;
    CopyButton: TToolButton;
    PasteButton: TToolButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    FileNewItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileSaveItem: TMenuItem;
    FileSaveAsItem: TMenuItem;
    N5: TMenuItem;
    FilePrintItem: TMenuItem;
    N1: TMenuItem;
    FileExitItem: TMenuItem;
    Edit2: TMenuItem;
    EditUndoItem: TMenuItem;
    EditRedoItem: TMenuItem;
    EditRepeatItem: TMenuItem;
    N3: TMenuItem;
    EditDeleteItem: TMenuItem;
    Help1: TMenuItem;
    HelpAboutItem: TMenuItem;
    StatusBar: TStatusBar;
    Panel1: TPanel;
    Memo1: TMemo;
    ToolButton2: TToolButton;
    ToolButtonOptions: TToolButton;
    N4: TMenuItem;
    EditOptions: TMenuItem;
    Image1: TImage;
    Splitter1: TSplitter;
    ImageDefault: TImage;
    Advanced1: TMenuItem;
    AdvancedReset: TMenuItem;
    hlcUndoRedoRepeat: ThlcUndoRedoRepeat;
    ToolbarImages: TImageList;
    AdvancedDisable: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure EditOptionsClick(Sender: TObject);
    procedure EditUndoItemClick(Sender: TObject);
    procedure EditRedoItemClick(Sender: TObject);
    procedure EditRepeatItemClick(Sender: TObject);
    procedure FileExitItemClick(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure AdvancedResetClick(Sender: TObject);
    procedure HelpAboutItemClick(Sender: TObject);
    procedure AdvancedDisableClick(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateButtons;
    procedure ShowHint(Sender: TObject);
    procedure WMEraseBkgnd(var m: TWMEraseBkgnd); message WM_ERASEBKGND;
    function ChangeDialogOptions(p1, p2, p3 : pointer):pointer;
  public
    { Public declarations }
    procedure ApplyDialogOptions(TheOptions :TOptionsDlg);
    procedure DrawBackGround;
  end;

var
  DialogForm: TDialogForm;

implementation

{$R *.DFM}

uses OptionDlgFrm, HlcAbout;


var
  NewOptions, CurrentOptions: TOptionsDlg;

// since we're going to be painting the whole form, handling this
// message will suppress the uneccessary repainting of the background
// which can result in flicker.
procedure TDialogForm.WMEraseBkgnd(var m : TWMEraseBkgnd);
begin
  m.Result := LRESULT(False);
end;

procedure TDialogForm.ApplyDialogOptions(TheOptions :TOptionsDlg);
begin
  if NewOptions.ODLayoutItem<>TheOptions.ODLayoutItem then
  begin
    if TheOptions.ODLayoutItem=0 then
    begin
      Image1.Picture.Bitmap.PixelFormat:=pf24bit;
      Image1.Picture.Bitmap.Width:=48;
      Image1.Picture.Bitmap.Height:=48;
      Image1.Picture.Bitmap.Canvas.CopyMode:=cmWhiteness;
      Image1.Picture.Bitmap.Canvas.Pen.Color:=clBtnFace;
      Image1.Picture.Bitmap.Canvas.Brush.Color:=clBtnFace;
      Image1.Picture.Bitmap.Canvas.Rectangle(0,0, 48, 48);
    end;
    if TheOptions.ODLayoutItem=1 then
    begin
      Image1.Picture.Bitmap.Assign(ImageDefault.Picture.Bitmap);
    end;
    if TheOptions.ODLayoutItem>1 then
    begin
      Image1.Picture.LoadFromFile(TheOptions.ODLayoutName);
    end;
    DrawBackGround;
  end;
  if NewOptions.ODFontName<>TheOptions.ODFontName then
  begin
    Memo1.Font.Name:=TheOptions.ODFontName;
  end;
  if NewOptions.ODFontStyle<>TheOptions.ODFontStyle then
  begin
    Memo1.Font.Style:=TheOptions.ODFontStyle;
  end;
  if NewOptions.ODFontSize<>TheOptions.ODFontSize then
  begin
    Memo1.Font.Size:=TheOptions.ODFontSize;
  end;
    if NewOptions.ODFontColor<>TheOptions.ODFontColor then
  begin
    Memo1.Font.Color:=TheOptions.ODFontColor;
  end;
  NewOptions:=TheOptions;
end;

procedure TDialogForm.UpdateButtons;
begin
  ToolButtonUndo.Enabled:=EditUndoItem.Enabled;
  if ToolButtonUndo.Enabled
  then ToolButtonUndo.Hint:='Undo '+hlcUndoRedoRepeat.GetUndoName
  else ToolButtonUndo.Hint:='Can''t undo.';
  EditUndoItem.Hint:=ToolButtonUndo.Hint;
  ToolButtonRedo.Enabled:=EditRedoItem.Enabled;
  if ToolButtonRedo.Enabled
  then ToolButtonRedo.Hint:='Redo '+hlcUndoRedoRepeat.GetRedoName
  else ToolButtonRedo.Hint:='Can''t redo.';
  EditRedoItem.Hint:=ToolButtonRedo.Hint;
  ToolButtonRepeat.Enabled:=EditRepeatItem.Enabled;
  if ToolButtonRepeat.Enabled
  then ToolButtonRepeat.Hint:='Repeat '+hlcUndoRedoRepeat.GetRepeatName
  else ToolButtonRepeat.Hint:='Can''t repeat.';
  EditRepeatItem.Hint:=ToolButtonRepeat.Hint;
  if ToolButtonRepeat.Enabled
  then StatusBar.Panels[0].Text:='Repeat allowed.'
  else StatusBar.Panels[0].Text:='Repeat not available.';
  StatusBar.Panels[1].Text:=IntToStr(hlcUndoRedoRepeat.MemoryAllocationSize div 1024)+'KB.';
  if CurrentOptions.ODLayoutName<>''
  then StatusBar.Panels[2].Text:='background: '+CurrentOptions.ODLayoutName
  else StatusBar.Panels[2].Text:='background: default';
  AdvancedReset.Enabled:=hlcUndoRedoRepeat.CanUndo or hlcUndoRedoRepeat.CanRedo or hlcUndoRedoRepeat.CanRepeat;
end;

procedure TDialogForm.FormCreate(Sender: TObject);
begin
  hlcUndoRedoRepeat.Reset;
  UpdateButtons;
  Application.OnHint := ShowHint;
  Image1.Picture.Bitmap.Assign(ImageDefault.Picture.Bitmap);
  with CurrentOptions do
  begin
    ODLayoutItem:=1;
    ODLayoutName:='';
    ODFontName:=Memo1.Font.Name;
    ODFontStyle:=Memo1.Font.Style;
    ODFontSize:=Memo1.Font.Size;
    ODFontColor:=Memo1.Font.Color;
  end;
end;

procedure TDialogForm.ShowHint(Sender: TObject);
begin
  if Length(Application.Hint) > 0 then
  begin
    StatusBar.SimplePanel := True;
    StatusBar.SimpleText := Application.Hint;
  end
  else StatusBar.SimplePanel := False;
end;

function TDialogForm.ChangeDialogOptions(p1, p2, p3 : pointer):pointer;
begin
  hlcUndoRedoRepeat.Add(ChangeDialogOptions, @CurrentOptions, nil, nil, SizeOf(CurrentOptions), 0, 0);
  ApplyDialogOptions(TOptionsDlg(p1^));
  CurrentOptions:=TOptionsDlg(p1^);
  Result:=nil;
end;

procedure TDialogForm.EditOptionsClick(Sender: TObject);
var
  i : integer;
begin
  NewOptions:=CurrentOptions;
  with CurrentOptions do
  begin
    OptionDlgForm.ListBox1.Clear;
    OptionDlgForm.ListBox1.Items.Assign(OptionDlgForm.FileListBox1.Items);
    OptionDlgForm.ListBox1.Items.Insert(0, 'Default');
    OptionDlgForm.ListBox1.Items.Insert(0, 'None');
    if ODLayoutItem>1 then
    begin
      for i:=0 to OptionDlgForm.ListBox1.Items.Count-1 do
      if OptionDlgForm.ListBox1.Items[i]=ODLayoutName
      then OptionDlgForm.ListBox1.ItemIndex:=i;
    end
    else
    begin
      OptionDlgForm.ListBox1.ItemIndex:=ODLayoutItem;
    end;
    for i:=0 to OptionDlgForm.ListBox2.Items.Count-1 do
      if OptionDlgForm.ListBox2.Items[i]=ODFontName
      then OptionDlgForm.ListBox2.ItemIndex:=i;
    OptionDlgForm.CheckBox1.Checked:= [fsBold]       <= ODFontStyle;
    OptionDlgForm.CheckBox2.Checked:= [fsItalic]     <= ODFontStyle;
    OptionDlgForm.CheckBox3.Checked:= [fsUnderline]  <= ODFontStyle;
    OptionDlgForm.CheckBox4.Checked:= [fsStrikeout]  <= ODFontStyle;
    OptionDlgForm.UpDown1.Position:=ODFontSize;
    OptionDlgForm.Panel1.Color:=ODFontColor;
  end;
  if OptionDlgForm.ShowModal=mrok then
  begin
    begin
      hlcUndoRedoRepeat.UndoBeginUpdate('Form Properties');
      hlcUndoRedoRepeat.Add(ChangeDialogOptions, @CurrentOptions, nil, nil, SizeOf(CurrentOptions)+200, 0, 0);
      hlcUndoRedoRepeat.UndoEndUpdate;
      CurrentOptions:=OptionDlgForm.ApplyOptions;
      UpdateButtons;
    end;
  end
  else
  begin
    ApplyDialogOptions(CurrentOptions);
  end;
end;

procedure TDialogForm.EditUndoItemClick(Sender: TObject);
begin
  // This is all you have to do, to undo previous actions:
  hlcUndoRedoRepeat.Undo;
  UpdateButtons;
end;

procedure TDialogForm.EditRedoItemClick(Sender: TObject);
begin
  // This is all you have to do:
  hlcUndoRedoRepeat.Redo;
  UpdateButtons;
end;

procedure TDialogForm.EditRepeatItemClick(Sender: TObject);
begin
  hlcUndoRedoRepeat.DoRepeat;
  UpdateButtons;
end;

procedure TDialogForm.DrawBackGround;
var
  x, y : integer;
begin
  Canvas.CopyMode:=cmSrcCopy;
  for x:=0 to (Width div Image1.Width) do
    for y:=0 to (Height div Image1.Height) do
    Canvas.Draw(x*Image1.Width, y*Image1.Height, Image1.Picture.Bitmap);
end;

procedure TDialogForm.FileExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TDialogForm.FormPaint(Sender: TObject);
begin
  DrawBackGround;
end;

procedure TDialogForm.AdvancedResetClick(Sender: TObject);
begin
  hlcUndoRedoRepeat.Reset;
  UpdateButtons;
end;

procedure TDialogForm.HelpAboutItemClick(Sender: TObject);
begin
  with HlcAboutBox do
  begin
    if hlcUndoRedoRepeat.MemoryAllocationSize=0
    then UsedByComponent.Caption := '0 Bytes'
    else UsedByComponent.Caption := FormatFloat('#,###" Bytes"', hlcUndoRedoRepeat.MemoryAllocationSize);
    LabelUsed2.Caption := '20';
    ShowModal;
  end;
end;

procedure TDialogForm.AdvancedDisableClick(Sender: TObject);
begin
  AdvancedDisable.Checked:=not AdvancedDisable.Checked;
  hlcUndoRedoRepeat.Active:=not AdvancedDisable.Checked;
end;

end.
