{*************************************************************************}
{                                                                         }
{  Drawing example to illustrate the UndoRedoRepeat Component             }
{                                                                         }
{  Example for Delphi 3 or higher                                         }
{                                                                         }
{  Copyright (c) 1997, 1999 High-Logic [Erwin Denissen]                   }
{                                                                         }
{  There are no known bugs at the moment, so please notify us when you    }
{  find one. Make sure you are using the latest version of this software. }
{  email: erwind@high-logic.com                                           }
{                                                                         }
{*************************************************************************}

unit DrawingFrm;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, ExtCtrls, StdCtrls, ComCtrls, Menus, ToolWin,
  HlcUndoRedoRepeat, ImgList;

type
  TDrawingTool = (dtLine, dtRectangle, dtEllipse, dtRoundRect);
  TDrawingForm = class(TForm)
    ScrollBox1: TScrollBox;
    Image: TImage;
    ColorDialog1: TColorDialog;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    New1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    Saveas1: TMenuItem;
    Print1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    hlcUndoRedoRepeat: ThlcUndoRedoRepeat;
    Edit1: TMenuItem;
    EditRepeatItem: TMenuItem;
    EditRedoItem: TMenuItem;
    EditUndoItem: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    Advanced1: TMenuItem;
    GroupUndo1: TMenuItem;
    AdvancedReset: TMenuItem;
    ToolBar: TToolBar;
    OpenButton: TToolButton;
    SaveButton: TToolButton;
    PrintButton: TToolButton;
    ToolButton5: TToolButton;
    ToolButtonUndo: TToolButton;
    ToolButtonRedo: TToolButton;
    ToolButtonRepeat: TToolButton;
    ToolButton1: TToolButton;
    CutButton: TToolButton;
    CopyButton: TToolButton;
    PasteButton: TToolButton;
    ToolbarImages: TImageList;
    ToolBarBrush: TToolBar;
    ToolButtonSolidBrush: TToolButton;
    ToolButtonClearBrush: TToolButton;
    ToolButtonHorizontalBrush: TToolButton;
    ToolButtonVerticalBrush: TToolButton;
    ToolButtonFDiagonalBrush: TToolButton;
    ToolButtonBDiagonalBrush: TToolButton;
    ToolButtonCrossBrush: TToolButton;
    ToolButtonDiagCrossBrush: TToolButton;
    ImageList1: TImageList;
    ToolBarShape: TToolBar;
    ToolButtonLine: TToolButton;
    ToolButtonRectangle: TToolButton;
    ToolButtonEllispe: TToolButton;
    ToolButtonRoundRect: TToolButton;
    StatusBar: TStatusBar;
    ToolButton15: TToolButton;
    ToolButtonPen: TToolButton;
    ToolButtonBrush: TToolButton;
    ToolButton18: TToolButton;
    ToolBarPen: TToolBar;
    ToolButtonSolidPen: TToolButton;
    ToolButtonDashPen: TToolButton;
    ToolButtonDotPen: TToolButton;
    ToolButtonDashDotPen: TToolButton;
    ToolButtonDashDotDotPen: TToolButton;
    ToolButtonClearPen: TToolButton;
    ImageList2: TImageList;
    PenSize: TEdit;
    PenWidth: TUpDown;
    ToolButton14: TToolButton;
    PenPanel: TPanel;
    ToolButton16: TToolButton;
    ImageList3: TImageList;
    BrushPanel: TPanel;
    ToolButton6: TToolButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure SetDrawingTool(Sender: TObject);
    procedure SetPenStyle(Sender: TObject);
    procedure PenSizeChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SetBrushStyle(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Saveas1Click(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure EditUndoItemClick(Sender: TObject);
    procedure EditRedoItemClick(Sender: TObject);
    procedure EditRepeatItemClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure About1Click(Sender: TObject);
    procedure GroupUndo1Click(Sender: TObject);
    procedure AdvancedResetClick(Sender: TObject);
    procedure ToolButtonPenClick(Sender: TObject);
    procedure ToolButtonBrushClick(Sender: TObject);
    procedure PenPanelClick(Sender: TObject);
    procedure BrushPanelClick(Sender: TObject);
  private
    { Private declarations }
    function CanContinue:Word;
    procedure ShowHint(Sender: TObject);
    procedure UpdateToolBarButtons;
  public
    { Public declarations }
    BrushStyle: TBrushStyle;
    PenStyle: TPenStyle;
    PenWide: Integer;
    Drawing: Boolean;
    Origin, MovePt: TPoint;
    DrawingTool: TDrawingTool;
    CurrentFile: string;
    procedure SaveStyles;
    procedure RestoreStyles;
    procedure DrawShape(TopLeft, BottomRight: TPoint; AMode: TPenMode; Final: boolean);
    function InsItem(TheData1, TheData2, TheData3: pointer):pointer;
    function DelItem(TheData1, TheData2, TheData3: pointer):pointer;
    procedure CompleteRedraw;

  end;

var
  DrawingForm: TDrawingForm;

implementation

uses NewBMPFrm, HlcAbout;

{$R *.DFM}

const
  MaxNumberOfObjects = 200;

type

  TAddObject =
  record
    RTopLeft, RBottomRight: TPoint;
    RDrawingTool: TDrawingTool;
    RPenStyle : TPenStyle;
    RBrushStyle : TBrushStyle;
    RPenColor : TColor;
    RBrushColor : TColor;
    RPenWidth : integer;
  end;

  TObjectRecord = TAddObject;
  TObjectArrayRecord = array [1..MaxNumberOfObjects] of TObjectRecord;

var
  NumberOfObjects : integer;
  TheObjectArray : TObjectArrayRecord;
  MyPenStyle : TPenStyle;
  MyBrushStyle : TBrushStyle;
  MyPenColor : TColor;
  MyBrushColor : TColor;
  TempBitmap: TBitmap;

procedure TDrawingForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Drawing := True;
  Image.Canvas.MoveTo(X, Y);
  Origin := Point(X, Y);
  MovePt := Origin;
  StatusBar.Panels[0].Text := Format('Origin: (%d, %d)', [X, Y]);
end;

procedure TDrawingForm.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Drawing then
  begin
    DrawShape(Origin, MovePt, pmNotXor, False);
    MovePt := Point(X, Y);
    DrawShape(Origin, MovePt, pmCopy, True);
    Drawing := False;
  end;
  UpdateToolBarButtons;
end;

procedure TDrawingForm.UpdateToolBarButtons;
begin
  ToolButtonUndo.Enabled:=EditUndoItem.Enabled;
  if ToolButtonUndo.Enabled
  then ToolButtonUndo.Hint:='Undo '+hlcUndoRedoRepeat.GetUndoName
  else ToolButtonUndo.Hint:='Can''t undo.';
  EditUndoItem.Hint:=ToolButtonUndo.Hint;
  ToolButtonRedo.Enabled:=EditRedoItem.Enabled;
  if ToolButtonRedo.Enabled
  then ToolButtonRedo.Hint:='Redo '+hlcUndoRedoRepeat.GetRedoName
  else ToolButtonRedo.Hint:='Can''t redo.';
  EditRedoItem.Hint:=ToolButtonRedo.Hint;
  ToolButtonRepeat.Enabled:=EditRepeatItem.Enabled;
  if ToolButtonRepeat.Enabled
  then ToolButtonRepeat.Hint:='Repeat '+hlcUndoRedoRepeat.GetRepeatName
  else ToolButtonRepeat.Hint:='Can''t repeat.';
  EditRepeatItem.Hint:=ToolButtonRepeat.Hint;
  if ToolButtonRepeat.Enabled
  then StatusBar.Panels[0].Text:='Repeat allowed.'
  else StatusBar.Panels[0].Text:='Repeat not available.';
  AdvancedReset.Enabled:=hlcUndoRedoRepeat.CanUndo or hlcUndoRedoRepeat.CanRedo or hlcUndoRedoRepeat.CanRepeat;
end;

procedure TDrawingForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if Drawing then
  begin
    DrawShape(Origin, MovePt, pmNotXor, False);
    MovePt := Point(X, Y);
    DrawShape(Origin, MovePt, pmNotXor, False);
  end;
  StatusBar.Panels[1].Text := Format('Current: (%d, %d)', [X, Y]);
end;

procedure TDrawingForm.SetDrawingTool(Sender: TObject);
begin
  if Sender = ToolButtonLine then DrawingTool := dtLine
  else if Sender = ToolButtonRectangle then DrawingTool := dtRectangle
  else if Sender = ToolButtonEllispe then DrawingTool := dtEllipse
  else if Sender = ToolButtonRoundRect then DrawingTool := dtRoundRect;
end;

procedure TDrawingForm.CompleteRedraw;
var
  i : integer;
  Bitmap: TBitmap;
begin
  Bitmap := TBitmap.Create;
  Bitmap.Assign(TempBitmap);
  for i:=1 to NumberOfObjects do
  with Bitmap.Canvas, TheObjectArray[i] do
  begin
    Pen.Mode := pmCopy;
    Pen.Style:=RPenStyle;
    Pen.Color:=RPenColor;
    Pen.Width:=RPenWidth;
    Brush.Color:=RBrushColor;
    Brush.Style:=RBrushStyle;

    case RDrawingTool of
      dtLine:
        begin
          Bitmap.Canvas.MoveTo(RTopLeft.X, RTopLeft.Y);
          Bitmap.Canvas.LineTo(RBottomRight.X, RBottomRight.Y);
        end;
      dtRectangle: Bitmap.Canvas.Rectangle(RTopLeft.X, RTopLeft.Y, RBottomRight.X,
        RBottomRight.Y);
      dtEllipse: Bitmap.Canvas.Ellipse(RTopleft.X, RTopLeft.Y, RBottomRight.X,
        RBottomRight.Y);
      dtRoundRect: Bitmap.Canvas.RoundRect(RTopLeft.X, RTopLeft.Y, RBottomRight.X,
        RBottomRight.Y, (RTopLeft.X - RBottomRight.X) div 2,
        (RTopLeft.Y - RBottomRight.Y) div 2);
    end;
  end;
  SaveStyles;
  Image.Picture.Graphic := Bitmap;
  RestoreStyles;
end;

function TDrawingForm.InsItem(TheData1, TheData2, TheData3: pointer):pointer;
begin
  // Now put the opposite in de hlcUndoRedoRepeat object
  inc(NumberOfObjects);
  TheObjectArray[NumberOfObjects]:=TAddObject(TheData1^);
  hlcUndoRedoRepeat.Add(DelItem, nil, nil, nil, 0, 0, 0);
  // Actually perform the operation
  with Image.Canvas, TheObjectArray[NumberOfObjects] do
  begin
    Pen.Mode := pmCopy;
    Pen.Style:=RPenStyle;
    Pen.Color:=RPenColor;
    Pen.Width:=RPenWidth;
    Brush.Color:=RBrushColor;
    Brush.Style:=RBrushStyle;

    case RDrawingTool of
      dtLine:
        begin
          Image.Canvas.MoveTo(RTopLeft.X, RTopLeft.Y);
          Image.Canvas.LineTo(RBottomRight.X, RBottomRight.Y);
        end;
      dtRectangle: Image.Canvas.Rectangle(RTopLeft.X, RTopLeft.Y, RBottomRight.X,
        RBottomRight.Y);
      dtEllipse: Image.Canvas.Ellipse(RTopleft.X, RTopLeft.Y, RBottomRight.X,
        RBottomRight.Y);
      dtRoundRect: Image.Canvas.RoundRect(RTopLeft.X, RTopLeft.Y, RBottomRight.X,
        RBottomRight.Y, (RTopLeft.X - RBottomRight.X) div 2,
        (RTopLeft.Y - RBottomRight.Y) div 2);
    end;
  end;
  result:=nil;
end;

function TDrawingForm.DelItem(TheData1, TheData2, TheData3: pointer):pointer;
var
  TheUndoData : TAddObject;
begin
  // put the opposite in de hlcUndoRedoRepeat object
  TheUndoData:=TheObjectArray[NumberOfObjects];
  hlcUndoRedoRepeat.Add(InsItem, @TheUndoData, nil, nil, SizeOf(TAddObject), 0, 0);
  // Actually perform the operation
  dec(NumberOfObjects);
  CompleteRedraw;
  result:=nil;
end;

procedure TDrawingForm.DrawShape(TopLeft, BottomRight: TPoint; AMode: TPenMode; Final: boolean);
var
  TheUndoData : TAddObject;
begin
  if not Final then
  with Image.Canvas do
  begin
    Pen.Mode := AMode;
    Brush.Style := bsSolid;
    Pen.Color:=MyPenColor;
    Brush.Color:=MyBrushColor;
    case DrawingTool of
      dtLine:
        begin
          Image.Canvas.MoveTo(TopLeft.X, TopLeft.Y);
          Image.Canvas.LineTo(BottomRight.X, BottomRight.Y);
        end;
      dtRectangle: Image.Canvas.Rectangle(TopLeft.X, TopLeft.Y, BottomRight.X,
        BottomRight.Y);
      dtEllipse: Image.Canvas.Ellipse(Topleft.X, TopLeft.Y, BottomRight.X,
        BottomRight.Y);
      dtRoundRect: Image.Canvas.RoundRect(TopLeft.X, TopLeft.Y, BottomRight.X,
        BottomRight.Y, (TopLeft.X - BottomRight.X) div 2,
        (TopLeft.Y - BottomRight.Y) div 2);
    end;
  end;
  if Final then
  with Image.Canvas do
  begin
    case DrawingTool of
      dtLine:
        hlcUndoRedoRepeat.UndoBeginUpdate('Draw Line');
      dtRectangle:
        hlcUndoRedoRepeat.UndoBeginUpdate('Draw Rectangle');
      dtEllipse:
        hlcUndoRedoRepeat.UndoBeginUpdate('Draw Ellipse');
      dtRoundRect:
        hlcUndoRedoRepeat.UndoBeginUpdate('Draw RoundRect');
    end;
    if NumberOfObjects=MaxNumberOfObjects then
    begin
      NumberOfObjects:=0;
      hlcUndoRedoRepeat.Reset;
    end;

    TheUndoData.RTopLeft:=TopLeft;
    TheUndoData.RBottomRight:=BottomRight;
    TheUndoData.RDrawingTool:=DrawingTool;
    TheUndoData.RPenStyle:=MyPenStyle;
    TheUndoData.RBrushStyle:=MyBrushStyle;

    TheUndoData.RPenColor:=MyPenColor;
    TheUndoData.RBrushColor:=MyBrushColor;
    TheUndoData.RPenWidth:=Image.Canvas.Pen.Width;

    InsItem(@TheUndoData, nil, nil);
    hlcUndoRedoRepeat.UndoEndUpdate;
  end;
end;

procedure TDrawingForm.SetPenStyle(Sender: TObject);
begin
  with Image.Canvas.Pen do
  begin
    if Sender = ToolButtonSolidPen then Style := psSolid
    else if Sender = ToolButtonDashPen then Style := psDash
    else if Sender = ToolButtonDotPen then Style := psDot
    else if Sender = ToolButtonDashDotPen then Style := psDashDot
    else if Sender = ToolButtonDashDotDotPen then Style := psDashDotDot
    else if Sender = ToolButtonClearPen then Style := psClear;
    MyPenStyle:=Image.Canvas.Pen.Style;
  end;
end;

procedure TDrawingForm.PenSizeChange(Sender: TObject);
begin
  Image.Canvas.Pen.Width := PenWidth.Position;
end;

procedure TDrawingForm.FormCreate(Sender: TObject);
begin
  TempBitmap := TBitmap.Create;
  TempBitmap.Width := 200;
  TempBitmap.Height := 200;
  Image.Picture.Graphic := TempBitmap;
  NumberOfObjects:=0;
  hlcUndoRedoRepeat.Reset;
  Application.OnHint := ShowHint;
  MyPenColor:=PenPanel.Color;
  MyBrushColor:=BrushPanel.Color;
end;

procedure TDrawingForm.ShowHint(Sender: TObject);
begin
  if Length(Application.Hint) > 0 then
  begin
    StatusBar.SimplePanel := True;
    StatusBar.SimpleText := Application.Hint;
  end
  else StatusBar.SimplePanel := False;
end;

procedure TDrawingForm.SetBrushStyle(Sender: TObject);
begin
  with Image.Canvas.Brush do
  begin
    if Sender = ToolButtonSolidBrush then Style := bsSolid
    else if Sender = ToolButtonClearBrush then Style := bsClear
    else if Sender = ToolButtonHorizontalBrush then Style := bsHorizontal
    else if Sender = ToolButtonVerticalBrush then Style := bsVertical
    else if Sender = ToolButtonFDiagonalBrush then Style := bsFDiagonal
    else if Sender = ToolButtonBDiagonalBrush then Style := bsBDiagonal
    else if Sender = ToolButtonCrossBrush then Style := bsCross
    else if Sender = ToolButtonDiagCrossBrush then Style := bsDiagCross;
    MyBrushStyle:=Image.Canvas.Brush.Style;
  end;
end;

procedure TDrawingForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TDrawingForm.Open1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    case CanContinue of
    mrOk :
      begin
        CurrentFile := OpenDialog1.FileName;
        SaveStyles;
        Image.Picture.LoadFromFile(CurrentFile);
        Image.Picture.Bitmap.PixelFormat:=pf24bit;
        TempBitmap.Assign(Image.Picture.Bitmap);
        RestoreStyles;
        hlcUndoRedoRepeat.Reset;
        RestoreStyles;
        NumberOfObjects:=0;
      end;
    end;
  end;
end;

procedure TDrawingForm.Save1Click(Sender: TObject);
begin
  if CurrentFile <> EmptyStr then
    Image.Picture.SaveToFile(CurrentFile)
  else SaveAs1Click(Sender);
end;

procedure TDrawingForm.Saveas1Click(Sender: TObject);
begin
  if SaveDialog1.Execute then
  begin
    CurrentFile := SaveDialog1.FileName;
    Save1Click(Sender);
  end;
end;

function TDrawingForm.CanContinue:Word;
var
  TheResult : word;
begin
  TheResult:=mrOk;
  if hlcUndoRedoRepeat.CanUndo then
  begin
    if CurrentFile=EmptyStr then
    begin
      TheResult:=MessageDlg('Save changes to new drawing?', mtWarning, mbYesNoCancel, 0);
      case TheResult of
      mrYes :
        begin
          if SaveDialog1.Execute then
          begin
            CurrentFile := SaveDialog1.FileName;
            Image.Picture.SaveToFile(CurrentFile);
            TheResult:=mrOk;
          end
          else
          TheResult:=mrCancel;
        end;
      mrNo :
        begin
          TheResult:=mrOk;
        end;
      mrCancel :
        begin
        end;
      end;
    end
    else
    begin
      TheResult:=MessageDlg(Format('Save changes to %s?', [ExtractFileName(CurrentFile)]), mtWarning, mbYesNoCancel, 0);
      case TheResult of
      mrYes :
        begin
          Image.Picture.SaveToFile(CurrentFile);
          TheResult:=mrOk;
        end;
      mrNo :
        begin
          TheResult:=mrOk;
        end;
      mrCancel :
        begin
        end;
      end;
    end;
  end;
  Result:=TheResult;
end;

procedure TDrawingForm.New1Click(Sender: TObject);
var
  Bitmap: TBitmap;
begin
  with NewBMPForm do
  begin
    ActiveControl := WidthEdit;
    WidthEdit.Text := IntToStr(Image.Picture.Graphic.Width);
    HeightEdit.Text := IntToStr(Image.Picture.Graphic.Height);
    if ShowModal <> idCancel then
    begin
      case CanContinue of
      mrOk :
        begin
          Bitmap := TBitmap.Create;
          Bitmap.Width := StrToInt(WidthEdit.Text);
          Bitmap.Height := StrToInt(HeightEdit.Text);
          SaveStyles;
          Image.Picture.Graphic := Bitmap;
          TempBitmap.Assign(Image.Picture.Bitmap);
          RestoreStyles;
          CurrentFile := EmptyStr;
          hlcUndoRedoRepeat.Reset;
          RestoreStyles;
          NumberOfObjects:=0;
        end;
      end;
    end;
  end;
end;

procedure TDrawingForm.SaveStyles;
begin
  with Image.Canvas do
  begin
    BrushStyle := MyBrushStyle;
    PenStyle := MyPenStyle;
    PenWide := PenWidth.Position;
  end;
end;

procedure TDrawingForm.RestoreStyles;
begin
  with Image.Canvas do
  begin
    Brush.Style := BrushStyle;
    Pen.Style := PenStyle;
    Pen.Width := PenWide;
    Pen.Color:=MyPenColor;
    Brush.Color:=MyBrushColor;
  end;
  UpdateToolBarButtons;
  AdvancedReset.Enabled:=hlcUndoRedoRepeat.CanUndo or hlcUndoRedoRepeat.CanRedo or hlcUndoRedoRepeat.CanRepeat;
end;

procedure TDrawingForm.Print1Click(Sender: TObject);
begin
  ShowMessage('You could print your drawing here.');
end;

procedure TDrawingForm.EditUndoItemClick(Sender: TObject);
begin
  SaveStyles;
  hlcUndoRedoRepeat.Undo;
  RestoreStyles;
end;

procedure TDrawingForm.EditRedoItemClick(Sender: TObject);
begin
  SaveStyles;
  hlcUndoRedoRepeat.Redo;
  RestoreStyles;
end;

procedure TDrawingForm.EditRepeatItemClick(Sender: TObject);
begin
  SaveStyles;
  hlcUndoRedoRepeat.DoRepeat;
  RestoreStyles;
end;

procedure TDrawingForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if CanContinue<>mrOk
  then Action:=caNone;
end;

procedure TDrawingForm.About1Click(Sender: TObject);
begin
  with HlcAboutBox do
  begin
    if hlcUndoRedoRepeat.MemoryAllocationSize=0
    then UsedByComponent.Caption := '0 Bytes'
    else UsedByComponent.Caption := FormatFloat('#,###" Bytes"', hlcUndoRedoRepeat.MemoryAllocationSize);
    LabelUsed2.Caption := '25';
    ShowModal;
  end;
end;

procedure TDrawingForm.GroupUndo1Click(Sender: TObject);
begin
  GroupUndo1.Checked:=not GroupUndo1.Checked;
  hlcUndoRedoRepeat.AllowGroupUndo:=GroupUndo1.Checked;
end;

procedure TDrawingForm.AdvancedResetClick(Sender: TObject);
begin
  hlcUndoRedoRepeat.Reset;
  RestoreStyles;
end;

procedure TDrawingForm.ToolButtonPenClick(Sender: TObject);
begin
  ToolBarPen.Visible := ToolButtonPen.Down;
  Repaint;
  ToolBarPen.Repaint;
  ToolBarBrush.Repaint;
end;

procedure TDrawingForm.ToolButtonBrushClick(Sender: TObject);
begin
  ToolBarBrush.Visible := ToolButtonBrush.Down;
  Repaint;
  ToolBarPen.Repaint;
  ToolBarBrush.Repaint;
end;

procedure TDrawingForm.PenPanelClick(Sender: TObject);
begin
  ColorDialog1.Color := Image.Canvas.Pen.Color;
  if ColorDialog1.Execute then
  begin
    Image.Canvas.Pen.Color := ColorDialog1.Color;
    MyPenColor:=ColorDialog1.Color;
    PenPanel.Color:=ColorDialog1.Color;
  end;
end;

procedure TDrawingForm.BrushPanelClick(Sender: TObject);
begin
  ColorDialog1.Color := Image.Canvas.Brush.Color;
  if ColorDialog1.Execute then
  begin
    Image.Canvas.Brush.Color := ColorDialog1.Color;
    BrushPanel.Color:=ColorDialog1.Color;
    MyBrushColor:=ColorDialog1.Color;
  end;
end;

end.
