{*************************************************************************}
{                                                                         }
{  Listbox example to illustrate the UndoRedoRepeat Component             }
{                                                                         }
{  Example for Delphi 3 or higher                                         }
{                                                                         }
{  Copyright (c) 1997, 1999 High-Logic [Erwin Denissen]                   }
{                                                                         }
{  There are no known bugs at the moment, so please notify us when you    }
{  find one. Make sure you are using the latest version of this software. }
{  email: erwind@high-logic.com                                           }
{                                                                         }
{*************************************************************************}

unit ListBoxFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, Buttons, ExtCtrls, ComCtrls, ToolWin,
  HlcUndoRedoRepeat, ImgList;

type
  TListBoxForm = class(TForm)
    ListBox1: TListBox;
    ButtonClearAll: TButton;
    Edit1: TEdit;
    ButtonInsert: TButton;
    MainMenu1: TMainMenu;
    Edit2: TMenuItem;
    EditRepeatItem: TMenuItem;
    EditUndoItem: TMenuItem;
    EditRedoItem: TMenuItem;
    ListBox2: TListBox;
    ListBox3: TListBox;
    Label2: TLabel;
    Label3: TLabel;
    ButtonMultipleUndo: TButton;
    ButtonMultipleRedo: TButton;
    StatusBar: TStatusBar;
    Help1: TMenuItem;
    HelpAboutItem: TMenuItem;
    File1: TMenuItem;
    FileExitItem: TMenuItem;
    N1: TMenuItem;
    FilePrintItem: TMenuItem;
    N5: TMenuItem;
    FileSaveAsItem: TMenuItem;
    FileSaveItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileNewItem: TMenuItem;
    ToolBar: TToolBar;
    OpenButton: TToolButton;
    SaveButton: TToolButton;
    PrintButton: TToolButton;
    ToolButton5: TToolButton;
    ToolButtonUndo: TToolButton;
    CutButton: TToolButton;
    CopyButton: TToolButton;
    PasteButton: TToolButton;
    ToolButton1: TToolButton;
    ToolButtonRedo: TToolButton;
    ToolButtonRepeat: TToolButton;
    N3: TMenuItem;
    EditDeleteItem: TMenuItem;
    Bevel1: TBevel;
    Advanced1: TMenuItem;
    AdvancedReset: TMenuItem;
    GroupUndo1: TMenuItem;
    ToolbarImages: TImageList;
    Bevel2: TBevel;
    ButtonDelete: TButton;
    hlcUndoRedoRepeat: ThlcUndoRedoRepeat;
    procedure FormCreate(Sender: TObject);
    procedure ButtonClearAllClick(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure ButtonInsertClick(Sender: TObject);
    procedure EditUndoItemClick(Sender: TObject);
    procedure EditRedoItemClick(Sender: TObject);
    procedure EditRepeatItemClick(Sender: TObject);
    procedure ListBox2Click(Sender: TObject);
    procedure ListBox3Click(Sender: TObject);
    procedure ButtonMultipleUndoClick(Sender: TObject);
    procedure ButtonMultipleRedoClick(Sender: TObject);
    procedure HelpAboutItemClick(Sender: TObject);
    procedure Edit1KeyPress(Sender: TObject; var Key: Char);
    procedure EditDeleteItemClick(Sender: TObject);
    procedure FileExitItemClick(Sender: TObject);
    procedure GroupUndo1Click(Sender: TObject);
    procedure AdvancedResetClick(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateButtons;
    procedure ShowHint(Sender: TObject);
  public
    { Public declarations }
    function InsItem(TheData1, TheData2, TheData3: pointer):pointer;
    function DelItem(TheData1, TheData2, TheData3: pointer):pointer;
  end;

var
  ListBoxForm: TListBoxForm;

implementation

{$R *.DFM}

uses HlcAbout;

type

TInsItem =
  record
    index : integer;
    tekst : string[200];
  end;

TDelItem = integer;

procedure TListBoxForm.UpdateButtons;
begin
  ToolButtonUndo.Enabled:=EditUndoItem.Enabled;
  if ToolButtonUndo.Enabled
  then ToolButtonUndo.Hint:='Undo '+hlcUndoRedoRepeat.GetUndoName
  else ToolButtonUndo.Hint:='Can''t undo.';
  EditUndoItem.Hint:=ToolButtonUndo.Hint;
  ToolButtonRedo.Enabled:=EditRedoItem.Enabled;
  if ToolButtonRedo.Enabled
  then ToolButtonRedo.Hint:='Redo '+hlcUndoRedoRepeat.GetRedoName
  else ToolButtonRedo.Hint:='Can''t redo.';
  EditRedoItem.Hint:=ToolButtonRedo.Hint;
  ToolButtonRepeat.Enabled:=EditRepeatItem.Enabled;
  if ToolButtonRepeat.Enabled
  then ToolButtonRepeat.Hint:='Repeat '+hlcUndoRedoRepeat.GetRepeatName
  else ToolButtonRepeat.Hint:='Can''t repeat.';
  EditRepeatItem.Hint:=ToolButtonRepeat.Hint;
  EditDeleteItem.Enabled:=ListBox1.ItemIndex>=0;
  ButtonDelete.Enabled:=ListBox1.ItemIndex>=0;
  ButtonClearAll.Enabled:=ListBox1.Items.Count>0;
  ListBox2.Items:=hlcUndoRedoRepeat.GetUndoItems;
  ListBox3.Items:=hlcUndoRedoRepeat.GetRedoItems;
  ButtonMultipleUndo.Enabled:=ListBox2.ItemIndex>=0;
  ButtonMultipleRedo.Enabled:=ListBox3.ItemIndex>=0;
  if ToolButtonRepeat.Enabled
  then StatusBar.Panels[0].Text:='Repeat allowed.'
  else StatusBar.Panels[0].Text:='Repeat not available.';
  StatusBar.Panels[1].Text:=IntToStr(ListBox1.Items.Count)+' lines.';
  StatusBar.Panels[2].Text:=
                  IntToStr(ListBox2.Items.Count)+' possible undo''s and '
                 +IntToStr(ListBox3.Items.Count)+' possible redo''s.';
  AdvancedReset.Enabled:=hlcUndoRedoRepeat.CanUndo or hlcUndoRedoRepeat.CanRedo or hlcUndoRedoRepeat.CanRepeat;
end;

procedure TListBoxForm.FormCreate(Sender: TObject);
begin
  hlcUndoRedoRepeat.Reset;  // Updates the MenuItems Undo, Redo and Repeat.
  UpdateButtons;
  Application.OnHint := ShowHint;
end;

procedure TListBoxForm.ShowHint(Sender: TObject);
begin
  if Length(Application.Hint) > 0 then
  begin
    StatusBar.SimplePanel := True;
    StatusBar.SimpleText := Application.Hint;
  end
  else StatusBar.SimplePanel := False;
end;

function TListBoxForm.InsItem(TheData1, TheData2, TheData3: pointer):pointer;
var
  TheUndoData : TDelItem;
begin
  // Now put the opposite in de hlcUndoRedoRepeat object
  TheUndoData:=TInsItem(TheData1^).index;
  hlcUndoRedoRepeat.Add(DelItem, @TheUndoData, nil, nil, SizeOf(TDelItem), 0, 0);
  // Actually perform the operation
  ListBox1.Items.Insert(TInsItem(TheData1^).index,TInsItem(TheData1^).tekst);
  result:=nil;
end;

function TListBoxForm.DelItem(TheData1, TheData2, TheData3: pointer):pointer;
var
  TheUndoData : TInsItem;
begin
  // first because the variables are deleted when we do this afterwards
  // put the opposite in de hlcUndoRedoRepeat object
  TheUndoData.index:=TDelItem(TheData1^);
  TheUndoData.tekst:=ListBox1.Items.Strings[TDelItem(TheData1^)];
  hlcUndoRedoRepeat.Add(InsItem, @TheUndoData, nil, nil, SizeOf(TInsItem), 0, 0);
  // Actually perform the operation
  ListBox1.Items.Delete(TDelItem(TheData1^));
  result:=nil;
end;

procedure TListBoxForm.ButtonClearAllClick(Sender: TObject);
var
  TheData : TDelItem;
  t : integer;
begin
  if ListBox1.Items.Count>0 then
  begin
    hlcUndoRedoRepeat.UndoBeginUpdate('Clear the listbox');
    for t:=ListBox1.Items.Count downto 1 do
    begin
      TheData:=0;
      DelItem(@TheData, nil, nil);
    end;
    hlcUndoRedoRepeat.UndoEndUpdate;
    UpdateButtons;
  end;
end;

procedure TListBoxForm.ListBox1Click(Sender: TObject);
begin
  UpdateButtons;
end;

procedure TListBoxForm.ButtonInsertClick(Sender: TObject);
var
  TheData : TInsItem;
begin
  TheData.tekst:=Edit1.Text;
  if ListBox1.ItemIndex>=0
  then TheData.index:=ListBox1.ItemIndex
  else TheData.index:=0;
  hlcUndoRedoRepeat.UndoBeginUpdate('Insert');
  hlcUndoRedoRepeat.AddRepeat(InsItem, @TheData, nil, nil, SizeOf(TheData), 0, 0);
  InsItem(@TheData, nil, nil);
  hlcUndoRedoRepeat.UndoEndUpdate;
  UpdateButtons;
end;

procedure TListBoxForm.EditUndoItemClick(Sender: TObject);
begin
  hlcUndoRedoRepeat.Undo;
  UpdateButtons;
end;

procedure TListBoxForm.EditRedoItemClick(Sender: TObject);
begin
  hlcUndoRedoRepeat.Redo;
  UpdateButtons;
end;

procedure TListBoxForm.EditRepeatItemClick(Sender: TObject);
begin
  hlcUndoRedoRepeat.DoRepeat;
  UpdateButtons;
end;

procedure TListBoxForm.ListBox2Click(Sender: TObject);
begin
  ButtonMultipleUndo.Enabled:=ListBox2.ItemIndex>=0;
end;

procedure TListBoxForm.ListBox3Click(Sender: TObject);
begin
  ButtonMultipleRedo.Enabled:=ListBox3.ItemIndex>=0;
end;

procedure TListBoxForm.ButtonMultipleUndoClick(Sender: TObject);
var i : integer;
begin
  for i:=0 to ListBox2.ItemIndex do hlcUndoRedoRepeat.Undo;
  UpdateButtons;
end;

procedure TListBoxForm.ButtonMultipleRedoClick(Sender: TObject);
var i : integer;
begin
  for i:=0 to ListBox3.ItemIndex do hlcUndoRedoRepeat.Redo;
  UpdateButtons;
end;

procedure TListBoxForm.HelpAboutItemClick(Sender: TObject);
begin
  with HlcAboutBox do
  begin
    if hlcUndoRedoRepeat.MemoryAllocationSize=0
    then UsedByComponent.Caption := '0 Bytes'
    else UsedByComponent.Caption := FormatFloat('#,###" Bytes"', hlcUndoRedoRepeat.MemoryAllocationSize);
    LabelUsed2.Caption := '25';
    ShowModal;
  end;
end;

procedure TListBoxForm.Edit1KeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#13 then
  begin
    ButtonInsert.Click;
    Key:=#0;
  end;
end;

procedure TListBoxForm.EditDeleteItemClick(Sender: TObject);
var
  TheData : TDelItem;
begin
  TheData:=ListBox1.ItemIndex;
  hlcUndoRedoRepeat.UndoBeginUpdate('Delete');
  DelItem(@TheData, nil, nil);
  hlcUndoRedoRepeat.UndoEndUpdate;
  UpdateButtons;
end;

procedure TListBoxForm.FileExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TListBoxForm.GroupUndo1Click(Sender: TObject);
begin
  GroupUndo1.Checked:=not GroupUndo1.Checked;
  hlcUndoRedoRepeat.AllowGroupUndo:=GroupUndo1.Checked;
end;

procedure TListBoxForm.AdvancedResetClick(Sender: TObject);
begin
  hlcUndoRedoRepeat.Reset;
  UpdateButtons;
end;

end.
